/*****************************************************************************
 *------------------------ Utility  -----------------------------------------*
 *****************************************************************************/
#include <float.h>
#include <math.h>
#include "rsrtm20xx.h"

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/
#pragma warning( disable : 4152 )
ViStatus rsrtm20xx_CheckStatusCallback (ViSession io);

/*****************************************************************************
 *- Common Callback Declarations --------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsrtm20xx_InitAttributes
 * Purpose:  This function inits attributes to the desired values if needed.
 *****************************************************************************/
ViStatus rsrtm20xx_InitAttributes (ViSession instrSession)
{
    ViStatus    error           = VI_SUCCESS;
    ViInt32     p2value         = 0;
    void        *p2void;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE],
                revision[RS_MAX_MESSAGE_BUF_SIZE],
                manufacturer[RS_MAX_MESSAGE_BUF_SIZE],
                model[RS_MAX_MESSAGE_BUF_SIZE],
                tmp_model[RS_MAX_MESSAGE_BUF_SIZE],
                idQ[RS_MAX_MESSAGE_BUF_SIZE],
                options_list[RS_MAX_MESSAGE_BUF_SIZE];

    buffer[RS_MAX_MESSAGE_BUF_SIZE -1] = 0;

    /*- Misc -*/

    if (!Rs_Simulating(instrSession))
        {
        /* Instrument identification */
        viCheckErr (viQueryf (instrSession, "*IDN?\n", "%[^\r\n]", idQ));
        idQ[RS_MAX_MESSAGE_BUF_SIZE - 1] = 0;
        checkErr (Rs_SetAttribute (instrSession, "", RSRTM20XX_ATTR_ID_QUERY_RESPONSE, 0, idQ));
        
        /* Option(s) string */
        viCheckErr (viQueryf (instrSession, "*OPT?\n", "%[^\r\n]", options_list));
        options_list[RS_MAX_MESSAGE_BUF_SIZE - 1] = 0;
        }
    else /* Simulating */
        {
        /* Instrument identification */
        strncpy (idQ, RSRTM20XX_SIMULATION_ID_QUERY, RS_MAX_MESSAGE_BUF_SIZE - 1);
        checkErr (Rs_SetAttribute (instrSession, "", RSRTM20XX_ATTR_ID_QUERY_RESPONSE, 0, idQ));
        
        /* Option(s) string */
        strncpy (options_list, RSRTM20XX_SIMULATION_OPT_QUERY, RS_MAX_MESSAGE_BUF_SIZE - 1);
        }

    /*- Inherent Instrument Specific Attributes ----------------------------*/

    /*- User Options -*/

    /*
    RS_ATTR_RANGE_CHECK             ... Rs_SpecificDriverNew
    RS_ATTR_QUERY_INSTRUMENT_STATUS ... Rs_SpecificDriverNew
    RS_ATTR_CACHE                   ... Rs_SpecificDriverNew
    RS_ATTR_SIMULATE                ... Rs_SpecificDriverNew
    RS_ATTR_RECORD_COERCIONS        ... not supported
    RS_ATTR_INTERCHANGE_CHECK       ... not supported
    RS_ATTR_SPY                     ... not supported
    RS_ATTR_USE_SPECIFIC_SIMULATION ... not supported
    */

    /*- Class Driver Identification -*/

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_DESCRIPTION, 0, RSRTM20XX_CLASS_DRIVER_DESCRIPTION));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_PREFIX, 0, RSRTM20XX_CLASS_DRIVER_PREFIX));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_VENDOR, 0, RSRTM20XX_CLASS_DRIVER_VENDOR));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_REVISION, 0, RSRTM20XX_CLASS_DRIVER_REVISION));
    p2value = RSRTM20XX_CLASS_SPEC_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, &p2value));
    p2value = RSRTM20XX_CLASS_SPEC_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, &p2value));

    /*- Driver Identification -*/
                                                        
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 0, RSRTM20XX_SPECIFIC_DRIVER_DESCRIPTION));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_PREFIX, 0, RSRTM20XX_SPECIFIC_DRIVER_PREFIX));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_LOCATOR, 0, RSRTM20XX_SPECIFIC_DRIVER_LOCATOR));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_VENDOR, 0, RSRTM20XX_SPECIFIC_DRIVER_VENDOR));

    sprintf (buffer, 
             "Driver: %s %d.%d.%d, Compiler: %s %3.2f, "
             "Components: RSEngine %.2f, VISA-Spec %.2f",
             RSRTM20XX_SPECIFIC_DRIVER_PREFIX,
             RSRTM20XX_MAJOR_VERSION, RSRTM20XX_MINOR_VERSION, RSRTM20XX_MINOR_MINOR_VERSION,
             RS_COMPILER_NAME, RS_COMPILER_VER_NUM, 
             RS_ENGINE_MAJOR_VERSION + RS_ENGINE_MINOR_VERSION/10.0, 
             Rs_ConvertVISAVer(VI_SPEC_VERSION));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, buffer));
    p2value = RSRTM20XX_CLASS_SPEC_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, &p2value));
    p2value = RSRTM20XX_CLASS_SPEC_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, &p2value));

    /*- Driver Capabilities -*/
                                   
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SUPPORTED_INSTRUMENT_MODELS, 0, RSRTM20XX_SUPPORTED_INSTRUMENT_MODELS));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_GROUP_CAPABILITIES, 0, RSRTM20XX_GROUP_CAPABILITIES));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_FUNCTION_CAPABILITIES, 0, RSRTM20XX_FUNCTION_CAPABILITIES));
    /* RS_ATTR_CHANNEL_COUNT        ... Rs_BuildRepCapTable */

    /*- Driver Setup -*/

    /* RS_ATTR_DRIVER_SETUP         ... Rs_SpecificDriverNew */

    /*- Instrument Identification -*/

    checkErr (Rs_GetAttribute (instrSession, "", RSRTM20XX_ATTR_ID_QUERY_RESPONSE, 0, RSRTM20XX_IO_BUFFER_SIZE, idQ));
    sscanf (idQ, "%[^,], %[^,],%*[^,], %256[^\n]", manufacturer, tmp_model, revision); // added 2 spaces in the parsing string
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_INSTRUMENT_MANUFACTURER, 0, manufacturer));
    sscanf (tmp_model,"%[^-0-9]",model); // added number elimination
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_INSTRUMENT_MODEL, 0, model));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, revision));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_OPTIONS_LIST, 0, options_list));

    /*- Advanced Session Information -*/
                                  
    /*  
    RS_ATTR_IO_RESOURCE_DESCRIPTOR  ... rsrtm20xx_InitWithOptions
    RS_ATTR_LOGICAL_NAME            ... not supported
    */

    /*- Inherent Vendor Specific Attributes --------------------------------*/

    /*- Error Info -*/

    /*
    RS_ATTR_PRIMARY_ERROR           ... Rs_SetErrorInfo
    RS_ATTR_SECONDARY_ERROR         ... Rs_SetErrorInfo
    RS_ATTR_ERROR_ELABORATION       ... Rs_SetErrorInfo
    */

    /*- Session I/O -*/

    /*
    RS_ATTR_VISA_RM_SESSION         ... rsrtm20xx_RsInit
    RS_ATTR_IO_SESSION              ... Rs_UpdateIOSession
    RS_ATTR_IO_SESSION_TYPE         ... not supported
    */

    /*- Version Info -*/                     

    p2value = RSRTM20XX_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION, 0, &p2value));
    p2value = RSRTM20XX_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_VERSION, 0, &p2value));
    p2value = RSRTM20XX_MINOR_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_MINOR_VERSION, 0, &p2value));
    p2value = RSRTM20XX_CLASS_SPEC_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_MAJOR_VERSION, 0, &p2value));
    p2value = RSRTM20XX_CLASS_SPEC_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_MINOR_VERSION, 0, &p2value));
    p2value = RS_ENGINE_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_ENGINE_MAJOR_VERSION, 0, &p2value));
    p2value = RS_ENGINE_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_ENGINE_MINOR_VERSION, 0, &p2value));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_ENGINE_REVISION, 0, RS_ENGINE_REVISION));

    /*- Session Callbacks -*/                

    p2void = rsrtm20xx_WaitForOPCCallback;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_OPC_CALLBACK, 0, &p2void));
    p2void = rsrtm20xx_CheckStatusCallback;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CHECK_STATUS_CALLBACK, 0, &p2void));
    p2value = RSRTM20XX_OPC_TIMEOUT;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_OPC_TIMEOUT, 0, &p2value));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsrtm20xx_DefaultInstrSetup
 * Purpose:  This function sends a default setup to the instrument. The
 *           rsrtm20xx_reset function calls this function. The
 *           rsrtm20xx_RsInit function calls this function when the
 *           user passes VI_FALSE for the reset parameter. This function is
 *           useful for configuring settings that other instrument driver
 *           functions require.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsrtm20xx_DefaultInstrSetup (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    /* Set all the attributes to the default state. Do not update inherent attributes! */
    checkErr (Rs_ApplyAttributeDefaults (instrSession, VI_FALSE));

    /* Init attributes */
    checkErr( rsrtm20xx_InitAttributes (instrSession));

    if (!Rs_Simulating(instrSession))
        {

        viCheckErr( viPrintf (instrSession, "*CLS;*ESE %hu;*SRE 0;FORM:BORD LSBF\n", IEEE_488_2_ESE_MASK));
        }

Error:
    return error;
}

/*****************************************************************************
 * Function: rsrtm20xx_CheckStatus
 * Purpose:  This function checks the status of the instrument to detect
 *           whether the instrument has encountered an error. This function
 *           is called at the end of most exported driver functions. If the
 *           instrument reports an error, this function returns
 *           RS_ERROR_INSTRUMENT_SPECIFIC. The user can set the
 *           rsrtm20xx_ATTR_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE to disable this
 *           check and increase execution speed.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsrtm20xx_CheckStatus (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (Rs_QueryInstrStatus (instrSession) && !Rs_Simulating (instrSession))
        {
        checkErr( rsrtm20xx_CheckStatusCallback (instrSession));
        }
	
	if (error == VI_SUCCESS_MAX_CNT)
        error = VI_SUCCESS;
	
Error:
    return error;
}


ViStatus rsrtm20xx_CheckStatusTimed (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (Rs_QueryInstrStatus (instrSession) && !Rs_Simulating (instrSession))
        {
        checkErr( rsrtm20xx_CheckTimedStatusCallback (instrSession));
        }

Error:
    return error;
}

/*****************************************************************************
 * Function: rsrtm20xx_WaitForOPC
 * Purpose:  This function waits for the instrument to complete the
 *           execution of all pending operations. This function is a
 *           wrapper for the WaitForOPCCallback. It can be called from
 *           other instrument driver functions.
 *
 *           The maxTime parameter specifies the maximum time to wait for
 *           operation complete in milliseconds.
 *****************************************************************************/
ViStatus rsrtm20xx_WaitForOPC (ViSession instrSession, ViUInt32 maxTime)
{
    ViStatus    error = VI_SUCCESS;
    
    if (!Rs_Simulating(instrSession))
        {
        ViInt32  oldOPCTimeout;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSRTM20XX_ATTR_OPC_TIMEOUT, 0,
                                   (unsigned int) sizeof (oldOPCTimeout), &oldOPCTimeout));

        Rs_SetAttribute (instrSession, VI_NULL, RSRTM20XX_ATTR_OPC_TIMEOUT, 0, &maxTime);

        error = rsrtm20xx_WaitForOPCCallback (instrSession);

        Rs_SetAttribute (instrSession, VI_NULL, RSRTM20XX_ATTR_OPC_TIMEOUT, 0, &oldOPCTimeout);

        viCheckErr( error);
        }
Error:
    return error;
}
 
/*===========================================================================*/
/* Function: String Index                                                    */
/* Purpose:  Returns the index of string in string array                     */
/*===========================================================================*/
ViInt32 rsrtm20xx_StringIndex(ViString array[], ViChar buff[])
{
    ViInt32 index = 0;

    while (array[index]){
        if (!strcmp(buff, array[index]))
            break; 
        index++;    
    }        
    /* If found string is not found returns -1. */
    if (!array[index]) return -1;

    return index;
}

/*===========================================================================*/
/* Function: Read Binary Data Block                                          */
/* Purpose:  Reads the binary data from instrument. Allocates memory for data*/
/* WARNING:  You must deallocate buffer, if any pointer has been returned.   */
/*           This function does not work correctly, when you read bin data   */
/*           via serial intfc                                            */
/*===========================================================================*/
ViStatus rsrtm20xx_readBinaryDataBlock (ViSession instrSession,
                                  ViChar **outputBuffer, ViUInt32 *byteCount)
{
    ViStatus    error   = VI_SUCCESS;
    ViUInt16    intf_type, term_en;
    ViUInt32    ret_cnt;
    ViInt32    len;
    ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar      model [RSRTM20XX_IO_BUFFER_SIZE] = "";
    
    *outputBuffer = VI_NULL;
    
    /* Disable termination character */
    checkErr (viGetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, &term_en));
    checkErr (viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

    /* Disable termination character on serial */
    checkErr (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intf_type));
    if (intf_type == VI_INTF_ASRL)
    {
        checkErr (viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
    }

    /* Parse header */
    checkErr (viRead (instrSession, (ViBuf) buffer, 2, &ret_cnt));
    buffer[2] = '\0';
    sscanf (buffer, "#%ld", &len);
    checkErr (viRead (instrSession, (ViBuf) buffer, len, &ret_cnt));
    buffer[len] = '\0';
    sscanf (buffer, "%ld", &len);
    
    if (byteCount)
        *byteCount = len;

    checkAlloc (*outputBuffer = (ViChar*) malloc (len));
    /* Read useful data */
    checkErr (viRead (instrSession, (ViBuf) *outputBuffer, len, &ret_cnt));
    
    if (len != (ViInt32) ret_cnt)
        viCheckErr (RS_ERROR_UNEXPECTED_RESPONSE);

    viCheckErr (rsrtm20xx_GetAttributeViString (instrSession, "",
                                               RS_ATTR_INSTRUMENT_MODEL,
                                               RSRTM20XX_IO_BUFFER_SIZE, model));
    /* Flush garbage */
    viScanf (instrSession,"%*[^\r\n]");
    
Error:
    
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, term_en);

    if (outputBuffer && error)
        free (outputBuffer);
    
    return error;
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViStatus rsrtm20xx_invalidViInt32Range (ViSession instrSession, ViInt32 val, ViInt32 min, ViInt32 max)
{
    return (Rs_RangeChecking (instrSession) && (ViBoolean)((val < min || val > max) ? RS_ERROR_INVALID_PARAMETER : VI_SUCCESS));
}
/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViStatus rsrtm20xx_invalidViUInt32Range (ViSession instrSession, ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return (Rs_RangeChecking (instrSession) && (ViBoolean)((val < min || val > max) ? RS_ERROR_INVALID_PARAMETER : VI_SUCCESS));

}
/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViStatus rsrtm20xx_invalidViReal64Range (ViSession instrSession, ViReal64 val, ViReal64 min, ViReal64 max)
{
    return (Rs_RangeChecking (instrSession) && (ViBoolean)((val < min || val > max) ? RS_ERROR_INVALID_PARAMETER : VI_SUCCESS));
}
/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViStatus rsrtm20xx_invalidViBooleanRange (ViSession instrSession, ViBoolean val)
{
    return (Rs_RangeChecking (instrSession) && (ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? RS_ERROR_INVALID_PARAMETER : VI_SUCCESS));
}
/*===========================================================================*/
/* Function: Swap 32bit Integer                                              */
/* Purpose:  Byte order for binary transfers depends on MOTOROLA or INTEL    */
/*           formats.                                                        */
/*===========================================================================*/
void rsrtm20xx_SWAP_32(unsigned char *data, long points)
{
    /* MOTOROLA (byte order 0123) or INTEL (byte order 3210). */
    unsigned int    word = 0x1122;
    unsigned char   *p2word = (unsigned char*)&word;

    if (*p2word == 0x11) /* MOTOROLA format */
    {
       unsigned char    src[4];
       long             *p2data = (long*)data;
       int              idx;

       for (idx = 0; idx < points; idx++, p2data++)
          {
              *((long *)src) = *(p2data);
              ((unsigned char *)(p2data))[0] =  ((unsigned char*)(src))[3];
              ((unsigned char *)(p2data))[1] =  ((unsigned char*)(src))[2];
              ((unsigned char *)(p2data))[2] =  ((unsigned char*)(src))[1];
              ((unsigned char *)(p2data))[3] =  ((unsigned char*)(src))[0];
          }
    }

   return;
}

/*===========================================================================*/
/* Function: Swap 16bit Integer                                              */
/* Purpose:  Byte order for binary transfers depends on MOTOROLA or INTEL    */
/*           formats.                                                        */
/*===========================================================================*/
void rsrtm20xx_SWAP_16(unsigned char *data, long points)
{
    /* MOTOROLA (byte order 0123) or INTEL (byte order 3210). */
    unsigned short    word = 0x1122;
    unsigned char   *p2word = (unsigned char*)&word;

    if (*p2word == 0x11) /* MOTOROLA format */
    {
       unsigned char    src[2];
       short            *p2data = (short*)data;
       int              idx;

       for (idx = 0; idx < points; idx++, p2data++)
          {
              *((short *)src) = *(p2data);
              ((unsigned char *)(p2data))[0] =  ((unsigned char*)(src))[1];
              ((unsigned char *)(p2data))[1] =  ((unsigned char*)(src))[0];
          }
    }

   return;
}

/*===========================================================================*/
/* Function: Read Trace Data                                                 */
/* Purpose:  This function reads out trace data from the instrument.         */
/*           It checks if data are ASCII or binary                           */
/*===========================================================================*/
ViStatus rsrtm20xx_dataReadTrace (ViSession instrSession,
                                 ViString trace, 
                                 ViInt32 arrayLength, 
                                 /*@out@*/ViReal64 _VI_FAR traceData[],
                                 ViPInt32 noofPoints)
{
    ViChar      *extBuf=VI_NULL;
    ViReal32    *temp;
    ViUInt32    i = 0;
    ViStatus    error = VI_SUCCESS;
    ViUInt16    intfc = VI_INTF_GPIB;
    ViUInt32    count=0;
    ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar      *p2buf=VI_NULL;
    ViUInt32    ret_cnt;
    ViInt32     len;
    
    checkErr (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));
/*Don't put parameter range checking here! 
All parameters has to be checked in high level function */

	viCheckErr (viPrintf (instrSession, "%s\n", trace));
    //Check first character            
	checkErr (viRead (instrSession, (ViBuf) buffer, 1, &ret_cnt)); 
    if (buffer[0]=='#')
        /*Binary data*/
    {
		ViUInt32 old_tmo = 0;
		viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &old_tmo);
        /* Disable termination character */
        checkErr (viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
    
         /* Disable termination character on serial */
        if (intfc == VI_INTF_ASRL)
        {
            checkErr (viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
        }
        /* Parse header */
        checkErr (viRead (instrSession, (ViBuf) buffer, 1, &ret_cnt));
        buffer[1] = '\0';
        sscanf (buffer, "%ld", &len);
        checkErr (viRead (instrSession, (ViBuf) buffer, len, &ret_cnt));
        buffer[len] = '\0';
        sscanf (buffer, "%ld", &len);
        
        checkAlloc (extBuf = (ViChar*) malloc (len));
        /* Read useful data */
        checkErr (viRead (instrSession, (ViBuf) extBuf, len, &ret_cnt));
    
        if (len != (ViInt32) ret_cnt)
            viCheckErr (RS_ERROR_UNEXPECTED_RESPONSE);

        /* Flush garbage */
		viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 0);
        viScanf (instrSession,"%*[^\r\n]");
		viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, old_tmo);
        /* Convert binary data into array of ViReal64 types. */
        count = (ViInt32)(ret_cnt/4);
        /* Binary data byte order (MOTOROLA or INTEL). */
        rsrtm20xx_SWAP_32((ViPBuf)extBuf, count);
        /* Pointer to data */
        temp = (ViReal32 *)extBuf;
        if (noofPoints) *noofPoints=count;    
    
        count=(count<(ViUInt32)arrayLength)?count:(ViUInt32)arrayLength;
    
        for (i = 0; i < count; i++)
            if (i<(ViUInt32)arrayLength)
            *traceData++ = (ViReal64)*temp++;
    }
    else
        /*ASCII data*/
    {
        //read rest of first value
        p2buf=buffer;
        while((p2buf[0]!=',')&&(p2buf[0]!='\r')&&(p2buf[0]!='\n')){
            p2buf++;
            checkErr (viRead (instrSession, (ViBuf) p2buf, 1, &ret_cnt)); 
        }
        traceData[0] = atof (p2buf); 
        //read all values
        viCheckErr (Rs_ReadDataUnknownLength(instrSession, &extBuf, &ret_cnt)); 

        p2buf = strtok(extBuf, ",");
        i=1;
        while (p2buf) 
        {
            if (i < (ViUInt32)arrayLength)
            {
                traceData[i] = atof (p2buf);
            }   
            p2buf = strtok(NULL, ",");  
            i++;
        }
    
        if (noofPoints) *noofPoints=i-1; 
    
    }

Error:
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
    switch (intfc)
    {
        case VI_INTF_ASRL:
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
        break;    
    }    
    if (extBuf) free(extBuf);
	if (error > 0) error = VI_SUCCESS;
    return error;
}

/*===========================================================================*/
/* Function: Set OPC Timeout                                                 */
/* Purpose:  Sets new OPC timeout                                            */
/*===========================================================================*/
ViStatus rsrtm20xx_SetOPCTimeout(ViSession   instrSession, ViInt32 timeout)
{
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_SetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0, &timeout));  
    
Error:    
    return error;
}

/*===========================================================================*/
/* Function: Get OPC Timeout                                                 */
/* Purpose:  Gets new OPC timeout                                            */
/*===========================================================================*/
ViStatus rsrtm20xx_GetOPCTimeout(ViSession   instrSession, ViInt32 *timeout)
{
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0, (unsigned int) sizeof(timeout), timeout));  
    
Error:    
    return error;
}

/// HIFN This function returns repeated capabilty name according to attribute and repeated capability id
/// HIRET Returns regular status code
/// HIPAR instrSession/Session.
/// HIPAR attributeID/Attribute ID
/// HIPAR repCapNameId/Repeated capability ID. For instance: Channel
/// HIPAR value/Numeric value of repeated capability (index in array of names)
/// HIPAR bufferSize/Size of repCapName array
/// HIPAR repCapName/String representing repeated capability name
ViStatus rsrtm20xx_GetAttributeRepCapName (ViSession instrSession,
                                ViUInt32 attributeID,
                                ViString repCapNameId,
                                ViInt32 value,
                                ViInt32 bufferSize,
                                ViChar _VI_FAR repCapName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   *p2buf = NULL;
    ViChar   names [RS_MAX_MESSAGE_BUF_SIZE] = "";
    int      i = 0;
    
    checkErr (rsrtm20xx_GetAttributeRepeatedCapabilityIdNames (instrSession,
                                                             attributeID,
                                                             repCapNameId,
                                                             RS_MAX_MESSAGE_BUF_SIZE,
                                                             names));
    
    p2buf = strtok (names, ",");
    while (p2buf)
    {
        if (value == i++)
        {
            strncpy (repCapName, p2buf, bufferSize);
            break;
        }
        p2buf = strtok (NULL, ",");
    }
    if (p2buf == NULL)
        error = RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME;
Error:
    return error;
}

/*****************************************************************************
 * Function: rsrtm20xx_CheckTimedStatusCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rsrtm20xx_CheckTimedStatusCallback (ViSession instrSession)
 {
    ViUInt32    ret_cnt;
    ViStatus    error   = VI_SUCCESS;
    ViInt16     esr     = 0;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE];
	ViUInt16 	intfc = VI_INTF_GPIB;
	
    //viCheckErr (viQueryf (instrSession, "*ESR?\n", "%hd", &esr));
    viCheckErr(viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));
	if (Rs_QueryInstrStatus (instrSession) && !Rs_Simulating (instrSession))
        {
			if (intfc == VI_INTF_TCPIP) {	
				Rs_Delay (4);
			} else {
				Rs_Delay (0.05);
			}
        }
    viCheckErr (viWrite (instrSession, (ViBuf) "*STB?\n", 6, NULL));
    if (Rs_QueryInstrStatus (instrSession) && !Rs_Simulating (instrSession))
        {
        	Rs_Delay (0.1);
        }
    viCheckErr (viRead (instrSession, (ViPBuf) buffer, RS_MAX_MESSAGE_BUF_SIZE, &ret_cnt));
	if(error==1073676293) error=0;
	buffer[ret_cnt] = '\0';
	//------------------ fix for slow instrument response  -----------------------
	if(strcmp(buffer,"\n")==0){
		Rs_Delay (1);
		viCheckErr (viRead (instrSession, (ViPBuf) buffer, RS_MAX_MESSAGE_BUF_SIZE, &ret_cnt));
		buffer[ret_cnt] = '\0';
	}
	//------------------- end fix for slow instrument response  ------------------
    esr = (ViInt16) atol (buffer);

    if ((esr & IEEE_488_2_ERROR_MASK) != 0)
    {
        viCheckErr (RS_ERROR_INSTR_SPECIFIC);
    }

Error:
	if (error==1073676293) error=0;
    return error;
 }

ViBoolean rsrtm20xx_CheckOption (ViSession instrSession, ViString option)
{
	ViChar   buffer [RSRTM20XX_IO_BUFFER_SIZE];
    RsSessionPropertiesPtr sessionProperties = Rs_ViSession (instrSession);

    if (sessionProperties->optionChecking)
	{
		if (rsrtm20xx_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM20XX_IO_BUFFER_SIZE, buffer) != VI_SUCCESS)
			return VI_FALSE;
		else
    		return (strstr(buffer, option) != NULL);
	}
	return VI_TRUE;
}

//This function replaces a string.
ViChar *rsrtm20xx_ReplaceString(ViChar *st, ViChar *orig, ViChar *repl) 
{
	static char buffer[4096];
	char *ch;
	ch = strstr(st, orig);
    if (!ch)
		return st;
	strncpy(buffer, st, ch-st);  
	buffer[ch-st] = 0;
	sprintf(buffer+(ch-st), "%s%s", repl, ch+strlen(orig));
	return buffer;
}
