/****************************************************************************
 *
 *  Rohde & Schwarz RTM Oscilloscope instrument driver include file
 *
 ****************************************************************************/

#ifndef __RSRTM20XX_HEADER
#define __RSRTM20XX_HEADER

#include "rsidr_core.h"
#include "rsrtm20xx_attributes.h"
#include "rsrtm20xx_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/
#define RSRTM20XX_SIMULATION_ID_QUERY      "Rohde&Schwarz,RTM20xx,123456/789,1.23"  /* Default definition of ID Query Response for simulation */
#define RSRTM20XX_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSRTM20XX_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSRTM20XX_IO_BUFFER_SIZE           1024L   /* I/O buffer size */
#define RSRTM20XX_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */
#define IEEE_488_2_SRE_MASK                0x24   /* 488.2 Event Status Register (ESR) Bits */
#define IEEE_488_2_ERROR_MASK              0x4    /* 488.2 STB Bits */   
#define IEEE_488_2_ESE_MASK                0x3D    /* 488.2 Event Status Register (ESE) Bits */
#define RSRTM20XX_NAN                 9.91e+37

/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSRTM20XX_CLASS_DRIVER_DESCRIPTION           ""
#define RSRTM20XX_CLASS_DRIVER_PREFIX                ""
#define RSRTM20XX_CLASS_DRIVER_VENDOR                ""
#define RSRTM20XX_CLASS_DRIVER_REVISION              ""
#define RSRTM20XX_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSRTM20XX_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSRTM20XX_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSRTM20XX_SPECIFIC_DRIVER_PREFIX             "RSRTM20XX"
#define RSRTM20XX_SPECIFIC_DRIVER_LOCATOR            ""
#define RSRTM20XX_SPECIFIC_DRIVER_VENDOR             ""
#define RSRTM20XX_MAJOR_VERSION                      1L    /* Instrument driver major version          */
#define RSRTM20XX_MINOR_VERSION                      5L    /* Instrument driver minor version          */
#define RSRTM20XX_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */          

#define RSRTM20XX_SUPPORTED_INSTRUMENT_MODELS        ""    /* Instrument driver supported model(s)     */
#define RSRTM20XX_GROUP_CAPABILITIES                 ""
#define RSRTM20XX_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSRTM20XX_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSRTM20XX_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSRTM20XX_ERROR_MARKER_NOT_ENABLED     (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSRTM20XX_ERROR_NOT_DELTA_MARKER       (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSRTM20XX_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSRTM20XX_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSRTM20XX_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSRTM20XX_ERRMSG_MARKER_NOT_ENABLED    "Marker Not Enabled"
#define RSRTM20XX_ERRMSG_NOT_DELTA_MARKER      "Not Delta Marker"
#define RSRTM20XX_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSRTM20XX_ERROR_CODES_AND_MSGS \
    {RSRTM20XX_WARN_MEASURE_UNCALIBRATED,  RSRTM20XX_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSRTM20XX_WARN_OVER_RANGE,            RSRTM20XX_WARNMSG_OVER_RANGE},\
    {RSRTM20XX_ERROR_MARKER_NOT_ENABLED,   RSRTM20XX_ERRMSG_MARKER_NOT_ENABLED},\
    {RSRTM20XX_ERROR_NOT_DELTA_MARKER,     RSRTM20XX_ERRMSG_NOT_DELTA_MARKER},\
    {RSRTM20XX_ERROR_MAX_TIME_EXCEEDED,    RSRTM20XX_ERRMSG_MAX_TIME_EXCEEDED}


/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSRTM20XX_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSRTM20XX_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSRTM20XX_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSRTM20XX_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSRTM20XX_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSRTM20XX_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSRTM20XX_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSRTM20XX_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSRTM20XX_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSRTM20XX_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSRTM20XX_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSRTM20XX_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSRTM20XX_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSRTM20XX_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSRTM20XX_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSRTM20XX_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSRTM20XX_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSRTM20XX_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSRTM20XX_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSRTM20XX_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSRTM20XX_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSRTM20XX_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSRTM20XX_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSRTM20XX_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSRTM20XX_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSRTM20XX_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSRTM20XX_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSRTM20XX_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSRTM20XX_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSRTM20XX_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSRTM20XX_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME

/* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

//#define RSRTM20XX_ATTR_ID_QUERY_RESPONSE                                     (RS_SPECIFIC_PUBLIC_ATTR_BASE + 1L)                     /* ViString (Read Only)              */

/* BEGIN GENERATE */
#define RSRTM20XX_ATTR_FILTER_FREQUENCY                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 613
#define RSRTM20XX_ATTR_INTERPOLATION                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3
#define RSRTM20XX_ATTR_NUM_AVERAGES                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4
#define RSRTM20XX_ATTR_AVERAGING_STATUS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1428
#define RSRTM20XX_ATTR_NUMBER_OF_WAVEFORMS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 614
#define RSRTM20XX_ATTR_SAMPLE_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5
#define RSRTM20XX_ATTR_WAVEFORM_RATE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 469
#define RSRTM20XX_ATTR_HORZ_TIME_PER_RECORD                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7
#define RSRTM20XX_ATTR_HORZ_SCALE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9
#define RSRTM20XX_ATTR_ACQUSITION_START_TIME                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10
#define RSRTM20XX_ATTR_ACQUISITION_TIME                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 12
#define RSRTM20XX_ATTR_HORZ_ADC_SAMPLE_RATE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 14
#define RSRTM20XX_ATTR_HORZ_SAMPLE_RATE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 610
#define RSRTM20XX_ATTR_AUTOSCALE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 15
#define RSRTM20XX_ATTR_HORZ_DIVISIONS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 16
#define RSRTM20XX_ATTR_HORZ_REFERENCE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 17
#define RSRTM20XX_ATTR_ROLL_MODE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 612
#define RSRTM20XX_ATTR_TIMESTAMPS_FILENAME                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1571
#define RSRTM20XX_ATTR_TIMESTAMPS_SAVE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1572
#define RSRTM20XX_ATTR_ACQUISITION_TYPE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1023
#define RSRTM20XX_ATTR_WAVEFORM_ACQUISITION_TYPE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2
#define RSRTM20XX_ATTR_HORZ_RECORD_LENGTH                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 667
#define RSRTM20XX_ATTR_GET_HORZ_RECORD_LENGTH                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1198
#define RSRTM20XX_ATTR_PROBE_TYPE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 313
#define RSRTM20XX_ATTR_PROBE_NAME                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 314
#define RSRTM20XX_ATTR_PROBE_INPUT_IMPEDANCE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 315
#define RSRTM20XX_ATTR_PROBE_INPUT_CAPACITANCE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 316
#define RSRTM20XX_ATTR_PROBE_ATTENUATION                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 23
#define RSRTM20XX_ATTR_PROBE_BANDWIDTH                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 24
#define RSRTM20XX_ATTR_PROBE_DC_OFFSET                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 471
#define RSRTM20XX_ATTR_PROBE_ADDITIONAL_OFFSET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 472
#define RSRTM20XX_ATTR_PROBE_VOLTMETER_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 473
#define RSRTM20XX_ATTR_PROBE_ATTENUATION_UNIT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 645
#define RSRTM20XX_ATTR_PROBE_ATTENUATION_MANUAL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 646
#define RSRTM20XX_ATTR_PROBE_MICRO_BUTTON_ACTION                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 319
#define RSRTM20XX_ATTR_PROBE_SOFTWARE_VERSION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 320
#define RSRTM20XX_ATTR_PROBE_PRODUCTION_DATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 321
#define RSRTM20XX_ATTR_PROBE_PART_NUMBER                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 322
#define RSRTM20XX_ATTR_PROBE_BUILD_NUMBER                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 474
#define RSRTM20XX_ATTR_PROBE_SERIAL_NUMBER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 323
#define RSRTM20XX_ATTR_LABEL_ENABLED                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 687
#define RSRTM20XX_ATTR_LABEL_NAME                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 688
#define RSRTM20XX_ATTR_BUS_LABEL_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1573
#define RSRTM20XX_ATTR_BUS_LABEL_NAME                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1574
#define RSRTM20XX_ATTR_CHANNEL_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 25
#define RSRTM20XX_ATTR_VERTICAL_RANGE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 26
#define RSRTM20XX_ATTR_VERTICAL_OFFSET                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 27
#define RSRTM20XX_ATTR_VERTICAL_COUPLING                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 28
#define RSRTM20XX_ATTR_VERTICAL_SCALE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 29
#define RSRTM20XX_ATTR_CHANNEL_OFFSET                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 30
#define RSRTM20XX_ATTR_CHANNEL_BANDWIDTH_LIMIT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 31
#define RSRTM20XX_ATTR_CHANNEL_POLARITY                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 615
#define RSRTM20XX_ATTR_CHANNEL_OVERLOAD                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 32
#define RSRTM20XX_ATTR_CHANNEL_SKEW                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 33
#define RSRTM20XX_ATTR_CHANNEL_THRESHOLD                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 34
#define RSRTM20XX_ATTR_CHANNEL_THRESHOLD_HYSTERESIS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1320
#define RSRTM20XX_ATTR_PROTOCOL_FIND_LEVEL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 693
#define RSRTM20XX_ATTR_CHANNEL_ALL_OFF                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1575
#define RSRTM20XX_ATTR_CHANNEL_ALL_ON                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1576
#define RSRTM20XX_ATTR_DIGITAL_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 822
#define RSRTM20XX_ATTR_DIGITAL_DESKEW                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1162
#define RSRTM20XX_ATTR_DIGITAL_THRESHOLD                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 823
#define RSRTM20XX_ATTR_DIGITAL_TECHNOLOGY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 824
#define RSRTM20XX_ATTR_DIGITAL_HYSTERESIS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 825
#define RSRTM20XX_ATTR_DIGITAL_APPLY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 826
#define RSRTM20XX_ATTR_DIGITAL_LABEL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 827
#define RSRTM20XX_ATTR_DIGITAL_LABEL_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 828
#define RSRTM20XX_ATTR_DIGITAL_VERTICAL_CHANNEL_SIZE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 829
#define RSRTM20XX_ATTR_DIGITAL_VERTICAL_POSITION                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 830
#define RSRTM20XX_ATTR_DIGITAL_CURRENT_STATUS_MAXIMUM                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1558
#define RSRTM20XX_ATTR_DIGITAL_CURRENT_STATUS_MINIMUM                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1559
#define RSRTM20XX_ATTR_TRIGGER_TYPE_A                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 37
#define RSRTM20XX_ATTR_TRIGGER_SOURCE_A                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 38
#define RSRTM20XX_ATTR_TRIGGER_SOURCE_B                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 768
#define RSRTM20XX_ATTR_EXTERNAL_TRIGGER_COUPLING                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 616
#define RSRTM20XX_ATTR_EXTERNAL_TRIGGER_TERMINATION                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1577
#define RSRTM20XX_ATTR_EXTERNAL_TRIGGER_OVERLOAD                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1578
#define RSRTM20XX_ATTR_TRIGGER_HOLDOFF_TIME                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 689
#define RSRTM20XX_ATTR_TRIGGER_HOLDOFF_MODE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 831
#define RSRTM20XX_ATTR_TRIGGER_HYSTERESIS_A                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 493
#define RSRTM20XX_ATTR_TRIGGER_HYSTERESIS_A_LEVEL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1579
#define RSRTM20XX_ATTR_TRIGGER_HYSTERESIS_B                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 769
#define RSRTM20XX_ATTR_TRIGGER_HYSTERESIS_B_LEVEL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1580
#define RSRTM20XX_ATTR_TRIGGER_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 482
#define RSRTM20XX_ATTR_TRIGGER_MODIFIER_A                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 40
#define RSRTM20XX_ATTR_TRIGGER_MODIFIER_B                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 770
#define RSRTM20XX_ATTR_TRIGGER_EVENT_COUNT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 489
#define RSRTM20XX_ATTR_TRIGGER_LEVEL_AUTO_FIND_A                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 41
#define RSRTM20XX_ATTR_TRIGGER_LEVEL_AUTO_FIND_B                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 771
#define RSRTM20XX_ATTR_TRIGGER_LEVEL_LOWER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 832
#define RSRTM20XX_ATTR_TRIGGER_LEVEL_UPPER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 833
#define RSRTM20XX_ATTR_TRIGGER_DELAY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 488
#define RSRTM20XX_ATTR_TRIGGER_LEVEL_A                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 39
#define RSRTM20XX_ATTR_TRIGGER_LEVEL_B                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 772
#define RSRTM20XX_ATTR_TRIGGER_SLOPE_A                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 45
#define RSRTM20XX_ATTR_TRIGGER_SLOPE_B                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 773
#define RSRTM20XX_ATTR_TRIGGER_COUPLING                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 490
#define RSRTM20XX_ATTR_TRIGGER_FILTER_LOW                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 491
#define RSRTM20XX_ATTR_TRIGGER_FILTER_HIGH                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 774
#define RSRTM20XX_ATTR_TV_TRIGGER_EVENT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 51
#define RSRTM20XX_ATTR_TV_TRIGGER_LINE_NUMBER                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 52
#define RSRTM20XX_ATTR_TV_TRIGGER_POLARITY                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 53
#define RSRTM20XX_ATTR_TV_STANDARD                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 477
#define RSRTM20XX_ATTR_WIDTH                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 481
#define RSRTM20XX_ATTR_WIDTH_DELTA                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 480
#define RSRTM20XX_ATTR_WIDTH_POLARITY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 478
#define RSRTM20XX_ATTR_WIDTH_CONDITION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 479
#define RSRTM20XX_ATTR_TRIGGER_PATTERN_SOURCE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 494
#define RSRTM20XX_ATTR_TRIGGER_PATTERN_FUNCTION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 495
#define RSRTM20XX_ATTR_TRIGGER_PATTERN_POINT_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 496
#define RSRTM20XX_ATTR_TRIGGER_PATTERN_MODE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 834
#define RSRTM20XX_ATTR_TRIGGER_PATTERN_WIDTH                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 835
#define RSRTM20XX_ATTR_TRIGGER_PATTERN_WIDTH_DELTA                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 836
#define RSRTM20XX_ATTR_TRIGGER_PATTERN_WIDTH_CONDITION                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 837
#define RSRTM20XX_ATTR_TRIGGER_TIME_POLARITY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 838
#define RSRTM20XX_ATTR_TRIGGER_TIME_CONDITION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 839
#define RSRTM20XX_ATTR_TRIGGER_TIME                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 840
#define RSRTM20XX_ATTR_TRIGGER_TIME_DELTA                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 841
#define RSRTM20XX_ATTR_TRIGGER_OUT_MODE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 842
#define RSRTM20XX_ATTR_TRIGGER_OUT_POLARITY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 843
#define RSRTM20XX_ATTR_TRIGGER_OUT_PULSE_WIDTH                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 844
#define RSRTM20XX_ATTR_RUNT_TRIGGER_POLARITY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1336
#define RSRTM20XX_ATTR_RUNT_TRIGGER_LOWER_LEVEL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1337
#define RSRTM20XX_ATTR_RUNT_TRIGGER_UPPER_LEVEL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1338
#define RSRTM20XX_ATTR_PROTOCOL_I2C_TRIGGER_ACCESS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 529
#define RSRTM20XX_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_TYPE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 531
#define RSRTM20XX_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 530
#define RSRTM20XX_ATTR_PROTOCOL_I2C_TRIGGER_MODE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 532
#define RSRTM20XX_ATTR_PROTOCOL_I2C_TRIGGER_CONDITION_LENGTH                RS_SPECIFIC_PUBLIC_ATTR_BASE + 534
#define RSRTM20XX_ATTR_PROTOCOL_I2C_TRIGGER_BIT_PATTERN                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 533
#define RSRTM20XX_ATTR_PROTOCOL_I2C_TRIGGER_FIRST_BYTE_OFFSET               RS_SPECIFIC_PUBLIC_ATTR_BASE + 535
#define RSRTM20XX_ATTR_PROTOCOL_SPI_TRIGGER_MODE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 537
#define RSRTM20XX_ATTR_PROTOCOL_SPI_TRIGGER_OFFSET                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 388
#define RSRTM20XX_ATTR_PROTOCOL_SPI_BIT_PATTERN_LENGTH                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 539
#define RSRTM20XX_ATTR_PROTOCOL_SPI_BIT_PATTERN                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 536
#define RSRTM20XX_ATTR_PROTOCOL_UART_TRIGGER_MODE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 540
#define RSRTM20XX_ATTR_PROTOCOL_UART_TRIGGER_DATA_PATTERN                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 541
#define RSRTM20XX_ATTR_PROTOCOL_UART_TRIGGER_DATA_POSITION_INDEX_MIN        RS_SPECIFIC_PUBLIC_ATTR_BASE + 542
#define RSRTM20XX_ATTR_PROTOCOL_UART_TRIGGER_CONDITION_LENGTH               RS_SPECIFIC_PUBLIC_ATTR_BASE + 543
#define RSRTM20XX_ATTR_PROTOCOL_LIN_TRIGGER_MODE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 705
#define RSRTM20XX_ATTR_PROTOCOL_LIN_TRIGGER_CONDITION_ID                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 706
#define RSRTM20XX_ATTR_PROTOCOL_LIN_TRIGGER_CONDITION_DATA                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 805
#define RSRTM20XX_ATTR_PROTOCOL_LIN_TRIGGER_ID_PATTERN                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 707
#define RSRTM20XX_ATTR_PROTOCOL_LIN_TRIGGER_DATA_LENGTH                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 708
#define RSRTM20XX_ATTR_PROTOCOL_LIN_TRIGGER_DATA_PATTERN                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 709
#define RSRTM20XX_ATTR_PROTOCOL_LIN_TRIGGER_ERROR_CHECKSUM                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 710
#define RSRTM20XX_ATTR_PROTOCOL_LIN_TRIGGER_ERROR_PARITY                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 806
#define RSRTM20XX_ATTR_PROTOCOL_LIN_TRIGGER_ERROR_SYNC                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 807
#define RSRTM20XX_ATTR_PROTOCOL_CAN_TRIGGER_MODE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 723
#define RSRTM20XX_ATTR_PROTOCOL_CAN_TRIGGER_FRAME_TYPE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 724
#define RSRTM20XX_ATTR_PROTOCOL_CAN_TRIGGER_CONDITION_ID                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 725
#define RSRTM20XX_ATTR_PROTOCOL_CAN_TRIGGER_CONDITION_DATA                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 811
#define RSRTM20XX_ATTR_PROTOCOL_CAN_TRIGGER_ID_TYPE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 726
#define RSRTM20XX_ATTR_PROTOCOL_CAN_TRIGGER_ID_PATTERN                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 727
#define RSRTM20XX_ATTR_PROTOCOL_CAN_TRIGGER_DATA_LENGTH                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 728
#define RSRTM20XX_ATTR_PROTOCOL_CAN_TRIGGER_DATA_PATTERN                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 729
#define RSRTM20XX_ATTR_PROTOCOL_CAN_TRIGGER_ERROR_CRC                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 730
#define RSRTM20XX_ATTR_PROTOCOL_CAN_TRIGGER_ERROR_STUFFING                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 812
#define RSRTM20XX_ATTR_PROTOCOL_CAN_TRIGGER_ERROR_FORM                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 813
#define RSRTM20XX_ATTR_PROTOCOL_CAN_TRIGGER_ERROR_ACK                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 814
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRIGGER_TYPE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1143
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRIGGER_WINDOW_LENGTH                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1144
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRIGGER_WORD_SELECT_SLOPE             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1145
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRIGGER_RIGHT_CONDITION               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1163
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRIGGER_RIGHT_MINIMUM                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1164
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRIGGER_RIGHT_MAXIMUM                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1165
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRIGGER_LEFT_CONDITION                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1166
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRIGGER_LEFT_MINIMUM                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1167
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRIGGER_LEFT_MAXIMUM                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1168
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRIGGER_COMBINATION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1169
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRIGGER_CHANNEL_CONDITION             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1170
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRIGGER_CHANNEL_MINIMUM               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1171
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRIGGER_CHANNEL_MAXIMUM               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1172
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_TYPE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1230
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_SYNCHRONIZATION                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1231
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_WORD                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1232
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_TRANSMISSION_TYPE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1424
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_SYNCHRONIZATION_ERROR                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1233
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_TIMEOUT_ERROR                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1234
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_PARITY_ERROR                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1235
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_MANCHESTER_ERROR                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1236
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_COMMAND_TYPE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1237
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_COMMAND_DATA_DIRECTION                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1238
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_RT_ADDRESS_CONDITION                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1239
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_RT_ADDRESS_MINIMUM                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1240
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_RT_ADDRESS_MAXIMUM                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1241
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_SUB_ADDRESS_CONDITION                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1242
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_SUB_ADDRESS_MINIMUM                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1243
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_SUB_ADDRESS_MAXIMUM                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1244
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_SUB_ADDRESS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1245
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_WORD_CONDITION                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1246
#define RSRTM20XX_ATTR_MILSTD_WORD_ADDRESS_MINIMUM                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1247
#define RSRTM20XX_ATTR_MILSTD_WORD_ADDRESS_MAXIMUM                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1248
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_COMMAND_MODE_CODE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1249
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_COMMAND_MODE_VALUE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1250
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_RECEIVED_BIT_STATUS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1251
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_BUSY_BIT_STATUS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1252
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_BUS_CONTROL_BIT_STATUS                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1253
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_INSTRUMENTATION_BIT_STATUS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1254
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_MESSAGE_ERROR_BIT_STATUS              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1255
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_SERVICE_REQUEST_BIT_STATUS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1256
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_SUBSYSTEM_BIT_STATUS                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1257
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_TERMINAL_BIT_STATUS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1258
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_DATA_CONDITION                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1259
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_DATA_MINIMUM                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1260
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_DATA_MAXIMUM                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1261
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_DATA_OFFSET_CONDITION                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1262
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_DATA_OFFSET                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1263
#define RSRTM20XX_ATTR_MILSTD_TRIGGER_DATA_OFFSET_WORDS                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1264
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_TYPE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1389
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_WORD_TYPE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1390
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_ERROR_CODING_ERROR         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1367
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_ERROR_PARITY_ERROR         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1368
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_ERROR_GAP_ERROR            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1391
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_LABEL_DATA_SDI             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1359
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_LABEL_DATA_SSM             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1360
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_LABEL_DATA_FORMAT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1392
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_LABEL_PATTERN_CONDITION    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1361
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_LABEL_PATTERN_MIN          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1362
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_LABEL_PATTERN_MAX          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1363
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_DATA_PATTERN_CONDITION     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1393
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_DATA_PATTERN_MIN           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1365
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_DATA_PATTERN_MAX           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1366
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_DATA_OFFSET                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1394
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_DATA_SIZE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1395
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_TIMING_CONDITION           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1364
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_TIMING_MIN                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1396
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_TIMING_MAX                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1397
#define RSRTM20XX_ATTR_RUN_SINGLE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 130
#define RSRTM20XX_ATTR_RUN_SINGLE_WAIT_FOR_OPC                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1563
#define RSRTM20XX_ATTR_RUN_CONTINUOUS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 131
#define RSRTM20XX_ATTR_STOP                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 132
#define RSRTM20XX_ATTR_SEND_SW_TRIGGER                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 134
#define RSRTM20XX_ATTR_WAVEFORM_SAMPLES                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 500
#define RSRTM20XX_ATTR_WAVEFORM_CONVERSION_X_START                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 744
#define RSRTM20XX_ATTR_WAVEFORM_CONVERSION_X_INCREMENT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 745
#define RSRTM20XX_ATTR_WAVEFORM_CONVERSION_Y_START                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 746
#define RSRTM20XX_ATTR_WAVEFORM_CONVERSION_Y_INCREMENT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 747
#define RSRTM20XX_ATTR_WAVEFORM_CONVERSION_Y_RESOLUTION                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 748
#define RSRTM20XX_ATTR_ENVELOPE_WAVEFORM_DATA_POINTS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1581
#define RSRTM20XX_ATTR_ENVELOPE_WAVEFORM_CONVERSION_X_START                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 749
#define RSRTM20XX_ATTR_ENVELOPE_WAVEFORM_CONVERSION_X_INCREMENT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 750
#define RSRTM20XX_ATTR_ENVELOPE_WAVEFORM_CONVERSION_Y_START                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 751
#define RSRTM20XX_ATTR_ENVELOPE_WAVEFORM_CONVERSION_Y_INCREMENT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 752
#define RSRTM20XX_ATTR_ENVELOPE_WAVEFORM_CONVERSION_Y_RESOLUTION            RS_SPECIFIC_PUBLIC_ATTR_BASE + 753
#define RSRTM20XX_ATTR_MATH_WAVEFORM_CONVERSION_X_START                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 754
#define RSRTM20XX_ATTR_MATH_WAVEFORM_CONVERSION_X_INCREMENT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 755
#define RSRTM20XX_ATTR_MATH_WAVEFORM_CONVERSION_Y_START                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 756
#define RSRTM20XX_ATTR_MATH_WAVEFORM_CONVERSION_Y_INCREMENT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 757
#define RSRTM20XX_ATTR_MATH_WAVEFORM_CONVERSION_Y_RESOLUTION                RS_SPECIFIC_PUBLIC_ATTR_BASE + 758
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_CONVERSION_X_START                RS_SPECIFIC_PUBLIC_ATTR_BASE + 759
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_CONVERSION_X_INCREMENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 760
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_CONVERSION_Y_START                RS_SPECIFIC_PUBLIC_ATTR_BASE + 761
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_CONVERSION_Y_INCREMENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 762
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_CONVERSION_Y_RESOLUTION           RS_SPECIFIC_PUBLIC_ATTR_BASE + 763
#define RSRTM20XX_ATTR_MASK_DATA_CONVERSION_X_START                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1331
#define RSRTM20XX_ATTR_MASK_DATA_CONVERSION_X_INCREMENT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1332
#define RSRTM20XX_ATTR_MASK_DATA_CONVERSION_Y_START                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1333
#define RSRTM20XX_ATTR_MASK_DATA_CONVERSION_Y_INCREMENT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1334
#define RSRTM20XX_ATTR_MASK_DATA_CONVERSION_Y_RESOLUTION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1335
#define RSRTM20XX_ATTR_CURSOR_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 173
#define RSRTM20XX_ATTR_ALL_CURSORS_OFF                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 174
#define RSRTM20XX_ATTR_CURSOR_MEASUREMENT_TYPE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 175
#define RSRTM20XX_ATTR_CURSOR_SOURCE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 176
#define RSRTM20XX_ATTR_CURSOR_TRACKING                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 177
#define RSRTM20XX_ATTR_CURSOR_X_POSITION_COUPLING                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 179
#define RSRTM20XX_ATTR_CURSOR_Y_POSITION_COUPLING                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 181
#define RSRTM20XX_ATTR_CURSOR_ADJUSTMENT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 507
#define RSRTM20XX_ATTR_CURSOR_X_RATIO_MEASUREMENT_UNIT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 187
#define RSRTM20XX_ATTR_CURSOR_Y_RATIO_MEASUREMENT_UNIT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 189
#define RSRTM20XX_ATTR_CURSOR_X_RATIO_MEASUREMENT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 188
#define RSRTM20XX_ATTR_CURSOR_Y_RATIO_MEASUREMENT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 190
#define RSRTM20XX_ATTR_CURSOR_DISTANCE_HORIZONTAL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 183
#define RSRTM20XX_ATTR_CURSOR_DISTANCE_VERTICAL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 779
#define RSRTM20XX_ATTR_CURSOR_INVERSE_DISTANCE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 184
#define RSRTM20XX_ATTR_CURSOR_VOLTAGE_INVERSE_DISTANCE_HORIZONTAL           RS_SPECIFIC_PUBLIC_ATTR_BASE + 503
#define RSRTM20XX_ATTR_CURSOR_VOLTAGE_INVERSE_DISTANCE_VERTICAL             RS_SPECIFIC_PUBLIC_ATTR_BASE + 780
#define RSRTM20XX_ATTR_CURSOR_POSITION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 506
#define RSRTM20XX_ATTR_CURSOR_AUTOSET                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 647
#define RSRTM20XX_ATTR_CURSOR_RESET                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 648
#define RSRTM20XX_ATTR_CURSOR_PEAK_NEXT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 649
#define RSRTM20XX_ATTR_CURSOR_PEAK_PREVIOUS                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 650
#define RSRTM20XX_ATTR_CURSOR_X1_POSITION                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 178
#define RSRTM20XX_ATTR_CURSOR_X2_POSITION                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 781
#define RSRTM20XX_ATTR_CURSOR_X3_POSITION                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 782
#define RSRTM20XX_ATTR_CURSOR_Y1_POSITION                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 180
#define RSRTM20XX_ATTR_CURSOR_Y2_POSITION                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 783
#define RSRTM20XX_ATTR_CURSOR_Y3_POSITION                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 784
#define RSRTM20XX_ATTR_MEAS_RELATIVE_MODE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 619
#define RSRTM20XX_ATTR_MEAS_RELATIVE_LOWER_LEVEL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 620
#define RSRTM20XX_ATTR_MEAS_RELATIVE_UPPER_LEVEL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 621
#define RSRTM20XX_ATTR_MEAS_RELATIVE_MIDDLE_LEVEL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 622
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_FREQUENCY                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 979
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_PERIOD                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 980
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_PEAK                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 981
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_MAXIMUM_PEAK                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 982
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_MINIMUM_PEAK                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 983
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_POSITIVE_PULSES              RS_SPECIFIC_PUBLIC_ATTR_BASE + 984
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_NEGATIVE_PULSES              RS_SPECIFIC_PUBLIC_ATTR_BASE + 985
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_RISING_EDGES                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 986
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_FALLING_EDGES                RS_SPECIFIC_PUBLIC_ATTR_BASE + 987
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_HIGH_LEVEL                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 988
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_LOW_LEVEL                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 989
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_AMPLITUDE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 990
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_MEAN                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 991
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_RMS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 992
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_RISE_TIME                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 993
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_FALLING_TIME                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 994
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_POSITIVE_CYCLE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1005
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_NEGATIVE_CYCLE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1006
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_WIDTH_POSITIVE_PULSE         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1007
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_WIDTH_NEGATIVE_PULSE         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1008
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_MEAN_VALUE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1009
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_RMS_VALUE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1010
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_STANDARD_DEVIATIONS          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1011
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_STANDARD_DEVIATION_VALUE     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1012
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_TRIGGER_FREQUENCY            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1013
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_TRIGGER_PERIOD               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1014
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_DELAY                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1015
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_PHASE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1016
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_BURST_WIDTH                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1017
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_POSITIVE_OVERSHOOT           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1018
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_NEGATIVE_OVERSHOOT           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1019
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_B_TRIGGER_FREQUENCY          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1020
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_B_TRIGGER_PERIOD             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1021
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_AVERAGE_VALUE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 508
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_STANDARD_DEVIATION           RS_SPECIFIC_PUBLIC_ATTR_BASE + 775
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_NEGATIVE_PEAK                RS_SPECIFIC_PUBLIC_ATTR_BASE + 776
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_POSITIVE_PEAK                RS_SPECIFIC_PUBLIC_ATTR_BASE + 777
#define RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_WAVEFORM_COUNT               RS_SPECIFIC_PUBLIC_ATTR_BASE + 778
#define RSRTM20XX_ATTR_AMPT_MAIN_MEASUREMENT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 147
#define RSRTM20XX_ATTR_MEASUREMENT_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 136
#define RSRTM20XX_ATTR_MEASUREMENT_CATEGORY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 138
#define RSRTM20XX_ATTR_QUICK_MEASUREMENTS_ENABLE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 139
#define RSRTM20XX_ATTR_QUICK_MEASUREMENTS_DISABLE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 140
#define RSRTM20XX_ATTR_QUICK_MEASUREMENTS_ENABLE_WITH_STATUS_BIT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 668
#define RSRTM20XX_ATTR_AUTOMATIC_MEASUREMENT_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 651
#define RSRTM20XX_ATTR_AUTOMATIC_MEASUREMENT_NUMBER_OF_WAVEFORMS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 652
#define RSRTM20XX_ATTR_RESET_AUTOMATIC_MEASUREMENT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 653
#define RSRTM20XX_ATTR_AUTOMATIC_MEASUREMENT_NUMBER_OF_MEASURED_WAVEFORMS   RS_SPECIFIC_PUBLIC_ATTR_BASE + 666
#define RSRTM20XX_ATTR_AUTOMATIC_MEASUREMENT_RESULT_AVERAGE_VALUE           RS_SPECIFIC_PUBLIC_ATTR_BASE + 654
#define RSRTM20XX_ATTR_AUTOMATIC_MEASUREMENT_RESULT_STANDARD_DEVIATION      RS_SPECIFIC_PUBLIC_ATTR_BASE + 785
#define RSRTM20XX_ATTR_AUTOMATIC_MEASUREMENT_RESULT_NEGATIVE_PEAK           RS_SPECIFIC_PUBLIC_ATTR_BASE + 786
#define RSRTM20XX_ATTR_AUTOMATIC_MEASUREMENT_RESULT_POSITIVE_PEAK           RS_SPECIFIC_PUBLIC_ATTR_BASE + 787
#define RSRTM20XX_ATTR_AUTOMATIC_MEASUREMENT_RESULT_INDEX                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 764
#define RSRTM20XX_ATTR_AUTOMATIC_MEASUREMENT_SINGLE_STATISTICAL_VALUE       RS_SPECIFIC_PUBLIC_ATTR_BASE + 765
#define RSRTM20XX_ATTR_EXPORT_AUTOMATIC_MEASUREMENTS_NAME                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1582
#define RSRTM20XX_ATTR_EXPORT_AUTOMATIC_MEASUREMENT_SAVE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1583
#define RSRTM20XX_ATTR_EXPORT_AUTOMATIC_MEASUREMENTS_ALL_NAME               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1584
#define RSRTM20XX_ATTR_EXPORT_AUTOMATIC_MEASUREMENT_ALL_SAVE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1585
#define RSRTM20XX_ATTR_MATH_WAVEFORM_HEADER                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 337
#define RSRTM20XX_ATTR_MATH_WAVEFORM_DATA_POINTS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1586
#define RSRTM20XX_ATTR_MATH_WAVEFORM_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 235
#define RSRTM20XX_ATTR_MATH_WAVEFORM_POSITION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 623
#define RSRTM20XX_ATTR_MATH_VERTICAL_SCALE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 624
#define RSRTM20XX_ATTR_MATH_ACQUISITION_TYPE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 236
#define RSRTM20XX_ATTR_MATH_EXPRESSION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 237
#define RSRTM20XX_ATTR_FFT_CENTER_FREQUENCY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 241
#define RSRTM20XX_ATTR_FFT_FREQUENCY_SPAN                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 242
#define RSRTM20XX_ATTR_FFT_FULL_SPAN                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 243
#define RSRTM20XX_ATTR_FFT_START_FREQUENCY                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 244
#define RSRTM20XX_ATTR_FFT_STOP_FREQUENCY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 245
#define RSRTM20XX_ATTR_FFT_AVERAGING                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 509
#define RSRTM20XX_ATTR_FFT_AVERAGING_STATUS                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1560
#define RSRTM20XX_ATTR_FFT_WINDOW_TYPE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 246
#define RSRTM20XX_ATTR_FFT_RESOLUTION_BANDWIDTH                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 254
#define RSRTM20XX_ATTR_FFT_RESOLUTION_BANDWIDTH_COUPLING_ENABLED            RS_SPECIFIC_PUBLIC_ATTR_BASE + 255
#define RSRTM20XX_ATTR_FFT_RESOLUTION_BANDWIDTH_RATIO                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 256
#define RSRTM20XX_ATTR_FFT_EFFECTIVE_RESOLUTION_BANDWIDTH                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 257
#define RSRTM20XX_ATTR_FFT_SCALING_UNIT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 625
#define RSRTM20XX_ATTR_FFT_SAMPLE_RATE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 655
#define RSRTM20XX_ATTR_FFT_YT_WINDOW_POSITION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 669
#define RSRTM20XX_ATTR_FFT_YT_WINDOW_WIDTH                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 670
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_HEADER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 511
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_DATA_POINTS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1587
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_SOURCE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 270
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_UPDATE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 276
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_DISPLAY_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 271
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_LOAD_FROM_FILE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 512
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_LOAD_CONFIGURATION                RS_SPECIFIC_PUBLIC_ATTR_BASE + 513
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_SAVE_TO_FILE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 514
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_CATALOG                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 515
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_VERTICAL_SCALE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 281
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_VERTICAL_POSITION                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 282
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_HORIZONTAL_SCALE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 516
#define RSRTM20XX_ATTR_REFERENCE_WAVEFORM_HORIZONTAL_POSITION               RS_SPECIFIC_PUBLIC_ATTR_BASE + 611
#define RSRTM20XX_ATTR_MASK_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 517
#define RSRTM20XX_ATTR_MASK_SOURCE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 525
#define RSRTM20XX_ATTR_MASK_TEST                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 526
#define RSRTM20XX_ATTR_MASK_COPY_CHANNEL                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 518
#define RSRTM20XX_ATTR_MASK_HIT_COUNT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 522
#define RSRTM20XX_ATTR_MASK_COUNT                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 521
#define RSRTM20XX_ATTR_MASK_STORE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 524
#define RSRTM20XX_ATTR_MASK_LOAD                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 523
#define RSRTM20XX_ATTR_MASK_TEST_ACTION_BEEP                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 671
#define RSRTM20XX_ATTR_MASK_TEST_ACTION_STOP                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 788
#define RSRTM20XX_ATTR_MASK_TEST_ACTION_SCREENSHOT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 789
#define RSRTM20XX_ATTR_MASK_TEST_ACTION_PRINT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 790
#define RSRTM20XX_ATTR_MASK_TEST_ACTION_SAVES_WAVEFORM                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 791
#define RSRTM20XX_ATTR_MASK_TEST_ACTION_PULSE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1173
#define RSRTM20XX_ATTR_MASK_TEST_VIOLATION_COUNT_BEEP                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 672
#define RSRTM20XX_ATTR_MASK_TEST_VIOLATION_COUNT_STOP                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 792
#define RSRTM20XX_ATTR_MASK_TEST_VIOLATION_COUNT_SCREENSHOT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 793
#define RSRTM20XX_ATTR_MASK_TEST_VIOLATION_COUNT_PRINT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 794
#define RSRTM20XX_ATTR_MASK_TEST_VIOLATION_COUNT_SAVES_WAVEFORM             RS_SPECIFIC_PUBLIC_ATTR_BASE + 795
#define RSRTM20XX_ATTR_MASK_TEST_VIOLATION_COUNT_PULSE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1174
#define RSRTM20XX_ATTR_MASK_TEST_VIOLATION_PULSE_LENGTH                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1175
#define RSRTM20XX_ATTR_MASK_TEST_VIOLATION_PULSE_POLARITY                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1176
#define RSRTM20XX_ATTR_MASK_RESET_TEST_COUNTER                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 673
#define RSRTM20XX_ATTR_MASK_VERTICAL_OFFSET                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 519
#define RSRTM20XX_ATTR_MASK_VERTICAL_SCALING                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 520
#define RSRTM20XX_ATTR_MASK_VERTICAL_WIDTH                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 528
#define RSRTM20XX_ATTR_MASK_HORIZONTAL_WIDTH                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 527
#define RSRTM20XX_ATTR_SEARCH_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 626
#define RSRTM20XX_ATTR_SEARCH_CONDITION                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 627
#define RSRTM20XX_ATTR_SEARCH_SOURCE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 628
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_EDGE_SLOPE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 629
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_EDGE_LEVEL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 630
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_EDGE_HYSTERSIS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 631
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_WIDTH_LEVEL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 632
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_WIDTH_HYSTERSIS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 633
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_WIDTH_WIDTH                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 634
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_WIDTH_DELTA                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 635
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_WIDTH_POLARITY                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 636
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_WIDTH_RANGE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 637
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_PEAK_POLARITY                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 638
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_PEAK_LIMIT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 639
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_RISE_TIME_POLARITY                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 656
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_RISE_TIME_LEVEL_LOWER                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 657
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_RISE_TIME_LEVEL_UPPER                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 796
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_RISE_TIME_RANGE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 658
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_RISE_TIME_REFERENCE_TIME              RS_SPECIFIC_PUBLIC_ATTR_BASE + 659
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_RISE_TIME_DELTA_TIME                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 660
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_RUNT_POLARITY                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 661
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_RUNT_LEVEL_LOWER                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 662
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_RUNT_LEVEL_UPPER                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 797
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_RUNT_RANGE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 663
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_RUNT_WIDTH                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 664
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_RUNT_DELTA_WIDTH                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 665
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_D2C_CLOCK_SOURCE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 674
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_D2C_CLOCK_LEVEL                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 675
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_D2C_DATA_LEVEL                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 798
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_D2C_CLOCK_HYSTERESIS                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 676
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_D2C_DATA_HYSTERESIS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 799
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_D2C_POLARITY                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 677
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_D2C_SETUP_TIME                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 678
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_D2C_HOLD_TIME                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 679
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_PATTERN                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 680
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_PATTERN_COMBINATION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 681
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_PATTERN_COMPARISON                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 682
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_PATTERN_WIDTH                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 683
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_PATTERN_WIDTH_DELTA                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 684
#define RSRTM20XX_ATTR_SEARCH_GATE_MODE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 690
#define RSRTM20XX_ATTR_SEARCH_GATE_START                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 691
#define RSRTM20XX_ATTR_SEARCH_GATE_STOP                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 692
#define RSRTM20XX_ATTR_SEARCH_SHOW_RESULT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 640
#define RSRTM20XX_ATTR_SEARCH_NUMBER_OF_RESULTS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 641
#define RSRTM20XX_ATTR_SEARCH_MAXIMUM_NUMBER_OF_RESULTS                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1425
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_PATTERN_LEVEL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 685
#define RSRTM20XX_ATTR_SEARCH_TRIGGER_PATTERN_HYSTERESIS                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 686
#define RSRTM20XX_ATTR_PROTOCOL_LIN_SEARCH_EVENT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 712
#define RSRTM20XX_ATTR_PROTOCOL_LIN_SEARCH_FRAME_SETUP                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 713
#define RSRTM20XX_ATTR_PROTOCOL_LIN_SEARCH_CONDITION_ID                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 714
#define RSRTM20XX_ATTR_PROTOCOL_LIN_SEARCH_CONDITION_DATA                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 808
#define RSRTM20XX_ATTR_PROTOCOL_LIN_SEARCH_ID_PATTERN                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 715
#define RSRTM20XX_ATTR_PROTOCOL_LIN_SEARCH_DATA_LENGTH                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 716
#define RSRTM20XX_ATTR_PROTOCOL_LIN_SEARCH_DATA_PATTERN                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 717
#define RSRTM20XX_ATTR_PROTOCOL_LIN_SEARCH_ERROR_CHECKSUM                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 718
#define RSRTM20XX_ATTR_PROTOCOL_LIN_SEARCH_ERROR_PARITY                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 809
#define RSRTM20XX_ATTR_PROTOCOL_LIN_SEARCH_ERROR_SYNC                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 810
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SEARCH_EVENT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 732
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SEARCH_FRAME_SETUP                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 733
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SEARCH_FRAME_TYPE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 734
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SEARCH_ID_TYPE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 735
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SEARCH_CONDITION_ID                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 736
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SEARCH_CONDITION_DATA                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 815
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SEARCH_ID_PATTERN                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 737
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SEARCH_DATA_LENGTH                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 738
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SEARCH_DATA_PATTERN                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 739
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SEARCH_ERROR_CRC                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 740
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SEARCH_ERROR_STUFFING                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 816
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SEARCH_ERROR_FORM                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 817
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SEARCH_ERROR_ACK                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 818
#define RSRTM20XX_ATTR_MILSTD_SEARCH_CONDITION                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1290
#define RSRTM20XX_ATTR_MILSTD_SEARCH_WORD_START                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1426
#define RSRTM20XX_ATTR_MILSTD_SEARCH_ERROR                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1291
#define RSRTM20XX_ATTR_MILSTD_SEARCH_TRANSMISSION_TYPE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1427
#define RSRTM20XX_ATTR_MILSTD_SEARCH_COMMAND_DATA_DIRECTION                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1265
#define RSRTM20XX_ATTR_MILSTD_SEARCH_RT_ADDRESS_CONDITION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1266
#define RSRTM20XX_ATTR_MILSTD_SEARCH_RT_ADDRESS_MINIMUM                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1267
#define RSRTM20XX_ATTR_MILSTD_SEARCH_RT_ADDRESS_MAXIMUM                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1268
#define RSRTM20XX_ATTR_MILSTD_SEARCH_SUB_ADDRESS_CONDITION                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1269
#define RSRTM20XX_ATTR_MILSTD_SEARCH_SUB_ADDRESS_MINIMUM                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1270
#define RSRTM20XX_ATTR_MILSTD_SEARCH_SUB_ADDRESS_MAXIMUM                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1271
#define RSRTM20XX_ATTR_MILSTD_SEARCH_SUB_ADDRESS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1272
#define RSRTM20XX_ATTR_MILSTD_SEARCH_WORD_CONDITION                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1273
#define RSRTM20XX_ATTR_MILSTD_SEARCH_WORD_MINIMUM                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1274
#define RSRTM20XX_ATTR_MILSTD_SEARCH_WORD_MAXIMUM                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1275
#define RSRTM20XX_ATTR_MILSTD_SEARCH_COMMAND_MODE_CODE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1276
#define RSRTM20XX_ATTR_MILSTD_SEARCH_RECEIVED_BIT_STATUS                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1277
#define RSRTM20XX_ATTR_MILSTD_SEARCH_BUSY_BIT_STATUS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1278
#define RSRTM20XX_ATTR_MILSTD_SEARCH_BUS_CONTROL_BIT_STATUS                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1279
#define RSRTM20XX_ATTR_MILSTD_SEARCH_INSTRUMENTATION_BIT_STATUS             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1280
#define RSRTM20XX_ATTR_MILSTD_SEARCHMESSAGE_ERROR_BIT_STATUS                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1281
#define RSRTM20XX_ATTR_MILSTD_SEARCH_SERVICE_REQUEST_BIT_STATUS             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1282
#define RSRTM20XX_ATTR_MILSTD_SEARCH_SUBSYSTEM_BIT_STATUS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1283
#define RSRTM20XX_ATTR_MILSTD_SEARCH_TERMINAL_BIT_STATUS                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1284
#define RSRTM20XX_ATTR_MILSTD_SEARCH_DATA_CONDITION                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1285
#define RSRTM20XX_ATTR_MILSTD_SEARCH_DATA_MINIMUM                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1286
#define RSRTM20XX_ATTR_MILSTD_SEARCH_DATA_MAXIMUM                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1287
#define RSRTM20XX_ATTR_MILSTD_SEARCH_DATA_OFFSET                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1288
#define RSRTM20XX_ATTR_MILSTD_SEARCH_DATA_OFFSET_WORDS                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1289
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SEARCH_TYPE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1398
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SEARCH_WORD_TYPE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1399
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SEARCH_ERROR                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1400
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SEARCH_LABEL_PATTERN_CONDITION     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1401
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SEARCH_LABEL_PATTERN_MIN           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1402
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SEARCH_LABEL_PATTERN_MAX           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1403
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SEARCH_DATA_PATTERN_CONDITION      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1404
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SEARCH_DATA_PATTERN_MIN            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1405
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SEARCH_DATA_PATTERN_MAX            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1406
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SEARCH_DATA_OFFSET                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1407
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SEARCH_DATA_SIZE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1408
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SEARCH_DATA_FORMAT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1409
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SEARCH_SDI                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1410
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SEARCH_SSM                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1411
#define RSRTM20XX_ATTR_PROTOCOL_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 577
#define RSRTM20XX_ATTR_PROTOCOL_TYPE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 578
#define RSRTM20XX_ATTR_BUS_SAVE_DECODED_DATA                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 695
#define RSRTM20XX_ATTR_PROTOCOL_CONTENT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 867
#define RSRTM20XX_ATTR_PROTOCOL_DISPLAY_FORMAT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 579
#define RSRTM20XX_ATTR_PROTOCOL_DISPLAY_HEIGHT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 580
#define RSRTM20XX_ATTR_PROTOCOL_DISPLAY_LINES                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 581
#define RSRTM20XX_ATTR_PROTOCOL_DISPLAY_VERTICAL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 582
#define RSRTM20XX_ATTR_PROTOCOL_DISPLAY_RESULTS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 868
#define RSRTM20XX_ATTR_PROTOCOL_I2C_CLOCK_SOURCE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 350
#define RSRTM20XX_ATTR_PROTOCOL_I2C_DATA_SOURCE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 800
#define RSRTM20XX_ATTR_PROTOCOL_I2C_FRAME_COUNT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 696
#define RSRTM20XX_ATTR_PROTOCOL_I2C_SELECTED_FRAME                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 869
#define RSRTM20XX_ATTR_PROTOCOL_I2C_FRAME_DATA                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 870
#define RSRTM20XX_ATTR_PROTOCOL_I2C_FRAME_STATUS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 871
#define RSRTM20XX_ATTR_PROTOCOL_I2C_FRAME_START                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 872
#define RSRTM20XX_ATTR_PROTOCOL_I2C_FRAME_END                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 873
#define RSRTM20XX_ATTR_PROTOCOL_I2C_FRAME_TRANSFER_DIRECTION                RS_SPECIFIC_PUBLIC_ATTR_BASE + 874
#define RSRTM20XX_ATTR_PROTOCOL_I2C_FRAME_ADDRESS_ACKNOWLEDGE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 875
#define RSRTM20XX_ATTR_PROTOCOL_I2C_FRAME_ADDRESS_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 876
#define RSRTM20XX_ATTR_PROTOCOL_I2C_FRAME_ADDRESS_START_ACK_BIT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 877
#define RSRTM20XX_ATTR_PROTOCOL_I2C_FRAME_ADDRESS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 878
#define RSRTM20XX_ATTR_PROTOCOL_I2C_FRAME_ADDRESS_WITHOUT_BIT               RS_SPECIFIC_PUBLIC_ATTR_BASE + 879
#define RSRTM20XX_ATTR_PROTOCOL_I2C_FRAME_ADDRESS_LENGTH                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 880
#define RSRTM20XX_ATTR_PROTOCOL_I2C_FRAME_ADDRESS_START                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 881
#define RSRTM20XX_ATTR_PROTOCOL_I2C_SELECTED_BYTE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 882
#define RSRTM20XX_ATTR_PROTOCOL_I2C_BYTE_COUNT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 883
#define RSRTM20XX_ATTR_PROTOCOL_I2C_BYTE_ACK_BIT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 884
#define RSRTM20XX_ATTR_PROTOCOL_I2C_BYTE_ACK_BIT_START                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 885
#define RSRTM20XX_ATTR_PROTOCOL_I2C_BYTE_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 886
#define RSRTM20XX_ATTR_PROTOCOL_I2C_BYTE_START                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 887
#define RSRTM20XX_ATTR_PROTOCOL_I2C_BYTE_VALUE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 888
#define RSRTM20XX_ATTR_PROTOCOL_SPI_CLOCK_SOURCE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 382
#define RSRTM20XX_ATTR_PROTOCOL_SPI_DATA_SOURCE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 819
#define RSRTM20XX_ATTR_PROTOCOL_SPI_CLOCK_POLARITY                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 383
#define RSRTM20XX_ATTR_PROTOCOL_SPI_DATA_POLARITY                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 820
#define RSRTM20XX_ATTR_PROTOCOL_SPI_BIT_ORDER                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 380
#define RSRTM20XX_ATTR_PROTOCOL_SPI_WORD_LENGTH                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 381
#define RSRTM20XX_ATTR_PROTOCOL_SPI_FRAME_COUNT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 698
#define RSRTM20XX_ATTR_PROTOCOL_SPI_SELECTED_FRAME                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 889
#define RSRTM20XX_ATTR_PROTOCOL_SPI_FRAME_STATUS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 890
#define RSRTM20XX_ATTR_PROTOCOL_SPI_FRAME_START                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 891
#define RSRTM20XX_ATTR_PROTOCOL_SPI_FRAME_END                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 892
#define RSRTM20XX_ATTR_PROTOCOL_SPI_FRAME_DATA_MOSI                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 893
#define RSRTM20XX_ATTR_PROTOCOL_SPI_FRAME_DATA_MISO                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 894
#define RSRTM20XX_ATTR_PROTOCOL_SPI_WORD_COUNT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 895
#define RSRTM20XX_ATTR_PROTOCOL_SPI_SELECTED_WORD                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 896
#define RSRTM20XX_ATTR_PROTOCOL_SPI_WORD_START                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 897
#define RSRTM20XX_ATTR_PROTOCOL_SPI_WORD_END                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 898
#define RSRTM20XX_ATTR_PROTOCOL_SPI_WORD_MOSI                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 899
#define RSRTM20XX_ATTR_PROTOCOL_SPI_WORD_MISO                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 900
#define RSRTM20XX_ATTR_PROTOCOL_SSPI_CLOCK_SOURCE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 609
#define RSRTM20XX_ATTR_PROTOCOL_SSPI_DATA_SOURCE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 801
#define RSRTM20XX_ATTR_PROTOCOL_SSPI_CLOCK_POLARITY                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 594
#define RSRTM20XX_ATTR_PROTOCOL_SSPI_DATA_POLARITY                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 802
#define RSRTM20XX_ATTR_PROTOCOL_SSPI_INTERFRAME_TIME                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 597
#define RSRTM20XX_ATTR_PROTOCOL_SSPI_BIT_ORDER                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 598
#define RSRTM20XX_ATTR_PROTOCOL_SSPI_WORD_LENGTH                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 599
#define RSRTM20XX_ATTR_PROTOCOL_UART_SOURCE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 600
#define RSRTM20XX_ATTR_PROTOCOL_UART_RX_SOURCE_LINE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 699
#define RSRTM20XX_ATTR_PROTOCOL_UART_TX_SOURCE_LINE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 804
#define RSRTM20XX_ATTR_PROTOCOL_UART_POLARITY                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 601
#define RSRTM20XX_ATTR_PROTOCOL_UART_INTERFRAME_TIME                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 605
#define RSRTM20XX_ATTR_PROTOCOL_UART_WORD_LENGTH                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 606
#define RSRTM20XX_ATTR_PROTOCOL_UART_BIT_RATE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 604
#define RSRTM20XX_ATTR_PROTOCOL_UART_PARITY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 405
#define RSRTM20XX_ATTR_PROTOCOL_UART_STOP_BITS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 603
#define RSRTM20XX_ATTR_PROTOCOL_UART_RX_FRAME_COUNT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 701
#define RSRTM20XX_ATTR_PROTOCOL_UART_TX_FRAME_COUNT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 803
#define RSRTM20XX_ATTR_PROTOCOL_UART_SELECTED_FRAME                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 901
#define RSRTM20XX_ATTR_PROTOCOL_UART_RX_WORD_COUNT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 902
#define RSRTM20XX_ATTR_PROTOCOL_UART_TX_WORD_COUNT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 903
#define RSRTM20XX_ATTR_PROTOCOL_UART_SELECTED_WORD                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 904
#define RSRTM20XX_ATTR_PROTOCOL_UART_RX_WORD_VALUE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 921
#define RSRTM20XX_ATTR_PROTOCOL_UART_TX_WORD_VALUE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 922
#define RSRTM20XX_ATTR_PROTOCOL_UART_RX_WORD_START                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 923
#define RSRTM20XX_ATTR_PROTOCOL_UART_TX_WORD_START                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 924
#define RSRTM20XX_ATTR_PROTOCOL_UART_RX_WORD_END                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 925
#define RSRTM20XX_ATTR_PROTOCOL_UART_TX_WORD_END                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 926
#define RSRTM20XX_ATTR_PROTOCOL_UART_RX_WORD_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 927
#define RSRTM20XX_ATTR_PROTOCOL_UART_TX_WORD_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 928
#define RSRTM20XX_ATTR_PROTOCOL_LIN_SOURCE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 702
#define RSRTM20XX_ATTR_PROTOCOL_LIN_POLARITY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 741
#define RSRTM20XX_ATTR_PROTOCOL_LIN_STANDARD                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 703
#define RSRTM20XX_ATTR_PROTOCOL_LIN_BIT_RATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 704
#define RSRTM20XX_ATTR_PROTOCOL_LIN_FRAME_COUNT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 711
#define RSRTM20XX_ATTR_PROTOCOL_LIN_SELECTED_FRAME                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 905
#define RSRTM20XX_ATTR_PROTOCOL_LIN_FRAME_DATA                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 906
#define RSRTM20XX_ATTR_PROTOCOL_LIN_FRAME_STATUS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 907
#define RSRTM20XX_ATTR_PROTOCOL_LIN_FRAME_START                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 908
#define RSRTM20XX_ATTR_PROTOCOL_LIN_FRAME_END                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 909
#define RSRTM20XX_ATTR_PROTOCOL_LIN_FRAME_CHECKSUM_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 910
#define RSRTM20XX_ATTR_PROTOCOL_LIN_FRAME_CHECKSUM_VALUE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 911
#define RSRTM20XX_ATTR_PROTOCOL_LIN_FRAME_PARITY_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 912
#define RSRTM20XX_ATTR_PROTOCOL_LIN_FRAME_PARITY_VALUE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 913
#define RSRTM20XX_ATTR_PROTOCOL_LIN_FRAME_ID_VALUE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 914
#define RSRTM20XX_ATTR_PROTOCOL_LIN_FRAME_SYNC_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 915
#define RSRTM20XX_ATTR_PROTOCOL_LIN_FRAME_SYNC_VALUE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 916
#define RSRTM20XX_ATTR_PROTOCOL_LIN_FRAME_VERSION                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 917
#define RSRTM20XX_ATTR_PROTOCOL_LIN_BYTE_COUNT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 918
#define RSRTM20XX_ATTR_PROTOCOL_LIN_SELECTED_BYTE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1022
#define RSRTM20XX_ATTR_PROTOCOL_LIN_BYTE_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 919
#define RSRTM20XX_ATTR_PROTOCOL_LIN_BYTE_VALUE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 920
#define RSRTM20XX_ATTR_PROTOCOL_CAN_LINE_TYPE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 719
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SOURCE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 720
#define RSRTM20XX_ATTR_PROTOCOL_CAN_BIT_RATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 721
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SAMPLE_POINT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 722
#define RSRTM20XX_ATTR_PROTOCOL_CAN_FRAME_COUNT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 731
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SELECTED_FRAME                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 929
#define RSRTM20XX_ATTR_PROTOCOL_CAN_FRAME_TYPE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 930
#define RSRTM20XX_ATTR_PROTOCOL_CAN_FRAME_STATUS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 931
#define RSRTM20XX_ATTR_PROTOCOL_CAN_FRAME_START                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 932
#define RSRTM20XX_ATTR_PROTOCOL_CAN_FRAME_END                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 933
#define RSRTM20XX_ATTR_PROTOCOL_CAN_FRAME_DATA                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 934
#define RSRTM20XX_ATTR_PROTOCOL_CAN_ACK_STATE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 935
#define RSRTM20XX_ATTR_PROTOCOL_CAN_ACK_VALUE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 936
#define RSRTM20XX_ATTR_PROTOCOL_CAN_CHECKSUM_STATE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 937
#define RSRTM20XX_ATTR_PROTOCOL_CAN_CHECKSUM_VALUE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 938
#define RSRTM20XX_ATTR_PROTOCOL_CAN_DATA_LENGTH_STATE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 939
#define RSRTM20XX_ATTR_PROTOCOL_CAN_DATA_LENGTH_VALUE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 940
#define RSRTM20XX_ATTR_PROTOCOL_CAN_ID_STATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 941
#define RSRTM20XX_ATTR_PROTOCOL_CAN_ID_VALUE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 942
#define RSRTM20XX_ATTR_PROTOCOL_CAN_ID_TYPE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 943
#define RSRTM20XX_ATTR_PROTOCOL_CAN_BSE_POSITION                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 944
#define RSRTM20XX_ATTR_PROTOCOL_CAN_BYTE_COUNT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 945
#define RSRTM20XX_ATTR_PROTOCOL_CAN_SELECTED_BYTE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 946
#define RSRTM20XX_ATTR_PROTOCOL_CAN_BYTE_STATE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 947
#define RSRTM20XX_ATTR_PROTOCOL_CAN_BYTE_VALUE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 948
#define RSRTM20XX_ATTR_PARALLEL_BUS_ANALYZED_LINES                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 608
#define RSRTM20XX_ATTR_PARALLEL_BUS_FRAME_COUNT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 846
#define RSRTM20XX_ATTR_PARALLEL_BUS_SELECTED_FRAME                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 857
#define RSRTM20XX_ATTR_PARALLEL_BUS_FRAME_DATA                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 858
#define RSRTM20XX_ATTR_PARALLEL_BUS_FRAME_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 859
#define RSRTM20XX_ATTR_PARALLEL_BUS_FRAME_START                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 860
#define RSRTM20XX_ATTR_PARALLEL_BUS_FRAME_END                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 861
#define RSRTM20XX_ATTR_PARALLEL_BUS_CLOCKED_ANALYZED_LINES                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 847
#define RSRTM20XX_ATTR_PARALLEL_BUS_CLOCKED_FRAME_COUNT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 848
#define RSRTM20XX_ATTR_PARALLEL_BUS_CLOCKED_SLOPE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 849
#define RSRTM20XX_ATTR_PARALLEL_BUS_CLOCKED_CS_POLARITY                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 850
#define RSRTM20XX_ATTR_PARALLEL_BUS_CLOCKED_SELECTED_FRAME                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 862
#define RSRTM20XX_ATTR_PARALLEL_BUS_CLOCKED_FRAME_DATA                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 863
#define RSRTM20XX_ATTR_PARALLEL_BUS_CLOCKED_FRAME_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 864
#define RSRTM20XX_ATTR_PARALLEL_BUS_CLOCKED_FRAME_START                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 865
#define RSRTM20XX_ATTR_PARALLEL_BUS_CLOCKED_FRAME_END                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 866
#define RSRTM20XX_ATTR_PARALLEL_BUS_CLOCKED_SOURCE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1588
#define RSRTM20XX_ATTR_PARALLEL_BUS_CLOCKED_CHIP_SELECT_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1589
#define RSRTM20XX_ATTR_PARALLEL_BUS_CLOCKED_CHIP_SELECT_SOURCE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1590
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_VARIANT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1111
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_WORD_LENGTH                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1113
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_CHANNEL_ORDER                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1114
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_BIT_ORDER                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1115
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_CHANNEL_OFFSET                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1116
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_CHANNEL_LENGTH                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1117
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_FRAME_OFFSET                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1118
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TDM_CHANNELS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1119
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_SELECTED_FRAME                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1120
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_FRAME_COUNT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1121
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_FRAME_STATUS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1122
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_FRAME_START                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1123
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_FRAME_STOP                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1124
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_FRAME_LEFT_CHANNEL_STATUS             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1125
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_FRAME_RIGHT_CHANNEL_STATUS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1126
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_FRAME_LEFT_CHANNEL_VALUE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1127
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_FRAME_RIGHT_CHANNEL_VALUE             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1128
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_FRAME_TDM_STATUS                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1131
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_FRAME_TDM_VALUE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1132
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_CLOCK_SOURCE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1133
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_CLOCK_THRESHOLD                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1134
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_CLOCK_POLARITY                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1135
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_WS_SOURCE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1136
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_WS_THRESHOLD                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1137
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_WS_POLARITY                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1138
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_DATA_SOURCE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1139
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_DATA_THRESHOLD                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1140
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_DATA_POLARITY                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1141
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_DISPLAY_MODE           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1151
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_LEFT_POSITION          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1152
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_LEFT_SCALE             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1153
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_RIGHT_POSITION         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1154
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_RIGHT_SCALE            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1155
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_SET_TO_DEFAULT         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1159
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_SET_TO_SCREEN          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1161
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_CHANNEL_ENABLED        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1156
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_CHANNEL_POSITION       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1142
#define RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_CHANNEL_SCALE          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1157
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_SOURCE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1200
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_POLARITY                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1201
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_TRESHOLD_HIGH                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1202
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_TRESHOLD_HLOW                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1203
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_TIMING                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1204
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_COUNT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1205
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_SELECTED_WORD                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1206
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_START                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1207
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_STOP                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1208
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_STATUS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1209
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_TRANSMISSION_DIRECTION          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1210
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_TYPE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1211
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_PARITY                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1219
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_RESPONSE_TIME                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1212
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_VALUE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1217
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_INTERMESSAGE_GATE_TIME          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1218
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_COMMAND_WORD_COUNT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1229
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_COMMAND_WORD_MODE_CODE_TYPE          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1213
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_COMMAND_WORD_MODE_CODE_VALUE         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1214
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_COMMAND_WORD_RT_ADDRESS              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1215
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_COMMAND_WORD_SUB_ADDRESS             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1216
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_RECEIVED_BIT_STATUS             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1220
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_BUSY_BIT_STATUS                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1221
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_BUS_CONTROL_BIT_STATUS          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1222
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_INSTRUMENTATION_BIT_STATUS      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1223
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_MESSAGE_ERROR_BIT_STATUS        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1224
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_STATUS_RT_ADDRESS               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1225
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_SUBSYSTEM_BIT_STATUS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1226
#define RSRTM20XX_ATTR_PROTOCOL_MILSTD_WORD_TERMINAL_BIT_STATUS             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1227
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SOURCE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1387
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_POLARITY                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1339
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_BIT_RATE_MODE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1340
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_BIT_RATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1341
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_THRESHOLD_HIGH                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1342
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_THRESHOLD_LOW                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1343
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_DATA_FORMAT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1388
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_COUNT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1349
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_SELECTED_WORD                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1444
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_STATUS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1412
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_START                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1413
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_STOP                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1414
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_SDI                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1415
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_SSM                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1416
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_LABEL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1417
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_LABEL_VALUE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1418
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_PATTERN                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1419
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_PARITY                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1420
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_FORMAT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1421
#define RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_DATA                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1423
#define RSRTM20XX_ATTR_POWER_ANALYSIS_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1197
#define RSRTM20XX_ATTR_POWER_ANALYSIS_TYPE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1024
#define RSRTM20XX_ATTR_POWER_ANALYSIS_RESULT_TABLE_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1025
#define RSRTM20XX_ATTR_POWER_ANALYSIS_RESET_STATISTICS                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1027
#define RSRTM20XX_ATTR_POWER_ANALYSIS_STATISTICS_VISIBLE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1028
#define RSRTM20XX_ATTR_POWER_ANALYSIS_START_DESKEW                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1030
#define RSRTM20XX_ATTR_POWER_ANALYSIS_ZERO_OFFSET                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1073
#define RSRTM20XX_ATTR_POWER_ANALYSIS_VOLTAGE_SOURCE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1074
#define RSRTM20XX_ATTR_POWER_ANALYSIS_CURRENT_SOURCE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1075
#define RSRTM20XX_ATTR_POWER_ANALYSIS_ADD_REPORT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1506
#define RSRTM20XX_ATTR_POWER_ANALYSIS_REPORT_DUT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1507
#define RSRTM20XX_ATTR_POWER_ANALYSIS_REPORT_USER                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1508
#define RSRTM20XX_ATTR_POWER_ANALYSIS_REPORT_SITE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1509
#define RSRTM20XX_ATTR_POWER_ANALYSIS_REPORT_TEMPERATURE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1510
#define RSRTM20XX_ATTR_POWER_ANALYSIS_REPORT_DESCRIPTION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1511
#define RSRTM20XX_ATTR_POWER_ANALYSIS_REPORT_OUTPUT_FOLDER                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1512
#define RSRTM20XX_ATTR_START_POWER_QUALITY_MEASUREMENT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1032
#define RSRTM20XX_ATTR_POWER_QUALITY_MEASUREMENT_ADD_REPORT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1513
#define RSRTM20XX_ATTR_START_POWER_CONSUMPTION_MEASUREMENT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1037
#define RSRTM20XX_ATTR_RESTART_POWER_CONSUMPTION_MEASUREMENT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1039
#define RSRTM20XX_ATTR_POWER_CONSUMPTION_MEASUREMENT_ADD_REPORT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1514
#define RSRTM20XX_ATTR_POWER_CONSUMPTION_MEASUREMENT_DURATION_RESULT        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1179
#define RSRTM20XX_ATTR_POWER_CONSUMPTION_MEASUREMENT_ENERGY_RESULT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1180
#define RSRTM20XX_ATTR_POWER_CONSUMPTION_MEASUREMENT_ACTIVE_RESULT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1181
#define RSRTM20XX_ATTR_START_POWER_CURRENT_HARMONICS_MEASUREMENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1042
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_STANDARD         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1043
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_FUNDAMENTAL      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1044
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_FUNDAMENTAL_MIL  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1456
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_FUNDAMENTAL_RTC  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1457
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_NUMBER_OF_MEASURED_HARMONICS RS_SPECIFIC_PUBLIC_ATTR_BASE + 1458
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_ADD_REPORT       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1515
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_LIMIT_CHECK_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1046
#define RSRTM20XX_ATTR_RESET_POWER_CURRENT_HARMONICS_MEASUREMENT_RESULTS    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1047
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_FREQUENCY_CURRENT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1459
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_FREQUENCY_AVERAGE RS_SPECIFIC_PUBLIC_ATTR_BASE + 1460
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_FREQUENCY_MAXIMUM RS_SPECIFIC_PUBLIC_ATTR_BASE + 1461
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_FREQUENCY_MINIMUM RS_SPECIFIC_PUBLIC_ATTR_BASE + 1462
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_FREQUENCY_STANDARD_DEVIATION RS_SPECIFIC_PUBLIC_ATTR_BASE + 1463
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_TOTAL_HARMONIC_DISTORTION_CURRENT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1464
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_TOTAL_HARMONIC_DISTORTION_AVERAGE RS_SPECIFIC_PUBLIC_ATTR_BASE + 1465
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_TOTAL_HARMONIC_DISTORTION_MAXIMUM RS_SPECIFIC_PUBLIC_ATTR_BASE + 1466
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_TOTAL_HARMONIC_DISTORTION_MINIMUM RS_SPECIFIC_PUBLIC_ATTR_BASE + 1467
#define RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_TOTAL_HARMONIC_DISTORTION_STANDARD_DEVIATION RS_SPECIFIC_PUBLIC_ATTR_BASE + 1468
#define RSRTM20XX_ATTR_CURRENT_HARMONICS_EXPORT_NAME                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1591
#define RSRTM20XX_ATTR_CURRENT_HARMONICS_EXPORT_SAVE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1592
#define RSRTM20XX_ATTR_START_POWER_INRUSH_CURRENT_MEASUREMENT               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1049
#define RSRTM20XX_ATTR_POWER_INRUSH_CURRENT_MEASUREMENT_GATE_COUNT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1050
#define RSRTM20XX_ATTR_POWER_INRUSH_CURRENT_MEASUREMENT_ADD_REPORT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1516
#define RSRTM20XX_ATTR_POWER_INRUSH_CURRENT_MEASUREMENT_GATE_START          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1051
#define RSRTM20XX_ATTR_POWER_INRUSH_CURRENT_MEASUREMENT_GATE_STOP           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1052
#define RSRTM20XX_ATTR_POWER_INRUSH_CURRENT_MEASUREMENT_AREA_RESULT         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1053
#define RSRTM20XX_ATTR_POWER_INRUSH_CURRENT_MEASUREMENT_MAXIMUM_RESULT      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1054
#define RSRTM20XX_ATTR_START_POWER_RIPPLE_MEASUREMENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1056
#define RSRTM20XX_ATTR_POWER_RIPPLE_MEASUREMENT_ADD_REPORT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1517
#define RSRTM20XX_ATTR_START_POWER_SPECTRUM_MEASUREMENT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1059
#define RSRTM20XX_ATTR_POWER_SPECTRUM_MEASUREMENT_FREQUENCY                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1060
#define RSRTM20XX_ATTR_POWER_SPECTRUM_MEASUREMENT_ADD_REPORT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1518
#define RSRTM20XX_ATTR_RESET_POWER_SPECTRUM_MEASUREMENT_RESULTS             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1062
#define RSRTM20XX_ATTR_START_POWER_SLEW_RATE_MEASUREMENT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1063
#define RSRTM20XX_ATTR_POWER_SLEW_RATE_MEASUREMENT_DELTA_SAMPLE             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1065
#define RSRTM20XX_ATTR_POWER_SLEW_RATE_MEASUREMENT_DELTA_TIME               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1066
#define RSRTM20XX_ATTR_POWER_SLEW_RATE_MEASUREMENT_ADD_REPORT               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1519
#define RSRTM20XX_ATTR_START_POWER_MODULATION_MEASUREMENT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1069
#define RSRTM20XX_ATTR_POWER_MODULATION_MEASUREMENT_TYPE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1070
#define RSRTM20XX_ATTR_POWER_MODULATION_MEASUREMENT_ADD_REPORT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1520
#define RSRTM20XX_ATTR_START_POWER_DYNAMIC_ON_RESISTANCE_MEASUREMENT        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1077
#define RSRTM20XX_ATTR_POWER_DYNAMIC_ON_RESISTANCE_MEASUREMENT_ADD_REPORT   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1521
#define RSRTM20XX_ATTR_POWER_DYNAMIC_ON_RESISTANCE_MEASUREMENT_GATE_START   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1078
#define RSRTM20XX_ATTR_POWER_DYNAMIC_ON_RESISTANCE_MEASUREMENT_GATE_STOP    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1079
#define RSRTM20XX_ATTR_POWER_DYNAMIC_ON_RESISTANCE_MEASUREMENT_RESULT       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1080
#define RSRTM20XX_ATTR_START_POWER_EFFICIENCY_MEASUREMENT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1082
#define RSRTM20XX_ATTR_POWER_EFFICIENCY_MEASUREMENT_ADD_REPORT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1522
#define RSRTM20XX_ATTR_START_POWER_SWITCHINGLOSS_MEASUREMENT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1085
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TYPE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1086
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_ADD_REPORT           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1523
#define RSRTM20XX_ATTR_SET_POWER_SWITSCHINGLOSS_MEASUREMENT_GATE_TO_WAVEFORM RS_SPECIFIC_PUBLIC_ATTR_BASE + 1088
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_GATE_CONDUCTION_START RS_SPECIFIC_PUBLIC_ATTR_BASE + 1089
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_GATE_CONDUCTION_STOP RS_SPECIFIC_PUBLIC_ATTR_BASE + 1090
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_GATE_NO_CONDUCTION_START RS_SPECIFIC_PUBLIC_ATTR_BASE + 1091
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_GATE_NO_CONDUCTION_STOP RS_SPECIFIC_PUBLIC_ATTR_BASE + 1092
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_GATE_TURN_ON_START   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1093
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_GATE_TURN_ON_STOP    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1094
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_GATE_TURN_OFF_START  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1095
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_GATE_TURN_OFF_STOP   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1096
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_CONDUCTION_ENERGY_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1097
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_CONDUCTION_POWER_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1098
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_NO_CONDUCTION_ENERGY_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1099
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_NO_CONDUCTION_POWER_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1100
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TURN_ON_ENERGY_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1101
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TURN_ON_POWER_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1102
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TURN_OFF_ENERGY_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1103
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TURN_OFF_POWER_RESULT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1104
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TOTAL_ENERGY_RESULT  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1105
#define RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TOTAL_POWER_RESULT   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1106
#define RSRTM20XX_ATTR_START_POWER_ON_OFF_MEASUREMENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1108
#define RSRTM20XX_ATTR_POWER_ON_OFF_MEASUREMENT_TYPE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1109
#define RSRTM20XX_ATTR_POWER_ON_OFF_MEASUREMENT_ADD_REPORT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1524
#define RSRTM20XX_ATTR_POWER_ON_OFF_MEASUREMENT_RESULT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1110
#define RSRTM20XX_ATTR_START_POWER_SAFE_OPERATING_AREA_MEASUREMENT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1429
#define RSRTM20XX_ATTR_RESTART_POWER_SAFE_OPERATING_AREA_MEASUREMENT        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1445
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_ADD_REPORT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1525
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_SCALE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1446
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_MASK_SCALE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1447
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_DISPLAY_SCALE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1448
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LIN_ADD                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1432
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LINEAR_INSERT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1433
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LINEAR_REMOVE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1434
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LINEAR_COUNT               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1435
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LOGARITHMIC_ADD            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1436
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LOGARITHMIC_INSERT         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1437
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LOGARITHMIC_REMOVE         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1438
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LOGARITHMIC_COUNT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1439
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_SELECTED_POINT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1449
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LINEAR_POINT_VOLTAGE       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1450
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LINEAR_POINT_CURRENT_MIN   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1451
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LINEAR_POINT_CURRENT_MAX   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1452
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LOGARITHMIC_POINT_VOLTAGE  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1453
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LOGARITHMIC_POINT_CURRENT_MIN RS_SPECIFIC_PUBLIC_ATTR_BASE + 1454
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LOGARITHMIC_POINT_CURRENT_MAX RS_SPECIFIC_PUBLIC_ATTR_BASE + 1455
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_ACQUISITION_TOLERANCE RS_SPECIFIC_PUBLIC_ATTR_BASE + 1469
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_ACQUISITION_PASSED_RESULTS RS_SPECIFIC_PUBLIC_ATTR_BASE + 1470
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_ACQUISITION_FAILED_RESULTS RS_SPECIFIC_PUBLIC_ATTR_BASE + 1471
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_ACQUISITION_FAILED_RATE RS_SPECIFIC_PUBLIC_ATTR_BASE + 1472
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_ACQUISITION_NUMBER_OF_POINTS RS_SPECIFIC_PUBLIC_ATTR_BASE + 1473
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_ACQUISITION_STATUS  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1474
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_ACQUISITION_VIOLATION_COUNT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1475
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_ACQUISITION_VIOLATION_CURRENT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1476
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_ACQUISITION_VIOLATION_VOLTAGE RS_SPECIFIC_PUBLIC_ATTR_BASE + 1477
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_TOLERANCE     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1478
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_PASSED_RESULTS RS_SPECIFIC_PUBLIC_ATTR_BASE + 1479
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_FAILED_RESULTS RS_SPECIFIC_PUBLIC_ATTR_BASE + 1480
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_FAILED_RATE   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1481
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_NUMBER_OF_ACQUISITIONS RS_SPECIFIC_PUBLIC_ATTR_BASE + 1482
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_STATUS        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1483
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VIOLATION_COUNT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1484
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_PASSED_SAMPLES RS_SPECIFIC_PUBLIC_ATTR_BASE + 1485
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_FAILED_SAMPLES RS_SPECIFIC_PUBLIC_ATTR_BASE + 1486
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_NUMBER_OF_SAMPLES RS_SPECIFIC_PUBLIC_ATTR_BASE + 1487
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VIOLATION_CURRENT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1488
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VIOLATION_VOLTAGE RS_SPECIFIC_PUBLIC_ATTR_BASE + 1489
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VOLTAGE_DATA_CONVERSION_X_START RS_SPECIFIC_PUBLIC_ATTR_BASE + 1493
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VOLTAGE_DATA_CONVERSION_X_INCREMENT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1494
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VOLTAGE_DATA_CONVERSION_Y_START RS_SPECIFIC_PUBLIC_ATTR_BASE + 1495
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VOLTAGE_DATA_CONVERSION_Y_INCREMENT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1496
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VOLTAGE_DATA_CONVERSION_Y_RESOLUTION RS_SPECIFIC_PUBLIC_ATTR_BASE + 1497
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_CURRENT_DATA_CONVERSION_X_START RS_SPECIFIC_PUBLIC_ATTR_BASE + 1501
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_CURRENT_DATA_CONVERSION_X_INCREMENT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1502
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_CURRENT_DATA_CONVERSION_Y_START RS_SPECIFIC_PUBLIC_ATTR_BASE + 1503
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_CURRENT_DATA_CONVERSION_Y_INCREMENT RS_SPECIFIC_PUBLIC_ATTR_BASE + 1504
#define RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_CURRENT_DATA_CONVERSION_Y_RESOLUTION RS_SPECIFIC_PUBLIC_ATTR_BASE + 1505
#define RSRTM20XX_ATTR_START_POWER_TRANSIENT_RESPONSE_MEASUREMENT           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1526
#define RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_SIGNAL_HIGH                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1532
#define RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_SIGNAL_LOW                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1533
#define RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_ADD_REPORT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1536
#define RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_START_TIME                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1538
#define RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_STOP_TIME                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1539
#define RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_DELAY_TIME                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1540
#define RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_OVERSHOOT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1541
#define RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_PEAK_TIME                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1542
#define RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_PEAK_VALUE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1543
#define RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_RISE_TIME                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1544
#define RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_SETTLING_TIME               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1545
#define RSRTM20XX_ATTR_SPECTRUM_ANALYSIS_ENABLE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1593
#define RSRTM20XX_ATTR_SPECTRUM_ANALYSIS_SOURCE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1594
#define RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_AVERAGE_COMPLETE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1595
#define RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_AVERAGE_COUNT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1596
#define RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_MAGNITUDE_SCALE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1597
#define RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_POSITION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1598
#define RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_RESET                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1599
#define RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_SCALE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1600
#define RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_WINDOW_TYPE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1601
#define RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_START                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1602
#define RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_STOP                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1603
#define RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_CENTER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1604
#define RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_SPAN                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1605
#define RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_BANDWIDTH_AUTO                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1606
#define RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_BANDWIDTH_RATIO                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1607
#define RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_BANDWIDTH_RESOLUTION              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1608
#define RSRTM20XX_ATTR_SPECTRUM_SPECTROGRAM_RESET                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1609
#define RSRTM20XX_ATTR_SPECTRUM_SPECTROGRAM_SCALE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1610
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_ENABLE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1611
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_SOURCE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1612
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_DISPLAY                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1613
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_SETUP_DISTANCE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1614
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_SETUP_EXCURSION                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1615
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_SETUP_MINIMUM_LEVEL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1616
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_SETUP_DETECTION_MODE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1617
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_SETUP_MAXIMUM_WIDTH                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1618
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_REFERENCE_MODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1619
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_REFERENCE_FREQUENCY                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1620
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_REFERENCE_SPAN                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1621
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_REFERENCE_INDEX                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1622
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_TABLE_ENABLE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1623
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_TABLE_DETECTED_PEAKS                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1624
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_TABLE_RESULT_MODE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1625
#define RSRTM20XX_ATTR_SPECTRUM_MARKER_TABLE_POSITION                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1626
#define RSRTM20XX_ATTR_SPECTRUM_DISPLAY_MAGNITUDE_MODE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1627
#define RSRTM20XX_ATTR_SPECTRUM_DISPLAY_MAXIMUM_LEVEL                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1628
#define RSRTM20XX_ATTR_SPECTRUM_DISPLAY_MINIMUM_LEVEL                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1629
#define RSRTM20XX_ATTR_SPECTRUM_DISPLAY_COLOR_SCHEME                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1630
#define RSRTM20XX_ATTR_SPECTRUM_DISPLAY_SPECTROGRAM_COLOR_SCHEME            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1631
#define RSRTM20XX_ATTR_SPECTRUM_DISPLAY_FREQUENCY_DOMAIN_ENABLE             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1632
#define RSRTM20XX_ATTR_SPECTRUM_DISPLAY_SPECTROGRAM_ENABLE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1633
#define RSRTM20XX_ATTR_SPECTRUM_DISPLAY_TIME_DOMAIN_ENABLE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1634
#define RSRTM20XX_ATTR_SPECTRUM_WAVEFORM_ENABLE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1635
#define RSRTM20XX_ATTR_SPECTRUM_WAVEFORM_MINIMUM_ENABLE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1636
#define RSRTM20XX_ATTR_SPECTRUM_WAVEFORM_MAXIMUM_ENABLE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1637
#define RSRTM20XX_ATTR_SPECTRUM_WAVEFORM_AVERAGE_ENABLE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1638
#define RSRTM20XX_ATTR_SPECTRUM_TIME_DOMAIN_POSITION                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1639
#define RSRTM20XX_ATTR_SPECTRUM_TIME_DOMAIN_RANGE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1640
#define RSRTM20XX_ATTR_DVM_ENABLE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1641
#define RSRTM20XX_ATTR_DVM_SOURCE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1642
#define RSRTM20XX_ATTR_DVM_TYPE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1643
#define RSRTM20XX_ATTR_DVM_POSITION                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1644
#define RSRTM20XX_ATTR_DVM_RESULT                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1645
#define RSRTM20XX_ATTR_DVM_RESULT_STATUS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1646
#define RSRTM20XX_ATTR_COUNTER_ENABLE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1647
#define RSRTM20XX_ATTR_COUNTER_FREQUENCY                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1648
#define RSRTM20XX_ATTR_COUNTER_PERIOD                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1649
#define RSRTM20XX_ATTR_COLOR_AND_BRIGHTNESS                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 547
#define RSRTM20XX_ATTR_SIGNAL_STYLE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 427
#define RSRTM20XX_ATTR_DIAGRAM_GRID_STYLE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 550
#define RSRTM20XX_ATTR_WAVEFORM_INTENSITY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 573
#define RSRTM20XX_ATTR_BACKGROUND_INTENSITY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 545
#define RSRTM20XX_ATTR_GRID_INTENSITY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 546
#define RSRTM20XX_ATTR_DISPLAY_LANGUAGE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 549
#define RSRTM20XX_ATTR_DIAGRAM_MODE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 548
#define RSRTM20XX_ATTR_DISPLAY_SEND_MESSAGE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1561
#define RSRTM20XX_ATTR_DISPLAY_CLOSE_DIALOG                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1321
#define RSRTM20XX_ATTR_DIALOG_TRANSPARENCY                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1650
#define RSRTM20XX_ATTR_PERSISTENCE_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 422
#define RSRTM20XX_ATTR_INFINITE_PERSISTENCE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 423
#define RSRTM20XX_ATTR_PERSISTENCE_TIME                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 424
#define RSRTM20XX_ATTR_PERSISTENCE_TIME_AUTO                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 544
#define RSRTM20XX_ATTR_PERSISTENCE_CLEAR                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 572
#define RSRTM20XX_ATTR_XY_X_SOURCE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 551
#define RSRTM20XX_ATTR_XY_Y_SOURCE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 552
#define RSRTM20XX_ATTR_XY_Y2_SOURCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 554
#define RSRTM20XX_ATTR_XY_Z_SOURCE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 553
#define RSRTM20XX_ATTR_XY_Z_TRESHOLD                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 556
#define RSRTM20XX_ATTR_XY_INTENSITY_CONTROL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 555
#define RSRTM20XX_ATTR_ZOOM_WINDOW_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 557
#define RSRTM20XX_ATTR_ZOOM_DIAGRAM_TIMEBASE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 558
#define RSRTM20XX_ATTR_ZOOM_OFFSET_OF_TRIGGER_POINT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 559
#define RSRTM20XX_ATTR_ZOOM_REFERENCE_POINT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 560
#define RSRTM20XX_ATTR_ZOOM_SAMPLE_RATE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1322
#define RSRTM20XX_ATTR_MARKER_SET                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 561
#define RSRTM20XX_ATTR_MARKER_CLEAR                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 562
#define RSRTM20XX_ATTR_MARKER_CLEAR_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 563
#define RSRTM20XX_ATTR_MARKER_NEXT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 564
#define RSRTM20XX_ATTR_MARKER_PREVIOUS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 565
#define RSRTM20XX_ATTR_VIRTUAL_SCREEN_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 851
#define RSRTM20XX_ATTR_VIRTUAL_SCREEN_POSITION                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 852
#define RSRTM20XX_ATTR_HISTORY_CHANNEL_CURRENT_ACQUISITION                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1292
#define RSRTM20XX_ATTR_HISTORY_CHANNEL_PLAYER                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1293
#define RSRTM20XX_ATTR_HISTORY_CHANNEL_START_ACQUISITION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1294
#define RSRTM20XX_ATTR_HISTORY_CHANNEL_STOP_ACQUISITION                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1295
#define RSRTM20XX_ATTR_HISTORY_CHANNEL_PLAY_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1296
#define RSRTM20XX_ATTR_HISTORY_CHANNEL_SPEED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1297
#define RSRTM20XX_ATTR_HISTORY_CHANNEL_REPEAT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1298
#define RSRTM20XX_ATTR_HISTORY_CHANNEL_ACQUISITON_RELATIVE_TIME             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1546
#define RSRTM20XX_ATTR_HISTORY_CHANNEL_ALL_DATES                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1651
#define RSRTM20XX_ATTR_HISTORY_CHANNEL_ALL_TIME_DIFFERENCES                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1652
#define RSRTM20XX_ATTR_HISTORY_CHANNEL_ALL_DAYTIMES                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1653
#define RSRTM20XX_ATTR_HISTORY_DIGITAL_CURRENT_ACQUISITION                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1299
#define RSRTM20XX_ATTR_HISTORY_DIGITAL_PLAYER                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1300
#define RSRTM20XX_ATTR_HISTORY_DIGITAL_START_ACQUISITION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1301
#define RSRTM20XX_ATTR_HISTORY_DIGITAL_STOP_ACQUISITION                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1302
#define RSRTM20XX_ATTR_HISTORY_DIGITAL_PLAY_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1303
#define RSRTM20XX_ATTR_HISTORY_DIGITAL_SPEED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1304
#define RSRTM20XX_ATTR_HISTORY_DIGITAL_REPEAT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1305
#define RSRTM20XX_ATTR_HISTORY_DIGITAL_ACQUISITON_RELATIVE_TIME             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1549
#define RSRTM20XX_ATTR_HISTORY_DIGITAL_ALL_DATES                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1654
#define RSRTM20XX_ATTR_HISTORY_DIGITAL_ALL_TIME_DIFFERENCES                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1655
#define RSRTM20XX_ATTR_HISTORY_DIGITAL_ALL_DAYTIMES                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1656
#define RSRTM20XX_ATTR_HISTORY_MATH_CURRENT_ACQUISITION                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1306
#define RSRTM20XX_ATTR_HISTORY_MATH_PLAYER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1307
#define RSRTM20XX_ATTR_HISTORY_MATH_START_ACQUISITION                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1308
#define RSRTM20XX_ATTR_HISTORY_MATH_STOP_ACQUISITION                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1309
#define RSRTM20XX_ATTR_HISTORY_MATH_PLAY_ALL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1310
#define RSRTM20XX_ATTR_HISTORY_MATH_SPEED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1311
#define RSRTM20XX_ATTR_HISTORY_MATH_REPEAT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1312
#define RSRTM20XX_ATTR_HISTORY_MATH_ACQUISITON_RELATIVE_TIME                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1552
#define RSRTM20XX_ATTR_HISTORY_MATH_ALL_DATES                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1657
#define RSRTM20XX_ATTR_HISTORY_MATH_ALL_TIME_DIFFERENCES                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1658
#define RSRTM20XX_ATTR_HISTORY_MATH_ALL_DAYTIMES                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1659
#define RSRTM20XX_ATTR_HISTORY_PROTOCOL_CURRENT_ACQUISITION                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1313
#define RSRTM20XX_ATTR_HISTORY_PROTOCOL_PLAYER                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1314
#define RSRTM20XX_ATTR_HISTORY_PROTOCOL_START_ACQUISITION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1315
#define RSRTM20XX_ATTR_HISTORY_PROTOCOL_STOP_ACQUISITION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1316
#define RSRTM20XX_ATTR_HISTORY_PROTOCOL_PLAY_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1317
#define RSRTM20XX_ATTR_HISTORY_PROTOCOL_SPEED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1318
#define RSRTM20XX_ATTR_HISTORY_PROTOCOL_REPEAT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1319
#define RSRTM20XX_ATTR_HISTORY_PROTOCOL_ACQUISITON_RELATIVE_TIME            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1555
#define RSRTM20XX_ATTR_HISTORY_PROTOCOL_ALL_DATES                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1660
#define RSRTM20XX_ATTR_HISTORY_PROTOCOL_ALL_TIME_DIFFERENCES                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1661
#define RSRTM20XX_ATTR_HISTORY_PROTOCOL_ALL_DAYTIMES                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1662
#define RSRTM20XX_ATTR_HISTORY_SPECTRUM_CURRENT_ACQUISITION                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1663
#define RSRTM20XX_ATTR_HISTORY_SPECTRUM_PLAYER                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1664
#define RSRTM20XX_ATTR_HISTORY_SPECTRUM_START_ACQUISITION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1665
#define RSRTM20XX_ATTR_HISTORY_SPECTRUM_STOP_ACQUISITION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1666
#define RSRTM20XX_ATTR_HISTORY_SPECTRUM_PLAY_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1667
#define RSRTM20XX_ATTR_HISTORY_SPECTRUM_SPEED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1668
#define RSRTM20XX_ATTR_HISTORY_SPECTRUM_REPEAT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1669
#define RSRTM20XX_ATTR_HISTORY_SPECTRUM_EXPORT_NAME                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1670
#define RSRTM20XX_ATTR_HISTORY_SPECTRUM_EXPORT_SAVE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1671
#define RSRTM20XX_ATTR_SEGMENTATION_RECORD_LENGTH                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1323
#define RSRTM20XX_ATTR_ULTRA_SEGMENTATION_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1324
#define RSRTM20XX_ATTR_SEGMENTATION_NX_SINGLE_MAXIMUM                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1325
#define RSRTM20XX_ATTR_SEGMENTATION_MAXIMUM_NUMBER_OF_SEGMENTS              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1326
#define RSRTM20XX_ATTR_SEGMENTATION_NUMBER_OF_AVAILABLE_SEGMENTS            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1327
#define RSRTM20XX_ATTR_DRIVES                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 297
#define RSRTM20XX_ATTR_CHANGE_DRIVE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 298
#define RSRTM20XX_ATTR_CURRENT_DIRECTORY                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 299
#define RSRTM20XX_ATTR_CREATE_DIRECTORY                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 300
#define RSRTM20XX_ATTR_REMOVE_DIRECTORY                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 301
#define RSRTM20XX_ATTR_DELETE_FILE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 302
#define RSRTM20XX_ATTR_HARDCOPY_FILE_NAME                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 303
#define RSRTM20XX_ATTR_HARDCOPY_INCLUDE_MENU_IN_SCREENSHOT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1562
#define RSRTM20XX_ATTR_HARDCOPY_DEVICE_DESTINATION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 455
#define RSRTM20XX_ATTR_HARDCOPY_COLOR_SCHEME                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 571
#define RSRTM20XX_ATTR_HARDCOPY_COMMAND_SET                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1443
#define RSRTM20XX_ATTR_HARDCOPY_DEVICE_LANGUAGE_OUTPUT_FORMAT               RS_SPECIFIC_PUBLIC_ATTR_BASE + 456
#define RSRTM20XX_ATTR_HARDCOPY_DEVICE_ORIENTATION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 457
#define RSRTM20XX_ATTR_HARDCOPY_PAGE_SIZE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 567
#define RSRTM20XX_ATTR_HARDCOPY_PRINT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 458
#define RSRTM20XX_ATTR_HARDCOPY_SELECT_PRINTER                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 459
#define RSRTM20XX_ATTR_HARDCOPY_GET_FIRST_PRINTER                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 460
#define RSRTM20XX_ATTR_HARDCOPY_GET_NEXT_PRINTER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 461
#define RSRTM20XX_ATTR_ID_QUERY_RESPONSE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 304
#define RSRTM20XX_ATTR_SYSTEM_PRESET                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1690
#define RSRTM20XX_ATTR_LOGGING                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1691
#define RSRTM20XX_ATTR_CALIBRATION                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 305
#define RSRTM20XX_ATTR_SELF_TEST_QUERY                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 306
#define RSRTM20XX_ATTR_SYSTEM_ERROR                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 307
#define RSRTM20XX_ATTR_SYST_ERR_LIST                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 642
#define RSRTM20XX_ATTR_INSTRUMENT_NAME                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 568
#define RSRTM20XX_ATTR_RESET_STATUS_REGISTER                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 643
#define RSRTM20XX_ATTR_QUESTIONABLE_REGISTER                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 952
#define RSRTM20XX_ATTR_SELF_ALIGNMENT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1177
#define RSRTM20XX_ATTR_SELF_ALIGNMENT_STATE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1178
#define RSRTM20XX_ATTR_QUERY_OPC                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1565
#define RSRTM20XX_ATTR_PROCESS_ALL_PREVIOUS_COMMANDS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1567
#define RSRTM20XX_ATTR_SOUND_ENABLED                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 853
#define RSRTM20XX_ATTR_SOUND_ERROR_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 854
#define RSRTM20XX_ATTR_SOUND_TRIGGER_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 855
#define RSRTM20XX_ATTR_SOUND_GENERATE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 856
#define RSRTM20XX_ATTR_OPERATION_CONDITION                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 953
#define RSRTM20XX_ATTR_OPERATION_ENABLE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 954
#define RSRTM20XX_ATTR_OPERATION_N_TRANSITION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 955
#define RSRTM20XX_ATTR_OPERATION_P_TRANSITION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 956
#define RSRTM20XX_ATTR_OPERATION_EVENT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 957
#define RSRTM20XX_ATTR_QUESTIONABLE_CONDITION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 958
#define RSRTM20XX_ATTR_QUESTIONABLE_ENABLE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 959
#define RSRTM20XX_ATTR_QUESTIONABLE_N_TRANSITION                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 960
#define RSRTM20XX_ATTR_QUESTIONABLE_P_TRANSITION                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 961
#define RSRTM20XX_ATTR_QUESTIONABLE_EVENT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 962
#define RSRTM20XX_ATTR_QUESTIONABLE_CHANNEL_OVERLOAD_CONDITION              RS_SPECIFIC_PUBLIC_ATTR_BASE + 963
#define RSRTM20XX_ATTR_QUESTIONABLE_CHANNEL_OVERLOAD_ENABLE                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 964
#define RSRTM20XX_ATTR_QUESTIONABLE_CHANNEL_OVERLOAD_N_TRANSITION           RS_SPECIFIC_PUBLIC_ATTR_BASE + 965
#define RSRTM20XX_ATTR_QUESTIONABLE_CHANNEL_OVERLOAD_P_TRANSITION           RS_SPECIFIC_PUBLIC_ATTR_BASE + 966
#define RSRTM20XX_ATTR_QUESTIONABLE_CHANNEL_OVERLOAD_EVENT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 967
#define RSRTM20XX_ATTR_QUESTIONABLE_LIMIT_CONDITION                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 968
#define RSRTM20XX_ATTR_QUESTIONABLE_LIMIT_ENABLE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 969
#define RSRTM20XX_ATTR_QUESTIONABLE_LIMIT_N_TRANSITION                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 970
#define RSRTM20XX_ATTR_QUESTIONABLE_LIMIT_P_TRANSITION                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 971
#define RSRTM20XX_ATTR_QUESTIONABLE_LIMIT_EVENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 972
#define RSRTM20XX_ATTR_QUESTIONABLE_MASK_CONDITION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 973
#define RSRTM20XX_ATTR_QUESTIONABLE_MASK_ENABLE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 974
#define RSRTM20XX_ATTR_QUESTIONABLE_MASK_N_TRANSITION                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 975
#define RSRTM20XX_ATTR_QUESTIONABLE_MASK_P_TRANSITION                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 976
#define RSRTM20XX_ATTR_QUESTIONABLE_MASK_EVENT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 977

// rsrtm20xx_rngSampleMode
#define RSRTM20XX_VAL_REAL_TIME                                             0
#define RSRTM20XX_VAL_EQUIVALENT_TIME                                       1

// rsrtm20xx_rngWaveformArithmetics
#define RSRTM20XX_VAL_OFF                                                   0
#define RSRTM20XX_VAL_ENVELOPE                                              1
#define RSRTM20XX_VAL_AVERAGE                                               2
#define RSRTM20XX_VAL_SMOOTH                                                3
#define RSRTM20XX_VAL_FILTER                                                4

// rsrtm20xx_rngVerticalCoupling
#define RSRTM20XX_VAL_AC                                                    0
#define RSRTM20XX_VAL_DC                                                    1
#define RSRTM20XX_VAL_DC_LIMIT                                              2
#define RSRTM20XX_VAL_AC_LIMIT                                              3
#define RSRTM20XX_VAL_GND                                                   4

// rsrtm20xx_rngChannelBWLimit
#define RSRTM20XX_VAL_BW_FULL                                               0
#define RSRTM20XX_VAL_BW_400                                                1
#define RSRTM20XX_VAL_BW_200                                                2
#define RSRTM20XX_VAL_BW_20                                                 3

// rsrtm20xx_rngTriggerSource
#define RSRTM20XX_VAL_CHANNEL1                                              1
#define RSRTM20XX_VAL_CHANNEL2                                              2
#define RSRTM20XX_VAL_CHANNEL3                                              3
#define RSRTM20XX_VAL_CHANNEL4                                              4
#define RSRTM20XX_VAL_EXTERNAL                                              5
#define RSRTM20XX_VAL_LINE                                                  6
#define RSRTM20XX_VAL_SBUS1                                                 7
#define RSRTM20XX_VAL_SBUS2                                                 8
#define RSRTM20XX_VAL_SBUS3                                                 9
#define RSRTM20XX_VAL_SBUS4                                                 10
#define RSRTM20XX_VAL_D0                                                    11
#define RSRTM20XX_VAL_D1                                                    12
#define RSRTM20XX_VAL_D2                                                    13
#define RSRTM20XX_VAL_D3                                                    14
#define RSRTM20XX_VAL_D4                                                    15
#define RSRTM20XX_VAL_D5                                                    16
#define RSRTM20XX_VAL_D6                                                    17
#define RSRTM20XX_VAL_D7                                                    18
#define RSRTM20XX_VAL_D8                                                    19
#define RSRTM20XX_VAL_D9                                                    20
#define RSRTM20XX_VAL_D10                                                   21
#define RSRTM20XX_VAL_D11                                                   22
#define RSRTM20XX_VAL_D12                                                   23
#define RSRTM20XX_VAL_D13                                                   24
#define RSRTM20XX_VAL_D14                                                   25
#define RSRTM20XX_VAL_D15                                                   26

// rsrtm20xx_rngTriggerSlope
#define RSRTM20XX_VAL_POSITIVE                                              0
#define RSRTM20XX_VAL_NEGATIVE                                              1
#define RSRTM20XX_VAL_EITHER                                                2

// rsrtm20xx_rngPolarity
#define RSRTM20XX_VAL_POLARITY_NEGATIVE                                     0
#define RSRTM20XX_VAL_POLARITY_POSITIVE                                     1

// rsrtm20xx_rngWidthCondition
#define RSRTM20XX_VAL_SHORTER                                               0
#define RSRTM20XX_VAL_LONGER                                                1
#define RSRTM20XX_VAL_WITHIN                                                2
#define RSRTM20XX_VAL_OUTSIDE                                               3

// rsrtm20xx_rngTVTriggerEvent
#define RSRTM20XX_VAL_TV_EVENT_EVEN                                         0
#define RSRTM20XX_VAL_TV_EVENT_ODD                                          1
#define RSRTM20XX_VAL_TV_EVENT_ANY_FIELD                                    2
#define RSRTM20XX_VAL_TV_EVENT_LINE                                         3
#define RSRTM20XX_VAL_TV_EVENT_ANY_LINE                                     4

// rsrtm20xx_rngChannelPatternCombination
#define RSRTM20XX_VAL_AND                                                   0
#define RSRTM20XX_VAL_OR                                                    1

// rsrtm20xx_rngFalseTrue
#define RSRTM20XX_VAL_FALSE                                                 0
#define RSRTM20XX_VAL_TRUE                                                  1

// rsrtm20xx_rngCursorFunction
#define RSRTM20XX_VAL_CURSOR_HORIZONTAL                                     0
#define RSRTM20XX_VAL_CURSOR_VERTICAL                                       1
#define RSRTM20XX_VAL_CURSOR_PAIRED                                         2
#define RSRTM20XX_VAL_CURSOR_HRAT                                           3
#define RSRTM20XX_VAL_CURSOR_VRAT                                           4
#define RSRTM20XX_VAL_CURSOR_PPC                                            5
#define RSRTM20XX_VAL_CURSOR_NPC                                            6
#define RSRTM20XX_VAL_CURSOR_REC                                            7
#define RSRTM20XX_VAL_CURSOR_FEC                                            8
#define RSRTM20XX_VAL_CURSOR_MEAN                                           9
#define RSRTM20XX_VAL_CURSOR_RMS                                            10
#define RSRTM20XX_VAL_CURSOR_RTIM                                           11
#define RSRTM20XX_VAL_CURSOR_FTIM                                           12
#define RSRTM20XX_VAL_CURSOR_PEAK                                           13
#define RSRTM20XX_VAL_CURSOR_UPE                                            14
#define RSRTM20XX_VAL_CURSOR_LPE                                            15
#define RSRTM20XX_VAL_CURSOR_BWIDTH                                         16

// rsrtm20xx_rngXRatioMeasurementUnit
#define RSRTM20XX_VAL_UNIT_RATIO                                            0
#define RSRTM20XX_VAL_UNIT_PCT                                              1
#define RSRTM20XX_VAL_UNIT_DEGREE                                           2
#define RSRTM20XX_VAL_UNIT_RAD                                              3

// rsrtm20xx_rngRollModeAuto
#define RSRTM20XX_VAL_AUTO                                                  1

// rsrtm20xx_rngInterpolation
#define RSRTM20XX_VAL_SINE_X                                                0
#define RSRTM20XX_VAL_LINEAR                                                1
#define RSRTM20XX_VAL_SAMPLE_HOLD                                           2

// rsrtm20xx_rngMeasurementCategory
#define RSRTM20XX_VAL_AMPTIME                                               0

// rsrtm20xx_rngFFTWindowType
#define RSRTM20XX_VAL_RECTANGULAR                                           0
#define RSRTM20XX_VAL_HAMMING                                               1
#define RSRTM20XX_VAL_HANN                                                  2
#define RSRTM20XX_VAL_BLACKMAN                                              3
#define RSRTM20XX_VAL_FLAT                                                  4

// rsrtm20xx_rngProtocolI2CFrameState
#define RSRTM20XX_VAL_FRAME_STATE_OK                                        0
#define RSRTM20XX_VAL_FRAME_STATE_INCOMPLETE                                1
#define RSRTM20XX_VAL_FRAME_STATE_UNEXPSTOP                                 2
#define RSRTM20XX_VAL_FRAME_STATE_INSUFFICIENT                              3
#define RSRTM20XX_VAL_FRAME_STATE_ADDIFFERENT                               4

// rsrtm20xx_rngProtocolAckBit
#define RSRTM20XX_VAL_I2C_EITHER                                            0
#define RSRTM20XX_VAL_I2C_ACK                                               1
#define RSRTM20XX_VAL_I2C_NACK                                              2
#define RSRTM20XX_VAL_I2C_INCOMPLETE                                        3

// rsrtm20xx_rngProtocolSPIBitOrder
#define RSRTM20XX_VAL_SPI_MSBF                                              0
#define RSRTM20XX_VAL_SPI_LSBF                                              1

// rsrtm20xx_rngAudioSource
#define RSRTM20XX_VAL_AUDIO_SOURCE_CH1                                      1
#define RSRTM20XX_VAL_AUDIO_SOURCE_CH2                                      2
#define RSRTM20XX_VAL_AUDIO_SOURCE_CH3                                      3
#define RSRTM20XX_VAL_AUDIO_SOURCE_CH4                                      4
#define RSRTM20XX_VAL_AUDIO_SOURCE_D1                                       6
#define RSRTM20XX_VAL_AUDIO_SOURCE_D2                                       7
#define RSRTM20XX_VAL_AUDIO_SOURCE_D3                                       8
#define RSRTM20XX_VAL_AUDIO_SOURCE_D4                                       9
#define RSRTM20XX_VAL_AUDIO_SOURCE_D5                                       10
#define RSRTM20XX_VAL_AUDIO_SOURCE_D6                                       11
#define RSRTM20XX_VAL_AUDIO_SOURCE_D7                                       12
#define RSRTM20XX_VAL_AUDIO_SOURCE_D8                                       13
#define RSRTM20XX_VAL_AUDIO_SOURCE_D9                                       14
#define RSRTM20XX_VAL_AUDIO_SOURCE_D10                                      15
#define RSRTM20XX_VAL_AUDIO_SOURCE_D11                                      16
#define RSRTM20XX_VAL_AUDIO_SOURCE_D12                                      17
#define RSRTM20XX_VAL_AUDIO_SOURCE_D13                                      18
#define RSRTM20XX_VAL_AUDIO_SOURCE_D14                                      19
#define RSRTM20XX_VAL_AUDIO_SOURCE_D15                                      20
#define RSRTM20XX_VAL_AUDIO_SOURCE_D0                                       5

// rsrtm20xx_rngProtocolSPIPolarity
#define RSRTM20XX_VAL_ACTLOW                                                0
#define RSRTM20XX_VAL_ACTHIGH                                               1

// rsrtm20xx_rngProtocolSPIDataAlignment
#define RSRTM20XX_VAL_SPI_TRIGGER_BIT                                       0
#define RSRTM20XX_VAL_SPI_TRIGGER_WORD                                      1

// rsrtm20xx_rngProtocolSPIFrameState
#define RSRTM20XX_VAL_FRAME_STATE_INCFIRST                                  1
#define RSRTM20XX_VAL_FRAME_STATE_INCLAST                                   2

// rsrtm20xx_rngProtocolUARTParity
#define RSRTM20XX_VAL_UART_NONE                                             0
#define RSRTM20XX_VAL_UART_ODD                                              1
#define RSRTM20XX_VAL_UART_EVEN                                             2

// rsrtm20xx_rngProtocolUARTStopBits
#define RSRTM20XX_VAL_UART_B1                                               0
#define RSRTM20XX_VAL_UART_B1_5                                             1
#define RSRTM20XX_VAL_UART_B2                                               2

// rsrtm20xx_rngProtocolUARTTriggerType
#define RSRTM20XX_VAL_UART_TRIGGER_BURSTSTART                               0
#define RSRTM20XX_VAL_UART_TRIGGER_SBIT                                     1
#define RSRTM20XX_VAL_UART_TRIGGER_NTHS                                     2
#define RSRTM20XX_VAL_UART_TRIGGER_SYMB                                     3
#define RSRTM20XX_VAL_UART_TRIGGER_PATT                                     4
#define RSRTM20XX_VAL_UART_TRIGGER_PERR                                     5
#define RSRTM20XX_VAL_UART_TRIGGER_FERR                                     6
#define RSRTM20XX_VAL_UART_TRIGGER_BRE                                      7

// rsrtm20xx_rngSignalStyle
#define RSRTM20XX_VAL_SIGNAL_STYLE_VECTORS                                  0
#define RSRTM20XX_VAL_SIGNAL_STYLE_DOTS                                     1

// rsrtm20xx_rngHardcopyDeviceOrientation
#define RSRTM20XX_VAL_HARDCOPY_ORIENTATION_PORTRAIT                         0
#define RSRTM20XX_VAL_HARDCOPY_ORIENTATION_LANDSCAPE                        1

// rsrtm20xx_rngWaveformParameter
#define RSRTM20XX_VAL_WAV_CH1                                               1
#define RSRTM20XX_VAL_WAV_CH2                                               2
#define RSRTM20XX_VAL_WAV_CH3                                               3
#define RSRTM20XX_VAL_WAV_CH4                                               4
#define RSRTM20XX_VAL_WAV_M1                                                5
#define RSRTM20XX_VAL_WAV_M2                                                6
#define RSRTM20XX_VAL_WAV_M3                                                7
#define RSRTM20XX_VAL_WAV_M4                                                8
#define RSRTM20XX_VAL_WAV_M5                                                9
#define RSRTM20XX_VAL_WAV_R1                                                10
#define RSRTM20XX_VAL_WAV_R2                                                11
#define RSRTM20XX_VAL_WAV_R3                                                12
#define RSRTM20XX_VAL_WAV_R4                                                13
#define RSRTM20XX_VAL_WAV_D0                                                14
#define RSRTM20XX_VAL_WAV_D1                                                15
#define RSRTM20XX_VAL_WAV_D2                                                16
#define RSRTM20XX_VAL_WAV_D3                                                17
#define RSRTM20XX_VAL_WAV_D4                                                18
#define RSRTM20XX_VAL_WAV_D5                                                19
#define RSRTM20XX_VAL_WAV_D6                                                20
#define RSRTM20XX_VAL_WAV_D7                                                21
#define RSRTM20XX_VAL_WAV_D8                                                22
#define RSRTM20XX_VAL_WAV_D9                                                23
#define RSRTM20XX_VAL_WAV_D10                                               24
#define RSRTM20XX_VAL_WAV_D11                                               25
#define RSRTM20XX_VAL_WAV_D12                                               26
#define RSRTM20XX_VAL_WAV_D13                                               27
#define RSRTM20XX_VAL_WAV_D14                                               28
#define RSRTM20XX_VAL_WAV_D15                                               29
#define RSRTM20XX_VAL_WAV_D70                                               30
#define RSRTM20XX_VAL_WAV_D158                                              31
#define RSRTM20XX_VAL_WAV_SPEC                                              32
#define RSRTM20XX_VAL_WAV_XY2                                               33
#define RSRTM20XX_VAL_WAV_MINH                                              34
#define RSRTM20XX_VAL_WAV_MAXH                                              35
#define RSRTM20XX_VAL_WAV_AVER                                              36

// rsrtm20xx_rngProtocolI2CAddressAccessBit
#define RSRTM20XX_VAL_I2C_ACCESS_EITHER                                     0
#define RSRTM20XX_VAL_I2C_ACCESS_ACK                                        1
#define RSRTM20XX_VAL_I2C_ACCESS_NACK                                       2
#define RSRTM20XX_VAL_I2C_ACCESS_INCOMPLETE                                 3

// rsrtm20xx_rngSampleRate
#define RSRTM20XX_VAL_RATE_AUTO                                             0
#define RSRTM20XX_VAL_RATE_MAX_SAM                                          2
#define RSRTM20XX_VAL_RATE_MAX_WAV                                          1

// rsrtm20xx_rngColorBrightness
#define RSRTM20XX_VAL_COLOR_NORM                                            0
#define RSRTM20XX_VAL_COLOR_INVERSE                                         1
#define RSRTM20XX_VAL_COLOR_FCOLOR                                          2
#define RSRTM20XX_VAL_COLOR_INVCOLOR                                        3

// rsrtm20xx_rngDiagramMode
#define RSRTM20XX_VAL_DISPLAY_YT                                            0
#define RSRTM20XX_VAL_DISPLAY_XY                                            1

// rsrtm20xx_rngLanguage
#define RSRTM20XX_VAL_DISPLAY_ENGLISH                                       0
#define RSRTM20XX_VAL_DISPLAY_GERMAN                                        1
#define RSRTM20XX_VAL_DISPLAY_FRENCH                                        2
#define RSRTM20XX_VAL_DISPLAY_SPANISH                                       3
#define RSRTM20XX_VAL_DISPLAY_RUSSIAN                                       4
#define RSRTM20XX_VAL_DISPLAY_SCHIN                                         5
#define RSRTM20XX_VAL_DISPLAY_TCHIN                                         6
#define RSRTM20XX_VAL_DISPLAY_JAPAN                                         7
#define RSRTM20XX_VAL_DISPLAY_KOREAN                                        8

// rsrtm20xx_rngGridStyle
#define RSRTM20XX_VAL_GRID_LINES                                            0
#define RSRTM20XX_VAL_GRID_CROSS                                            1
#define RSRTM20XX_VAL_GRID_NONE                                             2

// rsrtm20xx_rngSource
#define RSRTM20XX_VAL_SOURCE_CH1                                            1
#define RSRTM20XX_VAL_SOURCE_CH2                                            2
#define RSRTM20XX_VAL_SOURCE_CH3                                            3
#define RSRTM20XX_VAL_SOURCE_CH4                                            4

// rsrtm20xx_rngXY2Source
#define RSRTM20XX_VAL_SOURCE_NONE                                           0

// rsrtm20xx_rngIntensityMode
#define RSRTM20XX_VAL_ZMODE_ANALOG                                          0
#define RSRTM20XX_VAL_ZMODE_DIGITAL                                         1
#define RSRTM20XX_VAL_ZMODE_OFF                                             2

// rsrtm20xx_rngHardcopyDevice
#define RSRTM20XX_VAL_HARDCOPY_DEVICE_MEM                                   0
#define RSRTM20XX_VAL_HARDCOPY_DEVICE_PRINT                                 1

// rsrtm20xx_rngHardcopyDeviceLang
#define RSRTM20XX_VAL_HARDCOPY_DEVICE_LANG_BMP                              1
#define RSRTM20XX_VAL_HARDCOPY_DEVICE_LANG_PNG                              0

// rsrtm20xx_rngPageSize
#define RSRTM20XX_VAL_PAGE_A4                                               0
#define RSRTM20XX_VAL_PAGE_A5                                               1
#define RSRTM20XX_VAL_PAGE_B5                                               2
#define RSRTM20XX_VAL_PAGE_B6                                               3
#define RSRTM20XX_VAL_PAGE_EX                                               4

// rsrtm20xx_rngMeasResultRTM
#define RSRTM20XX_VAL_MEAS_RES_FREQ                                         0
#define RSRTM20XX_VAL_MEAS_RES_PERIOD                                       1
#define RSRTM20XX_VAL_MEAS_RES_PEAK                                         2
#define RSRTM20XX_VAL_MEAS_RES_UPEAK                                        3
#define RSRTM20XX_VAL_MEAS_RES_LPEAK                                        4
#define RSRTM20XX_VAL_MEAS_RES_PPC                                          5
#define RSRTM20XX_VAL_MEAS_RES_NPC                                          6
#define RSRTM20XX_VAL_MEAS_RES_REC                                          7
#define RSRTM20XX_VAL_MEAS_RES_HIGH                                         8
#define RSRTM20XX_VAL_MEAS_RES_LOW                                          9
#define RSRTM20XX_VAL_MEAS_RES_AMP                                          10
#define RSRTM20XX_VAL_MEAS_RES_MEAN                                         11
#define RSRTM20XX_VAL_MEAS_RES_RMS                                          12
#define RSRTM20XX_VAL_MEAS_RES_RTM                                          13
#define RSRTM20XX_VAL_MEAS_RES_FTIM                                         14
#define RSRTM20XX_VAL_MEAS_RES_PDCY                                         15
#define RSRTM20XX_VAL_MEAS_RES_NDCY                                         16
#define RSRTM20XX_VAL_MEAS_RES_PPW                                          17
#define RSRTM20XX_VAL_MEAS_RES_NPW                                          18
#define RSRTM20XX_VAL_MEAS_RES_CYCM                                         19
#define RSRTM20XX_VAL_MEAS_RES_CYCR                                         20
#define RSRTM20XX_VAL_MEAS_RES_SSTDD                                        21
#define RSRTM20XX_VAL_MEAS_RES_TFR                                          22
#define RSRTM20XX_VAL_MEAS_RES_TPER                                         23

// rsrtm20xx_rngChannelRTM
#define RSRTM20XX_VAL_CHANNELRTM_1                                          1
#define RSRTM20XX_VAL_CHANNELRTM_2                                          2
#define RSRTM20XX_VAL_CHANNELRTM_3                                          3
#define RSRTM20XX_VAL_CHANNELRTM_4                                          4

// rsrtm20xx_rngMaskCondition
#define RSRTM20XX_VAL_MASKCOND_RUN                                          0
#define RSRTM20XX_VAL_MASKCOND_STOP                                         1
#define RSRTM20XX_VAL_MASKCOND_PAUSE                                        2

// rsrtm20xx_rngProbeButtonAction
#define RSRTM20XX_VAL_PROBE_BUTTON_NO_ACTION                                0
#define RSRTM20XX_VAL_PROBE_BUTTON_RUN_CONTINUOUS                           1
#define RSRTM20XX_VAL_PROBE_BUTTON_RUN_SINGLE                               2
#define RSRTM20XX_VAL_PROBE_BUTTON_AUTOSET                                  3

// rsrtm20xx_rngTriggerCoupling
#define RSRTM20XX_VAL_COUPLING_DIRECT                                       0
#define RSRTM20XX_VAL_COUPLING_ALTERNATING                                  1
#define RSRTM20XX_VAL_COUPLING_HIGHFREQ                                     2

// rsrtm20xx_rngProtocolI2CAccessBit
#define RSRTM20XX_VAL_I2C_TRIGGER_READ                                      1
#define RSRTM20XX_VAL_I2C_TRIGGER_WRITE                                     2

// rsrtm20xx_rngAddressLength
#define RSRTM20XX_VAL_LENGTH_NORMAL                                         0
#define RSRTM20XX_VAL_LENGTH_EXTENDED                                       1

// rsrtm20xx_rngProtocolI2CTriggerType
#define RSRTM20XX_VAL_TRIGGER_MODE_START                                    0
#define RSRTM20XX_VAL_TRIGGER_MODE_RESTART                                  1
#define RSRTM20XX_VAL_TRIGGER_MODE_STOP                                     2
#define RSRTM20XX_VAL_TRIGGER_MODE_MISSING                                  3
#define RSRTM20XX_VAL_TRIGGER_MODE_PATT                                     4

// rsrtm20xx_rngHysteresisMode
#define RSRTM20XX_VAL_HYSTERESIS_AUTO                                       0
#define RSRTM20XX_VAL_HYSTERESIS_SMALL                                      1
#define RSRTM20XX_VAL_HYSTERESIS_MEDIUM                                     2
#define RSRTM20XX_VAL_HYSTERESIS_LARGE                                      3
#define RSRTM20XX_VAL_HYSTERESIS_MANUAL                                     4

// rsrtm20xx_rngTriggerMode
#define RSRTM20XX_VAL_TRIGGER_AUTO                                          0
#define RSRTM20XX_VAL_TRIGGER_NORMAL                                        1
#define RSRTM20XX_VAL_TRIGGER_DELAY                                         2
#define RSRTM20XX_VAL_TRIGGER_EVENTS                                        3

// rsrtm20xx_rngTriggerPatternFunction
#define RSRTM20XX_VAL_TRG_PATTERN_AND                                       0
#define RSRTM20XX_VAL_TRG_PATTERN_OR                                        1

// rsrtm20xx_rngProtocolSPITriggerType
#define RSRTM20XX_VAL_SPI_MODE_BURSTSTART                                   0
#define RSRTM20XX_VAL_SPI_MODE_BURSTEND                                     1
#define RSRTM20XX_VAL_SPI_MODE_BIT                                          2
#define RSRTM20XX_VAL_SPI_MODE_PATTERN                                      3

// rsrtm20xx_rngTriggerType
#define RSRTM20XX_VAL_EDGE_TRIGGER                                          0
#define RSRTM20XX_VAL_WIDTH_TRIGGER                                         1
#define RSRTM20XX_VAL_VIDEO_TRIGGER                                         2
#define RSRTM20XX_VAL_PATTERN_TRIGGER                                       3
#define RSRTM20XX_VAL_BUS_TRIGGER                                           4
#define RSRTM20XX_VAL_RISE_TIME_TRIGGER                                     5
#define RSRTM20XX_VAL_RUNT_TRIGGER                                          6

// rsrtm20xx_rngTVStandard
#define RSRTM20XX_VAL_TV_STANDART_PAL                                       0
#define RSRTM20XX_VAL_TV_STANDART_NTSC                                      1
#define RSRTM20XX_VAL_TV_STANDART_SECAM                                     2
#define RSRTM20XX_VAL_TV_STANDART_PALM                                      3
#define RSRTM20XX_VAL_TV_STANDART_I576                                      4
#define RSRTM20XX_VAL_TV_STANDART_P720                                      5
#define RSRTM20XX_VAL_TV_STANDART_P1080                                     6
#define RSRTM20XX_VAL_TV_STANDART_I1080                                     7

// rsrtm20xx_rngProtocolUARTTriggerMode
#define RSRTM20XX_VAL_UART_MODE_RTM_BURSTSTART                              0
#define RSRTM20XX_VAL_UART_MODE_RTM_SBIT                                    1
#define RSRTM20XX_VAL_UART_MODE_RTM_NTHS                                    2
#define RSRTM20XX_VAL_UART_MODE_RTM_SYMB                                    3
#define RSRTM20XX_VAL_UART_MODE_RTM_PATT                                    4
#define RSRTM20XX_VAL_UART_MODE_RTM_PERR                                    5
#define RSRTM20XX_VAL_UART_MODE_RTM_FERR                                    6
#define RSRTM20XX_VAL_UART_MODE_RTM_BRE                                     7

// rsrtm20xx_rngColorScheme
#define RSRTM20XX_VAL_SCHEME_COLOR                                          0
#define RSRTM20XX_VAL_SCHEME_GRAY                                           1
#define RSRTM20XX_VAL_SCHEME_INVERTED                                       2

// rsrtm20xx_rngAcquisitionType
#define RSRTM20XX_VAL_SAMPLE                                                0
#define RSRTM20XX_VAL_PEAK_DETECT                                           1
#define RSRTM20XX_VAL_HI_RES                                                2

// rsrtm20xx_rngAmplitudeTimeMeasurementType
#define RSRTM20XX_VAL_MEAS_FREQUENCY                                        1
#define RSRTM20XX_VAL_MEAS_PERIOD                                           2
#define RSRTM20XX_VAL_MEAS_PEAK                                             3
#define RSRTM20XX_VAL_MEAS_UPEAK                                            4
#define RSRTM20XX_VAL_MEAS_LPEAK                                            5
#define RSRTM20XX_VAL_MEAS_PPC                                              6
#define RSRTM20XX_VAL_MEAS_NPC                                              7
#define RSRTM20XX_VAL_MEAS_REC                                              8
#define RSRTM20XX_VAL_MEAS_FEC                                              9
#define RSRTM20XX_VAL_MEAS_HIGH                                             10
#define RSRTM20XX_VAL_MEAS_LOW                                              11
#define RSRTM20XX_VAL_MEAS_AMP                                              12
#define RSRTM20XX_VAL_MEAS_MEAN                                             13
#define RSRTM20XX_VAL_MEAS_RMS                                              14
#define RSRTM20XX_VAL_MEAS_RTM                                              15
#define RSRTM20XX_VAL_MEAS_FTIM                                             16
#define RSRTM20XX_VAL_MEAS_PDCY                                             17
#define RSRTM20XX_VAL_MEAS_NDCY                                             18
#define RSRTM20XX_VAL_MEAS_PPW                                              19
#define RSRTM20XX_VAL_MEAS_NPW                                              20
#define RSRTM20XX_VAL_MEAS_CYCM                                             21
#define RSRTM20XX_VAL_MEAS_CYCR                                             22
#define RSRTM20XX_VAL_MEAS_SSTDD                                            23
#define RSRTM20XX_VAL_MEAS_TFR                                              24
#define RSRTM20XX_VAL_MEAS_TPER                                             25
#define RSRTM20XX_VAL_MEAS_DEL                                              26
#define RSRTM20XX_VAL_MEAS_PHA                                              27
#define RSRTM20XX_VAL_MEAS_BWID                                             28
#define RSRTM20XX_VAL_MEAS_CYCS                                             29
#define RSRTM20XX_VAL_MEAS_POV                                              30
#define RSRTM20XX_VAL_MEAS_NOV                                              31
#define RSRTM20XX_VAL_MEAS_TBFR                                             32
#define RSRTM20XX_VAL_MEAS_TBP                                              33

// rsrtm20xx_rngHardcopyLang
#define RSRTM20XX_VAL_HARDCOPY_LANG_GDI                                     0
#define RSRTM20XX_VAL_HARDCOPY_LANG_BMP                                     1
#define RSRTM20XX_VAL_HARDCOPY_LANG_PNG                                     2

// rsrtm20xx_rngProtocolType
#define RSRTM20XX_VAL_PTYPE_RTM_PARALLEL                                    0
#define RSRTM20XX_VAL_PTYPE_RTM_CPARALLEL                                   1
#define RSRTM20XX_VAL_PTYPE_RTM_I2C                                         2
#define RSRTM20XX_VAL_PTYPE_RTM_SPI                                         3
#define RSRTM20XX_VAL_PTYPE_RTM_SSPI                                        4
#define RSRTM20XX_VAL_PTYPE_RTM_UART                                        5
#define RSRTM20XX_VAL_PTYPE_RTM_CAN                                         6
#define RSRTM20XX_VAL_PTYPE_RTM_LIN                                         7
#define RSRTM20XX_VAL_PTYPE_RTM_AUDIO                                       8
#define RSRTM20XX_VAL_PTYPE_RTM_MILSTD                                      9
#define RSRTM20XX_VAL_PTYPE_RTM_ARINC                                       10

// rsrtm20xx_rngProtocolFormat
#define RSRTM20XX_VAL_PFORMAT_ASCI                                          0
#define RSRTM20XX_VAL_PFORMAT_HEX                                           1
#define RSRTM20XX_VAL_PFORMAT_BIN                                           2
#define RSRTM20XX_VAL_PFORMAT_DEC                                           3
#define RSRTM20XX_VAL_PFORMAT_OCT                                           4

// rsrtm20xx_rngProtocolHeight
#define RSRTM20XX_VAL_PHEIGHT_SMALL                                         0
#define RSRTM20XX_VAL_PHEIGHT_MEDIUM                                        1
#define RSRTM20XX_VAL_PHEIGHT_LARGE                                         2
#define RSRTM20XX_VAL_PHEIGHT_DIV1                                          3
#define RSRTM20XX_VAL_PHEIGHT_DIV2                                          4
#define RSRTM20XX_VAL_PHEIGHT_DIV4                                          5
#define RSRTM20XX_VAL_PHEIGHT_DIV8                                          6

// rsrtm20xx_rngRelativeMode
#define RSRTM20XX_VAL_MEAS_RELATIVE_TEN                                     0
#define RSRTM20XX_VAL_MEAS_RELATIVE_TWENTY                                  1
#define RSRTM20XX_VAL_MEAS_RELATIVE_FIVE                                    2
#define RSRTM20XX_VAL_MEAS_RELATIVE_USER                                    3

// rsrtm20xx_rngChannelPolarity
#define RSRTM20XX_VAL_POLARITY_NORM                                         0
#define RSRTM20XX_VAL_POLARITY_INV                                          1

// rsrtm20xx_rngExternalTriggerCoupling
#define RSRTM20XX_VAL_COUPLING_AC                                           0
#define RSRTM20XX_VAL_COUPLING_DC                                           1

// rsrtm20xx_rngMeasCategory
#define RSRTM20XX_VAL_MEAS_AMPT                                             0

// rsrtm20xx_rngSearchCondition
#define RSRTM20XX_VAL_SEARCHCOND_EDGE                                       0
#define RSRTM20XX_VAL_SEARCHCOND_WIDTH                                      1
#define RSRTM20XX_VAL_SEARCHCOND_PEAK                                       2
#define RSRTM20XX_VAL_SEARCHCOND_RUNT                                       3
#define RSRTM20XX_VAL_SEARCHCOND_RTIME                                      4
#define RSRTM20XX_VAL_SEARCHCOND_D2C                                        5
#define RSRTM20XX_VAL_SEARCHCOND_PATTERN                                    6
#define RSRTM20XX_VAL_SEARCHCOND_PROTOCOL                                   7

// rsrtm20xx_rngSearchSource
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_C1                                 0
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_C2                                 1
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_C3                                 2
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_C4                                 3
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_MA1                                4
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_MA2                                5
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_MA3                                6
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_MA4                                7
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_RE1                                8
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_RE2                                9
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_RE3                                10
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_RE4                                11
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_SBUS1                              12
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_SBUS2                              13
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_SBUS3                              14
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_SBUS4                              15

// rsrtm20xx_rngScalingUnit
#define RSRTM20XX_VAL_FFT_LIN                                               0
#define RSRTM20XX_VAL_FFT_DBM                                               1
#define RSRTM20XX_VAL_FFT_DBV                                               2

// rsrtm20xx_rngProbeUnit
#define RSRTM20XX_VAL_PROBE_UNIT_V                                          0
#define RSRTM20XX_VAL_PROBE_UNIT_A                                          1

// rsrtm20xx_rngWaveformSamplesRange
#define RSRTM20XX_VAL_WAVEFORM_SAMPLES_DEFAULT                              0
#define RSRTM20XX_VAL_WAVEFORM_SAMPLES_MAXIMUM                              1
#define RSRTM20XX_VAL_WAVEFORM_SAMPLES_DISPLAY_MAXIMUM                      2

// rsrtm20xx_rngMaskTestActionExecution
#define RSRTM20XX_VAL_MASK_ACTION_EXECUTION_OFF                             0
#define RSRTM20XX_VAL_MASK_ACTION_EXECUTION_EACH                            1
#define RSRTM20XX_VAL_MASK_ACTION_EXECUTION_SINGLE                          2
#define RSRTM20XX_VAL_MASK_ACTION_EXECUTION_CYCLIC                          3

// rsrtm20xx_rngSearchTriggerPatternCombination
#define RSRTM20XX_VAL_PATTERN_COMBINATION_AND                               0
#define RSRTM20XX_VAL_PATTERN_COMBINATION_OR                                1
#define RSRTM20XX_VAL_PATTERN_COMBINATION_NAND                              2
#define RSRTM20XX_VAL_PATTERN_COMBINATION_NOR                               3

// rsrtm20xx_rngSearchGateMode
#define RSRTM20XX_VAL_SEARCH_GATE_ALL                                       0
#define RSRTM20XX_VAL_SEARCH_GATE_DISPLAY                                   1
#define RSRTM20XX_VAL_SEARCH_GATE_USER                                      2

// rsrtm20xx_rngProtocolSPITriggerSource
#define RSRTM20XX_VAL_SPI_MISO                                              3
#define RSRTM20XX_VAL_SPI_MOSI                                              4

// rsrtm20xx_rngProtocolUARTTriggerSource
#define RSRTM20XX_VAL_UART_RX                                               0
#define RSRTM20XX_VAL_UART_TX                                               1

// rsrtm20xx_rngProtocolLINStandard
#define RSRTM20XX_VAL_LIN_STANDARD_AUTO                                     0
#define RSRTM20XX_VAL_LIN_STANDARD_1X                                       1
#define RSRTM20XX_VAL_LIN_STANDARD_2X                                       2
#define RSRTM20XX_VAL_LIN_STANDARD_J2602                                    3

// rsrtm20xx_rngProtocolLINTriggerMode
#define RSRTM20XX_VAL_LIN_FRAME_START                                       0
#define RSRTM20XX_VAL_LIN_FRAME_ID                                          1
#define RSRTM20XX_VAL_LIN_FRAME_ID_DATA                                     2
#define RSRTM20XX_VAL_LIN_FRAME_WAKEUP                                      3
#define RSRTM20XX_VAL_LIN_ERR_COND                                          4

// rsrtm20xx_rngProtocolCondition
#define RSRTM20XX_VAL_COND_NQ                                               0
#define RSRTM20XX_VAL_COND_EQ                                               1
#define RSRTM20XX_VAL_COND_LT                                               2
#define RSRTM20XX_VAL_COND_GT                                               3

// rsrtm20xx_rngProtocolSearchEvent
#define RSRTM20XX_VAL_SEARCH_EVENT_FRAME                                    0
#define RSRTM20XX_VAL_SEARCH_EVENT_ERROR                                    1
#define RSRTM20XX_VAL_SEARCH_EVENT_ID                                       2
#define RSRTM20XX_VAL_SEARCH_EVENT_IDDT                                     3
#define RSRTM20XX_VAL_SEARCH_EVENT_IDERR                                    4

// rsrtm20xx_rngProtocolLINSearchFrameSetup
#define RSRTM20XX_VAL_LIN_SEARCH_FRAME_SOF                                  0
#define RSRTM20XX_VAL_LIN_SEARCH_FRAME_WAKEUP                               1

// rsrtm20xx_rngProtocolCANLineType
#define RSRTM20XX_VAL_CAN_LOW                                               0
#define RSRTM20XX_VAL_CAN_HIGH                                              1

// rsrtm20xx_rngProtocolCANTriggerMode
#define RSRTM20XX_VAL_CAN_FRAME_START                                       0
#define RSRTM20XX_VAL_CAN_FRAME_END                                         1
#define RSRTM20XX_VAL_CAN_MSG_ID                                            2
#define RSRTM20XX_VAL_CAN_MSG_IDDT                                          3
#define RSRTM20XX_VAL_CAN_FRAME_TYPE                                        4
#define RSRTM20XX_VAL_CAN_ERROR_COND                                        5

// rsrtm20xx_rngProtocolCANTriggerFrameType
#define RSRTM20XX_VAL_CAN_FRAME_TYPE_DATA                                   0
#define RSRTM20XX_VAL_CAN_FRAME_TYPE_REMOTE                                 1
#define RSRTM20XX_VAL_CAN_FRAME_TYPE_ERROR                                  2
#define RSRTM20XX_VAL_CAN_FRAME_TYPE_OVER_LOAD                              3
#define RSRTM20XX_VAL_CAN_FRAME_TYPE_ANY                                    4

// rsrtm20xx_rngProtocolCANIDType
#define RSRTM20XX_VAL_CAN_ID_TYPE_B11                                       0
#define RSRTM20XX_VAL_CAN_ID_TYPE_B29                                       1
#define RSRTM20XX_VAL_CAN_ID_TYPE_ANY                                       2

// rsrtm20xx_rngProtocolCANSearchFrameSetup
#define RSRTM20XX_VAL_CAN_SEARCH_FRAME_SOF                                  0
#define RSRTM20XX_VAL_CAN_SEARCH_FRAME_EOF                                  1
#define RSRTM20XX_VAL_CAN_SEARCH_FRAME_OVER                                 2
#define RSRTM20XX_VAL_CAN_SEARCH_FRAME_ERR                                  3
#define RSRTM20XX_VAL_CAN_SEARCH_FRAME_DTA11                                4
#define RSRTM20XX_VAL_CAN_SEARCH_FRAME_DTA29                                5
#define RSRTM20XX_VAL_CAN_SEARCH_FRAME_REM11                                6
#define RSRTM20XX_VAL_CAN_SEARCH_FRAME_REM29                                7

// rsrtm20xx_rngProtocolCANSearchFrameType
#define RSRTM20XX_VAL_CAN_SEARCH_FRAME_TYPE_DATA                            0
#define RSRTM20XX_VAL_CAN_SEARCH_FRAME_TYPE_REMOTE                          1
#define RSRTM20XX_VAL_CAN_SEARCH_FRAME_TYPE_ANY                             2

// rsrtm20xx_rngProtocolLINPolarity
#define RSRTM20XX_VAL_LIN_ACTLOW                                            0
#define RSRTM20XX_VAL_LIN_ACTHIGH                                           1

// rsrtm20xx_rngDigitalHysteresis
#define RSRTM20XX_VAL_DIGITAL_HYSTERESIS_MAX                                0
#define RSRTM20XX_VAL_DIGITAL_HYSTERESIS_ROB                                1
#define RSRTM20XX_VAL_DIGITAL_HYSTERESIS_NORM                               2

// rsrtm20xx_rngTriggerHoldoffMode
#define RSRTM20XX_VAL_HOLDOFF_MODE_TIME                                     0
#define RSRTM20XX_VAL_HOLDOFF_MODE_OFF                                      1

// rsrtm20xx_rngTriggerPatternMode
#define RSRTM20XX_VAL_PATTERN_MODE_OFF                                      0
#define RSRTM20XX_VAL_PATTERN_MODE_TIMEOUT                                  1
#define RSRTM20XX_VAL_PATTERN_MODE_WIDTH                                    2

// rsrtm20xx_rngTriggerOutMode
#define RSRTM20XX_VAL_TRIGGER_OUT_MODE_OFF                                  0
#define RSRTM20XX_VAL_TRIGGER_OUT_MODE_TRIG                                 1
#define RSRTM20XX_VAL_TRIGGER_OUT_MODE_MASK                                 2

// rsrtm20xx_rngDigitalTechnology
#define RSRTM20XX_VAL_DIGITAL_TECHNOLOGY_TTL                                0
#define RSRTM20XX_VAL_DIGITAL_TECHNOLOGY_ECL                                1
#define RSRTM20XX_VAL_DIGITAL_TECHNOLOGY_CMOS                               2
#define RSRTM20XX_VAL_DIGITAL_TECHNOLOGY_MANUAL                             3

// rsrtm20xx_rngParallelBusFrameEnabled
#define RSRTM20XX_PARALLEL_BUS_ENABLED_OK                                   0
#define RSRTM20XX_PARALLEL_BUS_ENABLED_ERROR                                1
#define RSRTM20XX_PARALLEL_BUS_ENABLED_INS                                  2

// rsrtm20xx_rngI2CFrameTransferDirection
#define RSRTM20XX_VAL_TRANSFER_DIRECTION_INC                                0
#define RSRTM20XX_VAL_TRANSFER_DIRECTION_READ                               1
#define RSRTM20XX_VAL_TRANSFER_DIRECTION_WRITE                              2
#define RSRTM20XX_VAL_TRANSFER_DIRECTION_EITH                               3
#define RSRTM20XX_VAL_TRANSFER_DIRECTION_UNDF                               4

// rsrtm20xx_rngFrameAddressLength
#define RSRTM20XX_VAL_FRAME_ADDRESS_LENGTH_BIT7                             0
#define RSRTM20XX_VAL_FRAME_ADDRESS_LENGTH_BIT10                            1

// rsrtm20xx_rngLINFrameStatus
#define RSRTM20XX_VAL_LIN_FRAME_STATUS_OK                                   0
#define RSRTM20XX_VAL_LIN_FRAME_STATUS_UART                                 1
#define RSRTM20XX_VAL_LIN_FRAME_STATUS_CHCK                                 2
#define RSRTM20XX_VAL_LIN_FRAME_STATUS_PRER                                 3
#define RSRTM20XX_VAL_LIN_FRAME_STATUS_SYER                                 4
#define RSRTM20XX_VAL_LIN_FRAME_STATUS_WAK                                  5
#define RSRTM20XX_VAL_LIN_FRAME_STATUS_INS                                  6
#define RSRTM20XX_VAL_LIN_FRAME_STATUS_ERR                                  7
#define RSRTM20XX_VAL_LIN_FRAME_STATUS_LEN                                  8

// rsrtm20xx_rngLINChecksum
#define RSRTM20XX_VAL_LIN_CHECKSUM_OK                                       0
#define RSRTM20XX_VAL_LIN_CHECKSUM_ERR                                      1
#define RSRTM20XX_VAL_LIN_CHECKSUM_UNDF                                     2

// rsrtm20xx_rngLINParity
#define RSRTM20XX_VAL_LIN_PARITY_OK                                         0
#define RSRTM20XX_VAL_LIN_PARITY_PRER                                       1
#define RSRTM20XX_VAL_LIN_PARITY_UVAL                                       2
#define RSRTM20XX_VAL_LIN_PARITY_INS                                        3

// rsrtm20xx_rngLINFrameVersion
#define RSRTM20XX_VAL_FRAME_VERSION_V1X                                     0
#define RSRTM20XX_VAL_FRAME_VERSION_V2X                                     1
#define RSRTM20XX_VAL_FRAME_VERSION_UNK                                     2

// rsrtm20xx_rngLINByteEnabled
#define RSRTM20XX_VAL_LIN_BYTE_ENABLED_OK                                   0
#define RSRTM20XX_VAL_LIN_BYTE_ENABLED_INS                                  1
#define RSRTM20XX_VAL_LIN_BYTE_ENABLED_UART                                 2

// rsrtm20xx_rngUARTWordEnabled
#define RSRTM20XX_VAL_UART_STATE_OK                                         0
#define RSRTM20XX_VAL_UART_STATE_FRST                                       1
#define RSRTM20XX_VAL_UART_STATE_FREN                                       2
#define RSRTM20XX_VAL_UART_STATE_FRME                                       3
#define RSRTM20XX_VAL_UART_STATE_STER                                       4
#define RSRTM20XX_VAL_UART_STATE_SPER                                       5
#define RSRTM20XX_VAL_UART_STATE_PRER                                       6
#define RSRTM20XX_VAL_UART_STATE_INS                                        7
#define RSRTM20XX_VAL_UART_STATE_BRE                                        8

// rsrtm20xx_rngCANFrameType
#define RSRTM20XX_VAL_CAN_FRAME_TYPE_REM                                    1
#define RSRTM20XX_VAL_CAN_FRAME_TYPE_ERR                                    2
#define RSRTM20XX_VAL_CAN_FRAME_TYPE_OVLD                                   3

// rsrtm20xx_rngCANFrameStatus
#define RSRTM20XX_VAL_CAN_FRAME_STATUS_OK                                   0
#define RSRTM20XX_VAL_CAN_FRAME_STATUS_BTST                                 1
#define RSRTM20XX_VAL_CAN_FRAME_STATUS_CRCD                                 2
#define RSRTM20XX_VAL_CAN_FRAME_STATUS_ACKD                                 3
#define RSRTM20XX_VAL_CAN_FRAME_STATUS_CRC                                  4
#define RSRTM20XX_VAL_CAN_FRAME_STATUS_EOFD                                 5
#define RSRTM20XX_VAL_CAN_FRAME_STATUS_NOAC                                 6
#define RSRTM20XX_VAL_CAN_FRAME_STATUS_INS                                  7

// rsrtm20xx_rngCANAckState
#define RSRTM20XX_VAL_CAN_ACK_OK                                            0
#define RSRTM20XX_VAL_CAN_ACK_UNDF                                          1

// rsrtm20xx_rngQuestionableRegister
#define RSRTM20XX_VAL_QREGISTER_None                                        0
#define RSRTM20XX_VAL_QREGISTER_CHANNEL_OVERLOAD                            1
#define RSRTM20XX_VAL_QREGISTER_LIMIT                                       2
#define RSRTM20XX_VAL_QREGISTER_MASK                                        3

// rsrtm20xx_rngRegisterPart
#define RSRTM20XX_VAL_REGISTER_PART_CONDITION                               0
#define RSRTM20XX_VAL_REGISTER_PART_ENABLE                                  1
#define RSRTM20XX_VAL_REGISTER_PART_EVENT                                   2
#define RSRTM20XX_VAL_REGISTER_PART_NTRANSITION                             3
#define RSRTM20XX_VAL_REGISTER_PART_PTRANSITION                             4

// rsrtm20xx_rngMeasurementSource
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_NONE                               0
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_CH1                                1
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_CH2                                2
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_CH3                                3
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_CH4                                4
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_M1                                 5
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_M2                                 6
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_M3                                 7
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_M4                                 8
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_M5                                 9
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_R1                                 10
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_R2                                 11
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_R3                                 12
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_R4                                 13
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_TRIG                               14
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D1                                 15
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D2                                 16
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D3                                 17
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D4                                 18
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D5                                 19
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D6                                 20
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D7                                 21
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D8                                 22
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D9                                 23
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D10                                24
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D11                                25
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D12                                26
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D13                                27
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D14                                28
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D15                                29
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D70                                30
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D158                               31
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_D0                                 32
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_SPEC                               33
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_MINH                               34
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_MAXH                               35
#define RSRTM20XX_VAL_MEASUREMENT_SOURCE_AVER                               36

// rsrtm20xx_rngPowerAnalysisType
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_OFF                               0
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_QUAL                              1
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_CONS                              2
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_HARM                              3
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_RIPP                              4
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_SPEC                              5
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_SLEW                              6
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_DONR                              7
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_EFFS                              8
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_TURN                              9
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_TRAN                              10
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_SWITC                             11
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_MOD                               12
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_INR                               13
#define RSRTM20XX_VAL_POWER_ANALYSIS_TYPE_SOA                               14

// rsrtm20xx_rngStatistics
#define RSRTM20XX_VAL_STATISTICS_ACT                                        0
#define RSRTM20XX_VAL_STATISTICS_AVG                                        1
#define RSRTM20XX_VAL_STATISTICS_STDD                                       2
#define RSRTM20XX_VAL_STATISTICS_NPE                                        3
#define RSRTM20XX_VAL_STATISTICS_PPE                                        4
#define RSRTM20XX_VAL_STATISTICS_WFMC                                       5

// rsrtm20xx_rngQualityType
#define RSRTM20XX_VAL_QUALITY_TYPE_RMS                                      0
#define RSRTM20XX_VAL_QUALITY_TYPE_FREQ                                     1
#define RSRTM20XX_VAL_QUALITY_TYPE_CRES                                     2

// rsrtm20xx_rngQualityTypePower
#define RSRTM20XX_VAL_QUALITY_TYPE_POWER_REAL                               0
#define RSRTM20XX_VAL_QUALITY_TYPE_POWER_REAC                               1
#define RSRTM20XX_VAL_QUALITY_TYPE_POWER_APP                                2
#define RSRTM20XX_VAL_QUALITY_TYPE_POWER_PFAC                               3
#define RSRTM20XX_VAL_QUALITY_TYPE_POWER_PHAS                               4

// rsrtm20xx_rngConsumptionType
#define RSRTM20XX_VAL_CONSUMPTION_TYPE_DURATION                             0
#define RSRTM20XX_VAL_CONSUMPTION_TYPE_ENERGY                               1
#define RSRTM20XX_VAL_CONSUMPTION_TYPE_ACTIVE                               2

// rsrtm20xx_rngHarmonicsStandard
#define RSRTM20XX_VAL_CURRENT_HARMONICS_STANDARD_ENA                        0
#define RSRTM20XX_VAL_CURRENT_HARMONICS_STANDARD_ENB                        1
#define RSRTM20XX_VAL_CURRENT_HARMONICS_STANDARD_ENC                        2
#define RSRTM20XX_VAL_CURRENT_HARMONICS_STANDARD_END                        3
#define RSRTM20XX_VAL_CURRENT_HARMONICS_STANDARD_MIL                        4
#define RSRTM20XX_VAL_CURRENT_HARMONICS_STANDARD_RTC                        5

// rsrtm20xx_rngHarmonicsType
#define RSRTM20XX_VAL_HARMONICS_TYPE_VIOLENT_COUNT                          0
#define RSRTM20XX_VAL_HARMONICS_TYPE_FREQUENCY                              1
#define RSRTM20XX_VAL_HARMONICS_TYPE_LEVEL                                  2
#define RSRTM20XX_VAL_HARMONICS_TYPE_MINIMUM                                3
#define RSRTM20XX_VAL_HARMONICS_TYPE_MAXIMUM                                4
#define RSRTM20XX_VAL_HARMONICS_TYPE_AVERAGE                                5
#define RSRTM20XX_VAL_HARMONICS_TYPE_LIMIT                                  6
#define RSRTM20XX_VAL_HARMONICS_TYPE_WFMC                                   7

// rsrtm20xx_rngLimitCheck
#define RSRTM20XX_VAL_FAIL                                                  0
#define RSRTM20XX_VAL_PASS                                                  1

// rsrtm20xx_rngRippleType
#define RSRTM20XX_VAL_RIPPLE_TYPE_FREQUENCY                                 0
#define RSRTM20XX_VAL_RIPPLE_TYPE_PEAK                                      1
#define RSRTM20XX_VAL_RIPPLE_TYPE_STD_DEV                                   2
#define RSRTM20XX_VAL_RIPPLE_TYPE_UPEAK                                     3
#define RSRTM20XX_VAL_RIPPLE_TYPE_LPEAK                                     4
#define RSRTM20XX_VAL_RIPPLE_TYPE_MEAN                                      5
#define RSRTM20XX_VAL_RIPPLE_TYPE_PERIOD                                    6
#define RSRTM20XX_VAL_RIPPLE_TYPE_PDCYCLE                                   7
#define RSRTM20XX_VAL_RIPPLE_TYPE_NDCYCLE                                   8

// rsrtm20xx_rngSpectrumType
#define RSRTM20XX_VAL_SPECTRUM_TYPE_FREQUENCY                               0
#define RSRTM20XX_VAL_SPECTRUM_TYPE_LEVEL                                   1
#define RSRTM20XX_VAL_SPECTRUM_TYPE_MINIMUM                                 2
#define RSRTM20XX_VAL_SPECTRUM_TYPE_MAXIMUM                                 3
#define RSRTM20XX_VAL_SPECTRUM_TYPE_AVERAGE                                 4
#define RSRTM20XX_VAL_SPECTRUM_TYPE_WFMC                                    5

// rsrtm20xx_rngHarmonicsFundamental
#define RSRTM20XX_VAL_HARMONICS_FUNDAMENTAL_AUTO                            0
#define RSRTM20XX_VAL_HARMONICS_FUNDAMENTAL_F60                             1
#define RSRTM20XX_VAL_HARMONICS_FUNDAMENTAL_F50                             2

// rsrtm20xx_rngSlewRateType
#define RSRTM20XX_VAL_SLEW_RATE_TYPE_UPEAK                                  1
#define RSRTM20XX_VAL_SLEW_RATE_TYPE_LPEAK                                  0

// rsrtm20xx_rngModulationType
#define RSRTM20XX_VAL_MODULATION_TYPE_PER                                   0
#define RSRTM20XX_VAL_MODULATION_TYPE_FREQ                                  1
#define RSRTM20XX_VAL_MODULATION_TYPE_PDCY                                  2
#define RSRTM20XX_VAL_MODULATION_TYPE_NDCY                                  3
#define RSRTM20XX_VAL_MODULATION_TYPE_PPW                                   4
#define RSRTM20XX_VAL_MODULATION_TYPE_NPW                                   5

// rsrtm20xx_rngModulationResultType
#define RSRTM20XX_VAL_MODULATION_RESULT_TYPE_LPE                            0
#define RSRTM20XX_VAL_MODULATION_RESULT_TYPE_UPE                            1
#define RSRTM20XX_VAL_MODULATION_RESULT_TYPE_MEAN                           2
#define RSRTM20XX_VAL_MODULATION_RESULT_TYPE_RMS                            3
#define RSRTM20XX_VAL_MODULATION_RESULT_TYPE_STDD                           4

// rsrtm20xx_rngEfficiencyType
#define RSRTM20XX_VAL_EFFICIENCY_TYPE_EFFICIENCY                            0
#define RSRTM20XX_VAL_EFFICIENCY_TYPE_INPUT                                 1
#define RSRTM20XX_VAL_EFFICIENCY_TYPE_OUTPUT                                2

// rsrtm20xx_rngSwitchinglossType
#define RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_ENERGY                             0
#define RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_POWER                              1

// rsrtm20xx_rngOnOffType
#define RSRTM20XX_VAL_ON_OFF_TYPE_ON                                        1
#define RSRTM20XX_VAL_ON_OFF_TYPE_OFF                                       0

// rsrtm20xx_rngPowerAnalysisSource
#define RSRTM20XX_VAL_POWER_ANALYSIS_SOURCE_CH1                             1
#define RSRTM20XX_VAL_POWER_ANALYSIS_SOURCE_CH2                             2
#define RSRTM20XX_VAL_POWER_ANALYSIS_SOURCE_CH3                             3
#define RSRTM20XX_VAL_POWER_ANALYSIS_SOURCE_CH4                             4
#define RSRTM20XX_VAL_POWER_ANALYSIS_SOURCE_RE1                             5
#define RSRTM20XX_VAL_POWER_ANALYSIS_SOURCE_RE2                             6
#define RSRTM20XX_VAL_POWER_ANALYSIS_SOURCE_RE3                             7
#define RSRTM20XX_VAL_POWER_ANALYSIS_SOURCE_RE4                             8

// rsrtm20xx_rngAudioVariant
#define RSRTM20XX_VAL_AUDIO_VARIANT_I2S                                     0
#define RSRTM20XX_VAL_AUDIO_VARIANT_LJ                                      1
#define RSRTM20XX_VAL_AUDIO_VARIANT_RJ                                      2
#define RSRTM20XX_VAL_AUDIO_VARIANT_TDM                                     3

// rsrtm20xx_rngAudioClockPolarity
#define RSRTM20XX_VAL_FALLING                                               0
#define RSRTM20XX_VAL_RISING                                                1

// rsrtm20xx_rngAudioWSPolarity
#define RSRTM20XX_VAL_NORMAL                                                0
#define RSRTM20XX_VAL_INVERT                                                1

// rsrtm20xx_rngAudioChannelOrder
#define RSRTM20XX_VAL_AUDIO_ORDER_LEFT                                      0
#define RSRTM20XX_VAL_AUDIO_ORDER_RIGHT                                     1

// rsrtm20xx_rngProtocolAudioBitOrder
#define RSRTM20XX_VAL_AUDIO_MSBF                                            0
#define RSRTM20XX_VAL_AUDIO_LSBF                                            1

// rsrtm20xx_rngAudioTriggerType
#define RSRTM20XX_VAL_AUDIO_TRIGGER_DATA                                    0
#define RSRTM20XX_VAL_AUDIO_TRIGGER_WINDOW                                  1
#define RSRTM20XX_VAL_AUDIO_TRIGGER_WSEL                                    2
#define RSRTM20XX_VAL_AUDIO_TRIGGER_ECON                                    3

// rsrtm20xx_rngAudioChannel
#define RSRTM20XX_VAL_AUDIO_CHANNEL_ANY                                     0
#define RSRTM20XX_VAL_AUDIO_CHANNEL_TDMC1                                   1
#define RSRTM20XX_VAL_AUDIO_CHANNEL_TDMC2                                   2
#define RSRTM20XX_VAL_AUDIO_CHANNEL_TDMC3                                   3
#define RSRTM20XX_VAL_AUDIO_CHANNEL_TDMC4                                   4
#define RSRTM20XX_VAL_AUDIO_CHANNEL_TDMC5                                   5
#define RSRTM20XX_VAL_AUDIO_CHANNEL_TDMC6                                   6
#define RSRTM20XX_VAL_AUDIO_CHANNEL_TDMC7                                   7
#define RSRTM20XX_VAL_AUDIO_CHANNEL_TDMC8                                   8
#define RSRTM20XX_VAL_AUDIO_CHANNEL_LEFT                                    9
#define RSRTM20XX_VAL_AUDIO_CHANNEL_RIGHT                                   10

// rsrtm20xx_rngAudioFrameStatus
#define RSRTM20XX_VAL_AUDIO_FRAME_STATE_OK                                  0
#define RSRTM20XX_VAL_AUDIO_FRAME_STATE_ERR                                 1
#define RSRTM20XX_VAL_AUDIO_FRAME_STATE_INS                                 2

// rsrtm20xx_rngTrackDisplayMode
#define RSRTM20XX_VAL_TRACK_DISPLAY_MODE_SEQ                                0
#define RSRTM20XX_VAL_TRACK_DISPLAY_MODE_PAR                                1
#define RSRTM20XX_VAL_TRACK_DISPLAY_MODE_STR                                2
#define RSRTM20XX_VAL_TRACK_DISPLAY_MODE_PTR                                3
#define RSRTM20XX_VAL_TRACK_DISPLAY_MODE_TRAC                               4
#define RSRTM20XX_VAL_TRACK_DISPLAY_MODE_SDS                                5

// rsrtm20xx_rngAudioTriggerCondition
#define RSRTM20XX_VAL_AUDIO_CONDITION_OFF                                   0
#define RSRTM20XX_VAL_AUDIO_CONDITION_EQU                                   1
#define RSRTM20XX_VAL_AUDIO_CONDITION_NEQ                                   2
#define RSRTM20XX_VAL_AUDIO_CONDITION_GTH                                   3
#define RSRTM20XX_VAL_AUDIO_CONDITION_LTH                                   4
#define RSRTM20XX_VAL_AUDIO_CONDITION_INR                                   5
#define RSRTM20XX_VAL_AUDIO_CONDITION_OOR                                   6

// rsrtm20xx_rngAudioTriggerCombination
#define RSRTM20XX_VAL_AUDIO_COMBINATION_AND                                 0
#define RSRTM20XX_VAL_AUDIO_COMBINATION_OR                                  1

// rsrtm20xx_rngSelfAlignmentState
#define RSRTM20XX_VAL_SELF_ALIGNMENT_NO_ALIGNMENT                           0
#define RSRTM20XX_VAL_SELF_ALIGNMENT_RUN                                    1
#define RSRTM20XX_VAL_SELF_ALIGNMENT_ERROR                                  2
#define RSRTM20XX_VAL_SELF_ALIGNMENT_OK                                     3
#define RSRTM20XX_VAL_SELF_ALIGNMENT_ABORT                                  4

// rsrtm20xx_rngMILSTDCodeType
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_DBC                                  0
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_TSYN                                 1
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_TST                                  2
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_SEL                                  3
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_TSH                                  4
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_OTSH                                 5
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_ITER                                 6
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_OIT                                  7
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_RES                                  8
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_VECT                                 9
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_RSYN                                 10
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_TLAS                                 11
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_BITW                                 12
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_STSH                                 13
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_OSTS                                 14
#define RSRTM20XX_VAL_MILSTD_CODE_TYPE_ANY                                  15

// rsrtm20xx_rngMILSTDStatus
#define RSRTM20XX_VAL_MILSTD_STATUS_OK                                      0
#define RSRTM20XX_VAL_MILSTD_STATUS_INS                                     1
#define RSRTM20XX_VAL_MILSTD_STATUS_PERR                                    2
#define RSRTM20XX_VAL_MILSTD_STATUS_MERR                                    3
#define RSRTM20XX_VAL_MILSTD_STATUS_TERR                                    4
#define RSRTM20XX_VAL_MILSTD_STATUS_SERR                                    5

// rsrtm20xx_rngMILSTDDirection
#define RSRTM20XX_VAL_MILSTD_DIRECTION_TRANSMIT                             0
#define RSRTM20XX_VAL_MILSTD_DIRECTION_RECEIVE                              1
#define RSRTM20XX_VAL_MILSTD_DIRECTION_EITHER                               2

// rsrtm20xx_rngMILSTDType
#define RSRTM20XX_VAL_MILSTD_TYPE_COMMAND                                   0
#define RSRTM20XX_VAL_MILSTD_TYPE_STATUS                                    1
#define RSRTM20XX_VAL_MILSTD_TYPE_DATA                                      2
#define RSRTM20XX_VAL_MILSTD_TYPE_CMCMODE                                   3

// rsrtm20xx_rngMILSTDTriggerType
#define RSRTM20XX_VAL_MILSTD_TRIGGER_TYPE_SYNC                              0
#define RSRTM20XX_VAL_MILSTD_TRIGGER_TYPE_FRAME                             1
#define RSRTM20XX_VAL_MILSTD_TRIGGER_TYPE_ERROR                             2
#define RSRTM20XX_VAL_MILSTD_TRIGGER_TYPE_STATUS                            3
#define RSRTM20XX_VAL_MILSTD_TRIGGER_TYPE_DATA                              4
#define RSRTM20XX_VAL_MILSTD_TRIGGER_TYPE_COMMAND                           5

// rsrtm20xx_rngMILSTDTriggerSync
#define RSRTM20XX_VAL_MILSTD_TRIGGER_SYNC_CST                               0
#define RSRTM20XX_VAL_MILSTD_TRIGGER_SYNC_DATA                              1
#define RSRTM20XX_VAL_MILSTD_TRIGGER_SYNC_ALL                               2

// rsrtm20xx_rngMILSTDTriggerWord
#define RSRTM20XX_VAL_MILSTD_TRIGGER_WORD_COMMAND                           0
#define RSRTM20XX_VAL_MILSTD_TRIGGER_WORD_STATUS                            1
#define RSRTM20XX_VAL_MILSTD_TRIGGER_WORD_DATA                              2
#define RSRTM20XX_VAL_MILSTD_TRIGGER_WORD_ALL                               3

// rsrtm20xx_rngMILSTDTriggerCommandType
#define RSRTM20XX_VAL_MILSTD_TRIGGER_COMMAND_AWORD                          0
#define RSRTM20XX_VAL_MILSTD_TRIGGER_COMMAND_MCODE                          1

// rsrtm20xx_rngMILSTDCondition
#define RSRTM20XX_VAL_MILSTD_CONDITION_OFF                                  0
#define RSRTM20XX_VAL_MILSTD_CONDITION_EQU                                  1
#define RSRTM20XX_VAL_MILSTD_CONDITION_NEQ                                  2
#define RSRTM20XX_VAL_MILSTD_CONDITION_GTH                                  3
#define RSRTM20XX_VAL_MILSTD_CONDITION_GEQ                                  4
#define RSRTM20XX_VAL_MILSTD_CONDITION_LEQ                                  5
#define RSRTM20XX_VAL_MILSTD_CONDITION_LTH                                  6
#define RSRTM20XX_VAL_MILSTD_CONDITION_WITH                                 7
#define RSRTM20XX_VAL_MILSTD_CONDITION_OUTS                                 8

// rsrtm20xx_rngMILSTDSubAddress
#define RSRTM20XX_VAL_MILSTD_TRIGGER_SUB_ADDRESS_A0                         0
#define RSRTM20XX_VAL_MILSTD_TRIGGER_SUB_ADDRESS_A31                        1
#define RSRTM20XX_VAL_MILSTD_TRIGGER_SUB_ADDRESS_EITH                       2

// rsrtm20xx_rngMILSTDSearchCondition
#define RSRTM20XX_VAL_MILSTD_SEARCH_CONDITION_WORD_START                    0
#define RSRTM20XX_VAL_MILSTD_SEARCH_CONDITION_ERROR                         1
#define RSRTM20XX_VAL_MILSTD_SEARCH_CONDITION_STATUS                        2
#define RSRTM20XX_VAL_MILSTD_SEARCH_CONDITION_DATA                          3
#define RSRTM20XX_VAL_MILSTD_SEARCH_CONDITION_COMMAND                       4
#define RSRTM20XX_VAL_MILSTD_SEARCH_CONDITION_MODE_CODE                     5
#define RSRTM20XX_VAL_MILSTD_SEARCH_CONDITION_COMMAND_DATA                  6

// rsrtm20xx_rngMILSTDSearchError
#define RSRTM20XX_VAL_MILSTD_SEARCH_ERROR_SYNCHRONIZATION                   0
#define RSRTM20XX_VAL_MILSTD_SEARCH_ERROR_PARITY                            1
#define RSRTM20XX_VAL_MILSTD_SEARCH_ERROR_TIMEOUT                           2
#define RSRTM20XX_VAL_MILSTD_SEARCH_ERROR_MANCHESTER                        3
#define RSRTM20XX_VAL_MILSTD_SEARCH_ERROR_ANY                               4

// rsrtm20xx_rngHistoryPlayer
#define RSRTM20XX_VAL_HISTORY_PLAYER_RUN                                    1
#define RSRTM20XX_VAL_HISTORY_PLAYER_STOP                                   0

// rsrtm20xx_rngHistorySpeed
#define RSRTM20XX_VAL_HISTORY_SPEED_SLOW                                    0
#define RSRTM20XX_VAL_HISTORY_SPEED_MEDIUM                                  1
#define RSRTM20XX_VAL_HISTORY_SPEED_FAST                                    2
#define RSRTM20XX_VAL_HISTORY_SPEED_AUTO                                    3

// rsrtm20xx_rngThresholdHysteresis
#define RSRTM20XX_VAL_TRESHOLD_HYSTERESIS_SMALL                             0
#define RSRTM20XX_VAL_TRESHOLD_HYSTERESIS_MEDIUM                            1
#define RSRTM20XX_VAL_TRESHOLD_HYSTERESIS_LARGE                             2

// rsrtm20xx_rngStatus
#define RSRTM20XX_VAL_STATUS_0                                              0
#define RSRTM20XX_VAL_STATUS_1                                              1
#define RSRTM20XX_VAL_STATUS_X                                              2

// rsrtm20xx_rngProtocolDataSource
#define RSRTM20XX_VAL_DATA_SOURCE_C1W1                                      0
#define RSRTM20XX_VAL_DATA_SOURCE_C2W1                                      1
#define RSRTM20XX_VAL_DATA_SOURCE_C3W1                                      2
#define RSRTM20XX_VAL_DATA_SOURCE_C4W1                                      3
#define RSRTM20XX_VAL_DATA_SOURCE_M1                                        4
#define RSRTM20XX_VAL_DATA_SOURCE_M2                                        5
#define RSRTM20XX_VAL_DATA_SOURCE_M3                                        6
#define RSRTM20XX_VAL_DATA_SOURCE_M4                                        7
#define RSRTM20XX_VAL_DATA_SOURCE_R1                                        8
#define RSRTM20XX_VAL_DATA_SOURCE_R2                                        9
#define RSRTM20XX_VAL_DATA_SOURCE_R3                                        10
#define RSRTM20XX_VAL_DATA_SOURCE_R4                                        11
#define RSRTM20XX_VAL_DATA_SOURCE_TRK1                                      12
#define RSRTM20XX_VAL_DATA_SOURCE_TRK2                                      13
#define RSRTM20XX_VAL_DATA_SOURCE_TRK3                                      14
#define RSRTM20XX_VAL_DATA_SOURCE_TRK4                                      15
#define RSRTM20XX_VAL_DATA_SOURCE_TRK5                                      16
#define RSRTM20XX_VAL_DATA_SOURCE_TRK6                                      17
#define RSRTM20XX_VAL_DATA_SOURCE_TRK7                                      18
#define RSRTM20XX_VAL_DATA_SOURCE_TRK8                                      19

// rsrtm20xx_rngProtocolARINC429Polarity
#define RSRTM20XX_VAL_ARINC429_POLARITY_ALEG                                0
#define RSRTM20XX_VAL_ARINC429_POLARITY_BLEG                                1
#define RSRTM20XX_VAL_ARINC429_POLARITY_NORM                                2
#define RSRTM20XX_VAL_ARINC429_POLARITY_INV                                 3

// rsrtm20xx_rngProtocolARINC429BitRateMode
#define RSRTM20XX_VAL_ARINC429_BIT_RATE_MODE_LOW                            0
#define RSRTM20XX_VAL_ARINC429_BIT_RATE_MODE_HIGH                           1
#define RSRTM20XX_VAL_ARINC429_BIT_RATE_MODE_USER                           2

// rsrtm20xx_rngProtocolARINC429ThresholdPreset
#define RSRTM20XX_VAL_ARINC429_PRESET_V25                                   0
#define RSRTM20XX_VAL_ARINC429_PRESET_V5                                    1
#define RSRTM20XX_VAL_ARINC429_PRESET_MAN                                   2

// rsrtm20xx_rngARINC429WordStatus
#define RSRTM20XX_VAL_ARINC_WORD_STATUS_OK                                  0
#define RSRTM20XX_VAL_ARINC_WORD_STATUS_CERR                                1
#define RSRTM20XX_VAL_ARINC_WORD_STATUS_GERR                                2
#define RSRTM20XX_VAL_ARINC_WORD_STATUS_PRER                                3
#define RSRTM20XX_VAL_ARINC_WORD_STATUS_INS                                 4
#define RSRTM20XX_VAL_ARINC_WORD_STATUS_INC                                 5

// rsrtm20xx_rngProtocolARINC429DataFormat
#define RSRTM20XX_VAL_ARINC429_DATA_FORMAT_DATA                             0
#define RSRTM20XX_VAL_ARINC429_DATA_FORMAT_DSSM                             1
#define RSRTM20XX_VAL_ARINC429_DATA_FORMAT_DSDI                             2
#define RSRTM20XX_VAL_ARINC429_DATA_FORMAT_DSSS                             3

// rsrtm20xx_rngProtocolARINC429TriggerMode
#define RSRTM20XX_VAL_ARINC429_TRIGGER_START                                0
#define RSRTM20XX_VAL_ARINC429_TRIGGER_STOP                                 1
#define RSRTM20XX_VAL_ARINC429_TRIGGER_LABEL                                2
#define RSRTM20XX_VAL_ARINC429_TRIGGER_ERROR                                3

// rsrtm20xx_rngProtocolARINC429Operator
#define RSRTM20XX_VAL_ARINC_COND_NQ                                         0
#define RSRTM20XX_VAL_ARINC_COND_EQ                                         1
#define RSRTM20XX_VAL_ARINC_COND_LT                                         2
#define RSRTM20XX_VAL_ARINC_COND_LE                                         3
#define RSRTM20XX_VAL_ARINC_COND_GT                                         4
#define RSRTM20XX_VAL_ARINC_COND_GE                                         5
#define RSRTM20XX_VAL_ARINC_COND_WITH                                       6
#define RSRTM20XX_VAL_ARINC_COND_OUTS                                       7

// rsrtm20xx_rngARINCSsmSdi
#define RSRTM20XX_VAL_ARINC_SSM_SDI_ANY                                     0
#define RSRTM20XX_VAL_ARINC_SSM_SDI_S00                                     1
#define RSRTM20XX_VAL_ARINC_SSM_SDI_S01                                     2
#define RSRTM20XX_VAL_ARINC_SSM_SDI_S10                                     3
#define RSRTM20XX_VAL_ARINC_SSM_SDI_S11                                     4

// rsrtm20xx_rngARINC429TimingCondition
#define RSRTM20XX_VAL_ARINC_CONDITION_TIMING_GTH                            0
#define RSRTM20XX_VAL_ARINC_CONDITION_TIMING_LTH                            1
#define RSRTM20XX_VAL_ARINC_CONDITION_TIMING_WITH                           2
#define RSRTM20XX_VAL_ARINC_CONDITION_TIMING_OUTS                           3

// rsrtm20xx_rngARINC429WordType
#define RSRTM20XX_VAL_ARINC_WORD_TYPE_START                                 0
#define RSRTM20XX_VAL_ARINC_WORD_TYPE_STOP                                  1

// rsrtm20xx_rngARINC429Type
#define RSRTM20XX_VAL_TRIGGER_ARINC_TYPE_WORD                               0
#define RSRTM20XX_VAL_TRIGGER_ARINC_TYPE_ERROR                              1
#define RSRTM20XX_VAL_TRIGGER_ARINC_TYPE_LABEL                              2
#define RSRTM20XX_VAL_TRIGGER_ARINC_TYPE_LDATA                              3
#define RSRTM20XX_VAL_TRIGGER_ARINC_TYPE_TTIME                              4

// rsrtm20xx_rngARINC429Error
#define RSRTM20XX_VAL_ARINC_ERROR_ANY                                       0
#define RSRTM20XX_VAL_ARINC_ERROR_PARITY                                    1
#define RSRTM20XX_VAL_ARINC_ERROR_GAP                                       2
#define RSRTM20XX_VAL_ARINC_ERROR_CODING                                    3

// rsrtm20xx_rngMILSTDTransmissionType
#define RSRTM20XX_MILSTD_TRANSMISSION_TYPE_BCRT                             0
#define RSRTM20XX_MILSTD_TRANSMISSION_TYPE_RTBC                             1
#define RSRTM20XX_MILSTD_TRANSMISSION_TYPE_RTRT                             2
#define RSRTM20XX_MILSTD_TRANSMISSION_TYPE_MCD                              3

// rsrtm20xx_rngMILSTDSearchWordStart
#define RSRTM20XX_VAL_MILSTD_WORD_START_STATUS                              0
#define RSRTM20XX_VAL_MILSTD_WORD_START_DATA                                1
#define RSRTM20XX_VAL_MILSTD_WORD_START_COMMAND                             2

// rsrtm20xx_rngHardcopyCommandSet
#define RSRTM20XX_VAL_COMMAND_SET_PCL5                                      0
#define RSRTM20XX_VAL_COMMAND_SET_PCLXI                                     1
#define RSRTM20XX_VAL_COMMAND_SET_PS                                        2
#define RSRTM20XX_VAL_COMMAND_SET_DESK                                      3

// rsrtm20xx_rngScaleLinLog
#define RSRTM20XX_VAL_SCALE_LIN                                             0
#define RSRTM20XX_VAL_SCALE_LOG                                             1

// rsrtm20xx_rngHarmonicsFundamentalMIL
#define RSRTM20XX_VAL_HARMONICS_FUNDAMENTAL_MIL_F60                         0
#define RSRTM20XX_VAL_HARMONICS_FUNDAMENTAL_MIL_F400                        1

// rsrtm20xx_rngHarmonicsFundamentalRTC
#define RSRTM20XX_VAL_HARMONICS_FUNDAMENTAL_RTC_F400                        0
#define RSRTM20XX_VAL_HARMONICS_FUNDAMENTAL_RTC_NVF                         1
#define RSRTM20XX_VAL_HARMONICS_FUNDAMENTAL_RTC_WVF                         2

// rsrtm20xx_rngTransientResponseTriggerSource
#define RSRTM20XX_VAL_POWER_TR_TRIGGER_CHAN1                                1
#define RSRTM20XX_VAL_POWER_TR_TRIGGER_CHAN2                                2
#define RSRTM20XX_VAL_POWER_TR_TRIGGER_CHAN3                                3
#define RSRTM20XX_VAL_POWER_TR_TRIGGER_CHAN4                                4
#define RSRTM20XX_VAL_POWER_TR_TRIGGER_EXT                                  5
#define RSRTM20XX_VAL_POWER_TR_TRIGGER_SBUS                                 6
#define RSRTM20XX_VAL_POWER_TR_TRIGGER_MSD                                  7
#define RSRTM20XX_VAL_POWER_TR_TRIGGER_MSDS                                 8
#define RSRTM20XX_VAL_POWER_TR_TRIGGER_MSBU                                 9
#define RSRTM20XX_VAL_POWER_TR_TRIGGER_LINE                                 10

// rsrtm20xx_rngPowerScaling
#define RSRTM20XX_VAL_POWER_SCALING_AUTO                                    0
#define RSRTM20XX_VAL_POWER_SCALING_MAN                                     1

// rsrtm20xx_rngPowerTransientResponseResult
#define RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_OVER                             0
#define RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_RTIM                             1
#define RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_DEL                              2
#define RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_PEAK                             3
#define RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_SETL                             4

// rsrtm20xx_rngProbeInputImpedance
#define RSRTM20XX_VAL_UNKNOWN                                               0
#define RSRTM20XX_VAL_50OHM                                                 1
#define RSRTM20XX_VAL_1MOHM                                                 2

// RsRtm20xx_rngClockedSource
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D0                                     0
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D1                                     1
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D2                                     2
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D3                                     3
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D4                                     4
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D5                                     5
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D6                                     6
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D7                                     7
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D8                                     8
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D9                                     9
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D10                                    10
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D11                                    11
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D12                                    12
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D13                                    13
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D14                                    14
#define RSRTM20XX_VAL_CLOCKED_SOURCE_D15                                    15

// RsRtm20xx_rngDVMType
#define RSRTM20XX_VAL_DVM_TYPE_DC                                           0
#define RSRTM20XX_VAL_DVM_TYPE_ACDC                                         1
#define RSRTM20XX_VAL_DVM_TYPE_ACRM                                         2
#define RSRTM20XX_VAL_DVM_TYPE_UPE                                          3
#define RSRTM20XX_VAL_DVM_TYPE_LPE                                          4
#define RSRTM20XX_VAL_DVM_TYPE_PEAK                                         5
#define RSRTM20XX_VAL_DVM_TYPE_CRES                                         6
#define RSRTM20XX_VAL_DVM_TYPE_OFF                                          7

// RsRtm20xx_rngDVMPosition
#define RSRTM20XX_VAL_DVM_POSITION_TLEFT                                    0
#define RSRTM20XX_VAL_DVM_POSITION_TRIGHT                                   1
#define RSRTM20XX_VAL_DVM_POSITION_BLEFT                                    2
#define RSRTM20XX_VAL_DVM_POSITION_BRIGHT                                   3

// RsRtm20xx_rngMagnitudeScale
#define RSRTM20XX_VAL_MAGNITUDE_SCALE_LIN                                   0
#define RSRTM20XX_VAL_MAGNITUDE_SCALE_DBM                                   1
#define RSRTM20XX_VAL_MAGNITUDE_SCALE_DBC                                   2

// RsRtm20xx_rngSpectrumFrequencyWindowType
#define RSRTM20XX_VAL_SPECTRUM_FREQ_WINDOW_TYPE_RECT                        0
#define RSRTM20XX_VAL_SPECTRUM_FREQ_WINDOW_TYPE_HAMM                        1
#define RSRTM20XX_VAL_SPECTRUM_FREQ_WINDOW_TYPE_HANN                        2
#define RSRTM20XX_VAL_SPECTRUM_FREQ_WINDOW_TYPE_BLAC                        3
#define RSRTM20XX_VAL_SPECTRUM_FREQ_WINDOW_TYPE_FLAT                        4

// RsRtm20xx_rngSpectrumMarkerDisplay
#define RSRTM20XX_VAL_SPECTRUM_MARKER_OFF                                   0
#define RSRTM20XX_VAL_SPECTRUM_MARKER_INDEX                                 1
#define RSRTM20XX_VAL_SPECTRUM_MARKER_RESULT                                2

// RsRtm20xx_rngSpectrumMarkerSource
#define RSRTM20XX_VAL_SPECTRUM_MARKER_SOURCE_SPEC                           0
#define RSRTM20XX_VAL_SPECTRUM_MARKER_SOURCE_MINH                           1
#define RSRTM20XX_VAL_SPECTRUM_MARKER_SOURCE_MAXH                           2
#define RSRTM20XX_VAL_SPECTRUM_MARKER_SOURCE_AVER                           3

// RsRtm20xx_rngSpectrumMarkerSetupDetectionMode
#define RSRTM20XX_VAL_SPECTRUM_MARKER_DETECTION_LEVEL                       0
#define RSRTM20XX_VAL_SPECTRUM_MARKER_DETECTION_ADVANCED                    1

// RsRtm20xx_rngSpectrumMarkerReferenceMode
#define RSRTM20XX_VAL_SPECTRUM_MARKER_REF_MODE_OFF                          0
#define RSRTM20XX_VAL_SPECTRUM_MARKER_REF_MODE_INDICATED                    1
#define RSRTM20XX_VAL_SPECTRUM_MARKER_REF_MODE_RANGE                        2

// RsRtm20xx_rngSpectrumMarkerResultMode
#define RSRTM20XX_VAL_SPECTRUM_MARKER_RESULT_MODE_ABSOLUTE                  0
#define RSRTM20XX_VAL_SPECTRUM_MARKER_RESULT_MODE_FREQUENCY                 1
#define RSRTM20XX_VAL_SPECTRUM_MARKER_RESULT_MODE_LEVEL                     2
#define RSRTM20XX_VAL_SPECTRUM_MARKER_RESULT_MODE_FLEVEL                    3

// RsRtm20xx_rngSpectrumMarkerTablePosition
#define RSRTM20XX_VAL_SPECTRUM_MARKER_TABLE_POSITION_TOP                    0
#define RSRTM20XX_VAL_SPECTRUM_MARKER_TABLE_POSITION_BOTTOM                 1
#define RSRTM20XX_VAL_SPECTRUM_MARKER_TABLE_POSITION_FULL                   2

// RsRtm20xx_rngSpectrumDisplayColorScheme
#define RSRTM20XX_VAL_SPECTRUM_COLOR_SCHEME_MONOCHROM                       0
#define RSRTM20XX_VAL_SPECTRUM_COLOR_SCHEME_TEMPERATURE                     1
#define RSRTM20XX_VAL_SPECTRUM_COLOR_SCHEME_RAINBOW                         2
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSRTM20XX_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSRTM20XX_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSRTM20XX_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSRTM20XX_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/
#define RSRTM20XX_VAL_CHANNEL_1   1
#define RSRTM20XX_VAL_CHANNEL_2   2
#define RSRTM20XX_VAL_CHANNEL_3   3
#define RSRTM20XX_VAL_CHANNEL_4   4
#define RSRTM20XX_VAL_CHANNEL_EXT 5

#define RSRTM20XX_VAL_SERIAL_BUS_1  1
#define RSRTM20XX_VAL_SERIAL_BUS_2  2
#define RSRTM20XX_VAL_SERIAL_BUS_3  3
#define RSRTM20XX_VAL_SERIAL_BUS_4  4

#define RSRTM20XX_VAL_TRIGGER_A 1
#define RSRTM20XX_VAL_TRIGGER_B 2

#define RSRTM20XX_VAL_TRIGGER_FILTER_LOW  0
#define RSRTM20XX_VAL_TRIGGER_FILTER_HIGH 1
    
#define RSRTM20XX_VAL_DISPLAY_BACKGROUND 0
#define RSRTM20XX_VAL_DISPLAY_GRID       1
#define RSRTM20XX_VAL_DISPLAY_WAVEFORM   2

#define RSRTM20XX_VAL_MARKER_ACTION_SET    0
#define RSRTM20XX_VAL_MARKER_ACTION_NEXT   1
#define RSRTM20XX_VAL_MARKER_ACTION_PREV   2
#define RSRTM20XX_VAL_MARKER_ACTION_CLEAR  3
#define RSRTM20XX_VAL_MARKER_ACTION_ACLEAR 4

#define RSRTM20XX_VAL_MEASUREMENT_1 1
#define RSRTM20XX_VAL_MEASUREMENT_2 2
#define RSRTM20XX_VAL_MEASUREMENT_3 3
#define RSRTM20XX_VAL_MEASUREMENT_4 4
#define RSRTM20XX_VAL_MEASUREMENT_5 5
#define RSRTM20XX_VAL_MEASUREMENT_6 6
#define RSRTM20XX_VAL_MEASUREMENT_7 7
#define RSRTM20XX_VAL_MEASUREMENT_8 8

#define RSRTM20XX_VAL_CURSOR_1 1
#define RSRTM20XX_VAL_CURSOR_2 2
#define RSRTM20XX_VAL_CURSOR_3 3

#define RSRTM20XX_VAL_I2C_SCL       0
#define RSRTM20XX_VAL_I2C_SDA       1

#define RSRTM20XX_VAL_SPI_CLOCK  0
#define RSRTM20XX_VAL_SPI_DATA   1
#define RSRTM20XX_VAL_SPI_CSEL   2

#define RSRTM20XX_VAL_PARALLEL_BIT0 0
#define RSRTM20XX_VAL_PARALLEL_BIT1 1
#define RSRTM20XX_VAL_PARALLEL_BIT2 2
#define RSRTM20XX_VAL_PARALLEL_BIT3 3

#define RSRTM20XX_VAL_WAV_LOAD      0
#define RSRTM20XX_VAL_WAV_LOAD_CFG  1
#define RSRTM20XX_VAL_WAV_SAVE      2

#define RSRTM20XX_VAL_REF_WAVEFORM_1 1
#define RSRTM20XX_VAL_REF_WAVEFORM_2 2
#define RSRTM20XX_VAL_REF_WAVEFORM_3 3
#define RSRTM20XX_VAL_REF_WAVEFORM_4 4

#define RSRTM20XX_VAL_INIT_SINGLE     0
#define RSRTM20XX_VAL_INIT_CONTINUOUS 1
    
#define RSRTM20XX_VAL_FILE_COPY    0
#define RSRTM20XX_VAL_FILE_MOVE    1
#define RSRTM20XX_VAL_FILE_DELETE  2
#define RSRTM20XX_VAL_FILE_NEW_DIR 3
#define RSRTM20XX_VAL_FILE_CDIR    4
#define RSRTM20XX_VAL_FILE_RDIR    5
#define RSRTM20XX_VAL_FILE_CDRIVE  6

#define RSRTM20XX_VAL_REG_OPER 0 
#define RSRTM20XX_VAL_REG_QUES 1

#define RSRTM20XX_VAL_REG_NONE     0
#define RSRTM20XX_VAL_REG_COV      1
#define RSRTM20XX_VAL_REG_LIM      2
#define RSRTM20XX_VAL_REG_MASK     3


#define RSRTM20XX_VAL_REG_COND   0
#define RSRTM20XX_VAL_REG_ENABLE 1
#define RSRTM20XX_VAL_REG_EVENT  2
#define RSRTM20XX_VAL_REG_NTR    3
#define RSRTM20XX_VAL_REG_PTR    4
    
#define RSRTM20XX_VAL_MEM_SAVE    0
#define RSRTM20XX_VAL_MEM_RECALL  1
	
#define RSRTM20XX_VAL_MEAS_SLOPE_POS 0
#define RSRTM20XX_VAL_MEAS_SLOPE_NEG 1
	
#define RSRTM20XX_VAL_CURSOR_POSITION_AUTOSET  0
#define RSRTM20XX_VAL_CURSOR_POSITION_RESET    1
	
#define RSRTM20XX_VAL_CURSOR_PEAK_NEXT		   0
#define RSRTM20XX_VAL_CURSOR_PEAK_PREVIOUS	   1
	
#define RSRTM20XX_VAL_AUTOMATIC_MEASUREMENT_AVERAGE      0
#define RSRTM20XX_VAL_AUTOMATIC_MEASUREMENT_DEVIATION    1
#define RSRTM20XX_VAL_AUTOMATIC_MEASUREMENT_MINIMUM      2
#define RSRTM20XX_VAL_AUTOMATIC_MEASUREMENT_MAXIMUM      3

#define RSRTM20XX_VAL_MASK_ACTION_SOUND    0
#define RSRTM20XX_VAL_MASK_ACTION_STOP     1
#define RSRTM20XX_VAL_MASK_ACTION_SCREEN   2
#define RSRTM20XX_VAL_MASK_ACTION_PRINT    3
#define RSRTM20XX_VAL_MASK_ACTION_WAVEFORM 4

#define RSRTM20XX_VAL_SEARCH_D2C_CLOCK 0
#define RSRTM20XX_VAL_SEARCH_D2C_DATA  1

#define RSRTM20XX_VAL_LIN_FIELD_SYNC   0
#define RSRTM20XX_VAL_LIN_FIELD_CHKSUM 1

#define RSRTM20XX_VAL_CAN_FIELD_DLC 0
#define RSRTM20XX_VAL_CAN_FIELD_CRC 1
#define RSRTM20XX_VAL_CAN_FIELD_ACK 2

#define RSRTM20XX_VAL_CAN_FIELD_ACK 2

#define RSSCOPE_VAL_SERIAL_BUS_1 1
#define RSSCOPE_VAL_SERIAL_BUS_2 2
#define RSSCOPE_VAL_SERIAL_BUS_3 3
#define RSSCOPE_VAL_SERIAL_BUS_4 4

#define RSRTM20XX_VAL_CONSUMPTION_TYPE_DURATION 0
#define RSRTM20XX_VAL_CONSUMPTION_TYPE_ENERGY   1
#define RSRTM20XX_VAL_CONSUMPTION_TYPE_ACTIVE   2

#define RSRTM20XX_VAL_INRUSH_CURRENT_TYPE_AREA      0
#define RSRTM20XX_VAL_INRUSH_CURRENT_TYPE_MAXIMUM   1

#define RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_CONDUCTION 0
#define RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_NO_COND    1
#define RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_TURN_ON    2
#define RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_TURN_OFF   3
#define RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_TOTAL      4

#define RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_OVERSHOOT    0
#define RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_RISE_TIME    1
#define RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_DELAY_TIME   2
#define RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_PEAK_TIME    3
#define RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_PEAK_VALUE   4
#define RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_SETTLING     5

#define RSRTM20XX_VAL_ADD     0
#define RSRTM20XX_VAL_REMOVE  1
#define RSRTM20XX_VAL_INSERT  2

#define RSRTM20XX_VAL_POWER_SOURCE_CURRENT_MIN  0
#define RSRTM20XX_VAL_POWER_SOURCE_CURRENT_MAX  1
#define RSRTM20XX_VAL_POWER_SOURCE_VOLTAGE      2 

#define RSRTM20XX_VAL_POWER_SOA_TYPE_CURRENT  0
#define RSRTM20XX_VAL_POWER_SOA_TYPE_VOLTAGE  1

#define RSRTM20XX_VAL_POWER_SOA_TYPE_CURRENT  0
#define RSRTM20XX_VAL_POWER_SOA_TYPE_VOLTAGE  1

#define RSRTM20XX_VAL_FAST_SWEEP_NORMAL 0
#define RSRTM20XX_VAL_FAST_SWEEP_FAST   1
/****************************************************************************
*---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/
ViStatus _VI_FUNC rsrtm20xx_init (ViRsrc resourceName, ViBoolean IDQuery,
                                  ViBoolean resetDevice,
                                  ViSession* instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                             ViBoolean resetDevice,
                                             ViString optionString,
                                             ViSession* instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_Autoset (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_ConfigureAcquisitionType (ViSession instrumentHandle,
                                                      ViInt32 channel,
                                                      ViInt32 acquisitionType);
ViStatus _VI_FUNC rsrtm20xx_ConfigureAcquisitionRecord (ViSession instrumentHandle,
                                                        ViReal64 timePerRecord,
                                                        ViReal64 acquisitionStartTime_Offset);
ViStatus _VI_FUNC rsrtm20xx_ConfigureInterpolation (ViSession instrumentHandle,
                                                    ViInt32 interpolation);
ViStatus _VI_FUNC rsrtm20xx_ConfigureNumberOfAverages (ViSession instrumentHandle,
                                                       ViInt32 numberOfAverages);
ViStatus _VI_FUNC rsrtm20xx_ConfigureNumberOfWaveforms (ViSession instrumentHandle,
                                                        ViInt32 numberOfWaveforms);
ViStatus _VI_FUNC rsrtm20xx_ConfigureChannel (ViSession instrumentHandle,
                                              ViInt32 channel,
                                              ViBoolean channelEnabled,
                                              ViReal64 verticalRange,
                                              ViReal64 verticalOffset,
                                              ViInt32 verticalCoupling);
ViStatus _VI_FUNC rsrtm20xx_ConfigureChannelBandwidth (ViSession instrumentHandle,
                                                       ViInt32 channel,
                                                       ViInt32 bandwidth);
ViStatus _VI_FUNC rsrtm20xx_QueryProbeCharacteristics (ViSession instrumentHandle,
                                                       ViInt32 channel,
                                                       ViInt32 bufferSize,
                                                       ViChar _VI_FAR type[],
                                                       ViChar _VI_FAR name[],
                                                       ViInt32* inputImpedance,
                                                       ViReal64* inputCapacitance,
                                                       ViReal64* attenuation,
                                                       ViReal64* bandwidth);
ViStatus _VI_FUNC rsrtm20xx_ConfigureProbeAttenuationManual
             (ViSession instrumentHandle, ViInt32 channel, ViReal64 attenuation);
ViStatus _VI_FUNC rsrtm20xx_ConfigureTrigger (ViSession instrumentHandle,
                                              ViInt32 trigger, ViInt32 triggerType);
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerSource (ViSession instrumentHandle,
                                                    ViInt32 trigger,
                                                    ViInt32 channel);
ViStatus _VI_FUNC rsrtm20xx_ConfigureExternalTriggerCoupling
             (ViSession instrumentHandle, ViInt32 externalCoupling);
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerHoldoffTime (ViSession instrumentHandle,
                                                         ViReal64 holdoffTime);
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerBState (ViSession instrumentHandle,
                                                    ViBoolean state);
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerModifier (ViSession instrumentHandle,
                                                      ViInt32 trigger,
                                                      ViInt32 triggerModifier);
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerEventCount (ViSession instrumentHandle,
                                                        ViInt32 eventCount);
ViStatus _VI_FUNC rsrtm20xx_TriggerLevelAutoFind (ViSession instrumentHandle,
                                                  ViInt32 trigger);
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerChannelLevel (ViSession instrumentHandle,
                                                          ViInt32 trigger,
                                                          ViInt32 channel,
                                                          ViReal64 triggerLevel);
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerHysteresis (ViSession instrumentHandle,
                                                        ViInt32 trigger,
                                                        ViInt32 hysteresis);
ViStatus _VI_FUNC rsrtm20xx_ConfigureEdgeTriggerSource (ViSession instrumentHandle,
                                                        ViInt32 trigger,
                                                        ViInt32 channel,
                                                        ViInt32 triggerSource,
                                                        ViReal64 triggerLevel,
                                                        ViInt32 triggerSlope);
ViStatus _VI_FUNC rsrtm20xx_ConfigureEdgeTriggerCoupling (ViSession instrumentHandle,
                                                          ViInt32 coupling);
ViStatus _VI_FUNC rsrtm20xx_ConfigureEdgeTriggerFilter (ViSession instrumentHandle,
                                                        ViInt32 filterType,
                                                        ViBoolean filterState);
ViStatus _VI_FUNC rsrtm20xx_ConfigureWidthTriggerSource (ViSession instrumentHandle,
                                                         ViInt32 triggerSource,
                                                         ViReal64 triggerLevel,
                                                         ViReal64 width,
                                                         ViReal64 widthDelta,
                                                         ViInt32 widthPolarity,
                                                         ViInt32 widthCondition);
ViStatus _VI_FUNC rsrtm20xx_ConfigurePatternTriggerSource
             (ViSession instrumentHandle, ViString patternSource,
              ViInt32 patternFunction, ViBoolean patternCondition);
ViStatus _VI_FUNC rsrtm20xx_ConfigureVideoTriggerSource (ViSession instrumentHandle,
                                                         ViInt32 standard,
                                                         ViInt32 polarity,
                                                         ViInt32 field,
                                                         ViInt32 line);
ViStatus _VI_FUNC rsrtm20xx_ConfigureRuntTrigger (ViSession instrumentHandle,
                                                  ViInt32 polarity, ViInt32 channel,
                                                  ViReal64 lowerLevel,
                                                  ViReal64 upperLevel);
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerDelay (ViSession instrumentHandle,
                                                   ViReal64 delayTime);
ViStatus _VI_FUNC rsrtm20xx_DisplayAddLanguage (ViSession instrumentHandle,
                                                ViInt32 language, ViString source);
ViStatus _VI_FUNC rsrtm20xx_MarkerSearch (ViSession instrumentHandle,
                                          ViInt32 markerAction);
ViStatus _VI_FUNC rsrtm20xx_ClearPersistence (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryChannelAcquisitionAbsoluteTime
             (ViSession instrumentHandle, ViInt32 channel, ViInt32* hour,
              ViInt32* minute, ViReal64* second);
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryChannelAcquisitionDate
             (ViSession instrumentHandle, ViInt32 channel, ViInt32* year,
              ViInt32* month, ViInt32* day);
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryDigitalAcquisitionAbsoluteTime
             (ViSession instrumentHandle, ViInt32 digitalChannel, ViInt32* hour,
              ViInt32* minute, ViReal64* second);
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryDigitalAcquisitionDate
             (ViSession instrumentHandle, ViInt32 digitalChannel, ViInt32* year,
              ViInt32* month, ViInt32* day);
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryMathAcquisitionAbsoluteTime
             (ViSession instrumentHandle, ViInt32 mathNumber, ViInt32* hour,
              ViInt32* minute, ViReal64* second);
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryMathAcquisitionDate
             (ViSession instrumentHandle, ViInt32 mathNumber, ViInt32* year,
              ViInt32* month, ViInt32* day);
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryProtocolAcquisitionAbsoluteTime
             (ViSession instrumentHandle, ViInt32 serialBus, ViInt32* hour,
              ViInt32* minute, ViReal64* second);
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryProtocolAcquisitionDate
             (ViSession instrumentHandle, ViInt32 serialBus, ViInt32* year,
              ViInt32* month, ViInt32* day);
ViStatus _VI_FUNC rsrtm20xx_ConfigureMeasurementSource (ViSession instrumentHandle,
                                                        ViInt32 measurement,
                                                        ViInt32 signalSource,
                                                        ViInt32 referenceSource);
ViStatus _VI_FUNC rsrtm20xx_ConfigureDelayMeasuremenSlope
             (ViSession instrumentHandle, ViInt32 measurement, ViInt32 source1Slope,
              ViInt32 source2Slope);
ViStatus _VI_FUNC rsrtm20xx_ConfigureReferenceLevelMode (ViSession instrumentHandle,
                                                         ViInt32 relativeMode);
ViStatus _VI_FUNC rsrtm20xx_ConfigureUserReferenceLevel (ViSession instrumentHandle,
                                                         ViReal64 lowerLevel,
                                                         ViReal64 upperLevel,
                                                         ViReal64 middleLevel);
ViStatus _VI_FUNC rsrtm20xx_AllCursorsOff (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_SetCursorPosition (ViSession instrumentHandle,
                                               ViInt32 cursor, ViInt32 position);
ViStatus _VI_FUNC rsrtm20xx_ConfigureCursorPosition (ViSession instrumentHandle,
                                                     ViReal64 xPosition1,
                                                     ViReal64 xPosition2,
                                                     ViReal64 yPosition1,
                                                     ViReal64 yPosition2);
ViStatus _VI_FUNC rsrtm20xx_SetFFTAnalysisCursorToPeak (ViSession instrumentHandle,
                                                        ViInt32 cursor,
                                                        ViInt32 peak);
ViStatus _VI_FUNC rsrtm20xx_ConfigureQuickMeasurements (ViSession instrumentHandle,
                                                        ViInt32 measurement,
                                                        ViBoolean quickMeasurementsEnabled);
ViStatus _VI_FUNC rsrtm20xx_QueryQuickMeasurementResults (ViSession instrumentHandle,
                                                          ViInt32 measurement,
                                                          ViInt32 arraySize,
                                                          ViReal64 _VI_FAR results[]);
ViStatus _VI_FUNC rsrtm20xx_ResetAutomaticMeasurement (ViSession instrumentHandle,
                                                       ViInt32 measurement);
ViStatus _VI_FUNC rsrtm20xx_QueryAutomaticMeasurementAllStatisticalValues
             (ViSession instrumentHandle, ViInt32 measurement, ViInt32 arraySize,
              ViReal64 _VI_FAR results[]);
ViStatus _VI_FUNC rsrtm20xx_SampleMode (ViSession instrumentHandle,
                                        ViInt32* sampleMode);
ViStatus _VI_FUNC rsrtm20xx_MaskCopyChannel (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_MaskResetTestCounter (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QuerySearchResult (ViSession instrumentHandle,
                                               ViInt32 resultNumber,
                                               ViReal64* xPosition,
                                               ViReal64* yPosition,
                                               ViInt32* resultType,
                                               ViInt32* polarity,
                                               ViReal64* voltageWidth);
ViStatus _VI_FUNC rsrtm20xx_QueryAllSearchResults (ViSession instrumentHandle,
                                                   ViInt32 arraySize,
                                                   ViReal64 _VI_FAR xPosition[],
                                                   ViReal64 _VI_FAR yPosition[],
                                                   ViInt32 _VI_FAR resultType[],
                                                   ViInt32 _VI_FAR polarity[],
                                                   ViReal64 _VI_FAR voltageWidth[],
                                                   ViInt32* numberOfResults);
ViStatus _VI_FUNC rsrtm20xx_ProtocolFindLevel (ViSession instrumentHandle,
                                               ViInt32 channel);
ViStatus _VI_FUNC rsrtm20xx_QueryBusDecodedData (ViSession instrumentHandle,
                                                 ViInt32 serialBus,
                                                 ViInt32 bufferSize,
                                                 ViChar _VI_FAR decodedData[]);
ViStatus _VI_FUNC rsrtm20xx_QueryI2CFrameAllData (ViSession instrumentHandle,
                                                  ViInt32 serialBus, ViInt32 frame,
                                                  ViInt32 arraySize,
                                                  ViInt32 _VI_FAR data[],
                                                  ViInt32* returnedValues);
ViStatus _VI_FUNC rsrtm20xx_QuerySPIFrameAllData (ViSession instrumentHandle,
                                                  ViInt32 serialBus, ViInt32 line,
                                                  ViInt32 frame, ViInt32 arraySize,
                                                  ViInt32 _VI_FAR data[],
                                                  ViInt32* returnedValues);
ViStatus _VI_FUNC rsrtm20xx_QueryLINFrameAllData (ViSession instrumentHandle,
                                                  ViInt32 serialBus, ViInt32 frame,
                                                  ViInt32 arraySize,
                                                  ViInt32 _VI_FAR data[],
                                                  ViInt32* returnedValues);
ViStatus _VI_FUNC rsrtm20xx_QueryCANFrameAllData (ViSession instrumentHandle,
                                                  ViInt32 serialBus, ViInt32 frame,
                                                  ViInt32 arraySize,
                                                  ViInt32 _VI_FAR data[],
                                                  ViInt32* returnedValues);
ViStatus _VI_FUNC rsrtm20xx_ProtocolAudioTrackWaveformSetToDefault
             (ViSession instrumentHandle, ViInt32 serialBus);
ViStatus _VI_FUNC rsrtm20xx_ProtocolAudioTrackWaveformSetToScreen
             (ViSession instrumentHandle, ViInt32 serialBus);
ViStatus _VI_FUNC rsrtm20xx_ConfigureARINC429Source (ViSession instrumentHandle,
                                                     ViInt32 serialBus,
                                                     ViInt32 source);
ViStatus _VI_FUNC rsrtm20xx_ConfigureARINC429Polarity (ViSession instrumentHandle,
                                                       ViInt32 serialBus,
                                                       ViInt32 polarity);
ViStatus _VI_FUNC rsrtm20xx_ConfigureARINC429BitRate (ViSession instrumentHandle,
                                                      ViInt32 serialBus,
                                                      ViInt32 bitRateMode,
                                                      ViReal64 bitRateValue);
ViStatus _VI_FUNC rsrtm20xx_ConfigureARINC429Threshold (ViSession instrumentHandle,
                                                        ViInt32 serialBus,
                                                        ViReal64 thresholdHigh,
                                                        ViReal64 thresholdLow);
ViStatus _VI_FUNC rsrtm20xx_ConfigureARINC429TriggerType (ViSession instrumentHandle,
                                                          ViInt32 type);
ViStatus _VI_FUNC rsrtm20xx_QueryARINC429WordCount (ViSession instrumentHandle,
                                                    ViInt32 serialBus,
                                                    ViInt32* count);
ViStatus _VI_FUNC rsrtm20xx_QueryARINC429Status (ViSession instrumentHandle,
                                                 ViInt32 serialBus, ViInt32 word,
                                                 ViInt32* status, ViReal64* start,
                                                 ViReal64* stop, ViInt32* pattern,
                                                 ViInt32* SDI, ViInt32* SSM);
ViStatus _VI_FUNC rsrtm20xx_QueryARINC429AllData (ViSession instrumentHandle,
                                                  ViInt32 serialBus, ViInt32 word,
                                                  ViInt32 arraySize,
                                                  ViInt32 _VI_FAR data[],
                                                  ViInt32* returnedValues);
ViStatus _VI_FUNC rsrtm20xx_ConfigureHardcopySettings (ViSession instrumentHandle,
                                                       ViString fileName,
                                                       ViInt32 deviceDestination,
                                                       ViBoolean includeMenuInScreenshot,
                                                       ViInt32 deviceLanguageOutputFormat);
ViStatus _VI_FUNC rsrtm20xx_HardcopyPrint (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_SetAttributeViInt32 (ViSession instrumentHandle,
                                                 ViString channelName,
                                                 ViAttr attributeID,
                                                 ViInt32 attributeValue);
ViStatus _VI_FUNC rsrtm20xx_SetAttributeViReal64 (ViSession instrumentHandle,
                                                  ViString channelName,
                                                  ViAttr attributeID,
                                                  ViReal64 attributeValue);
ViStatus _VI_FUNC rsrtm20xx_SetAttributeViString (ViSession instrumentHandle,
                                                  ViString channelName,
                                                  ViAttr attributeID,
                                                  ViString attributeValue);
ViStatus _VI_FUNC rsrtm20xx_SetAttributeViBoolean (ViSession instrumentHandle,
                                                   ViString channelName,
                                                   ViAttr attributeID,
                                                   ViBoolean attributeValue);
ViStatus _VI_FUNC rsrtm20xx_SetAttributeViSession (ViSession instrumentHandle,
                                                   ViString channelName,
                                                   ViAttr attributeID,
                                                   ViSession attributeValue);
ViStatus _VI_FUNC rsrtm20xx_GetAttributeViInt32 (ViSession instrumentHandle,
                                                 ViString channelName,
                                                 ViAttr attributeID,
                                                 ViInt32* attributeValue);
ViStatus _VI_FUNC rsrtm20xx_GetAttributeViReal64 (ViSession instrumentHandle,
                                                  ViString channelName,
                                                  ViAttr attributeID,
                                                  ViReal64* attributeValue);
ViStatus _VI_FUNC rsrtm20xx_GetAttributeViString (ViSession instrumentHandle,
                                                  ViString channelName,
                                                  ViAttr attributeID,
                                                  ViInt32 bufferSize,
                                                  ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rsrtm20xx_GetAttributeViBoolean (ViSession instrumentHandle,
                                                   ViString channelName,
                                                   ViAttr attributeID,
                                                   ViBoolean* attributeValue);
ViStatus _VI_FUNC rsrtm20xx_GetAttributeViSession (ViSession instrumentHandle,
                                                   ViString channelName,
                                                   ViAttr attributeID,
                                                   ViSession* attributeValue);
ViStatus _VI_FUNC rsrtm20xx_CheckAttributeViInt32 (ViSession instrumentHandle,
                                                   ViString channelName,
                                                   ViAttr attributeID,
                                                   ViInt32 attributeValue);
ViStatus _VI_FUNC rsrtm20xx_CheckAttributeViReal64 (ViSession instrumentHandle,
                                                    ViString channelName,
                                                    ViAttr attributeID,
                                                    ViReal64 attributeValue);
ViStatus _VI_FUNC rsrtm20xx_CheckAttributeViString (ViSession instrumentHandle,
                                                    ViString channelName,
                                                    ViAttr attributeID,
                                                    ViString attributeValue);
ViStatus _VI_FUNC rsrtm20xx_CheckAttributeViBoolean (ViSession instrumentHandle,
                                                     ViString channelName,
                                                     ViAttr attributeID,
                                                     ViBoolean attributeValue);
ViStatus _VI_FUNC rsrtm20xx_CheckAttributeViSession (ViSession instrumentHandle,
                                                     ViString channelName,
                                                     ViAttr attributeID,
                                                     ViSession attributeValue);
ViStatus _VI_FUNC rsrtm20xx_GetAttributeRepeatedCapabilityIds
             (ViSession instrumentHandle, ViUInt32 attributeID, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityId_s[]);
ViStatus _VI_FUNC rsrtm20xx_GetAttributeRepeatedCapabilityIdNames
             (ViSession instrumentHandle, ViUInt32 attributeID,
              ViString repeatedCapabilityId, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityIdName_s[]);
ViStatus _VI_FUNC rsrtm20xx_ConfigureFFTFrequencySpanFull
             (ViSession instrumentHandle, ViInt32 mathNumber);
ViStatus _VI_FUNC rsrtm20xx_UpdateReferenceWaveformWithSelectedSource
             (ViSession instrumentHandle, ViInt32 referenceNumber);
ViStatus _VI_FUNC rsrtm20xx_ReferenceWaveformFileOperations
             (ViSession instrumentHandle, ViInt32 referenceWfm, ViInt32 operation,
              ViString fileName);
ViStatus _VI_FUNC rsrtm20xx_ReadWaveform (ViSession instrumentHandle,
                                          ViInt32 channel, ViInt32 waveformSize,
                                          ViInt32 maximumTime,
                                          ViReal64 _VI_FAR waveformArray[],
                                          ViInt32* actualPoints, ViReal64* initialX,
                                          ViReal64* xIncrement);
ViStatus _VI_FUNC rsrtm20xx_ReadMaskData (ViSession instrumentHandle,
                                          ViInt32 waveformSize, ViInt32 maximumTime,
                                          ViReal64 _VI_FAR waveformArray[],
                                          ViInt32* actualPoints, ViReal64* initialX,
                                          ViReal64* xIncrement);
ViStatus _VI_FUNC rsrtm20xx_ReadMainWaveformMeasurement (ViSession instrumentHandle,
                                                         ViInt32 measurement,
                                                         ViInt32 measurementFunction,
                                                         ViInt32 maximumTime,
                                                         ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_InitiateAcquisition (ViSession instrumentHandle,
                                                 ViInt32 action);
ViStatus _VI_FUNC rsrtm20xx_SendSoftwareTrigger (ViSession instrumentHandle,
                                                 ViInt32 maximumTime);
ViStatus _VI_FUNC rsrtm20xx_FetchWaveform (ViSession instrumentHandle,
                                           ViInt32 channel, ViInt32 waveformSize,
                                           ViReal64 _VI_FAR waveformArray[],
                                           ViInt32* actualPoints,
                                           ViReal64* initialX,
                                           ViReal64* xIncrement);
ViStatus _VI_FUNC rsrtm20xx_FetchWaveformInteger (ViSession instrumentHandle,
                                                  ViInt32 channel,
                                                  ViInt32 waveformSize,
                                                  ViUInt16 _VI_FAR waveformArray[],
                                                  ViInt32* actualPoints,
                                                  ViReal64* initialX,
                                                  ViReal64* xIncrement);
ViStatus _VI_FUNC rsrtm20xx_FetchWaveformByte (ViSession instrumentHandle,
                                               ViInt32 channel,
                                               ViInt32 waveformSize,
                                               ViByte _VI_FAR waveformArray[],
                                               ViInt32* actualPoints,
                                               ViReal64* initialX,
                                               ViReal64* xIncrement);
ViStatus _VI_FUNC rsrtm20xx_FetchWaveformHeader (ViSession instrumentHandle,
                                                 ViInt32 channel, ViReal64* xStart,
                                                 ViReal64* xStop,
                                                 ViInt32* recordLength,
                                                 ViInt32* valuesPerSample);
ViStatus _VI_FUNC rsrtm20xx_FetchWaveformConversionData (ViSession instrumentHandle,
                                                         ViInt32 channel,
                                                         ViReal64* xStart,
                                                         ViReal64* xIncrement,
                                                         ViReal64* yStart,
                                                         ViReal64* yIncrement,
                                                         ViInt32* yResolution);
ViStatus _VI_FUNC rsrtm20xx_FetchEnvelopeWaveform (ViSession instrumentHandle,
                                                   ViInt32 channel,
                                                   ViInt32 waveformSize,
                                                   ViReal64 _VI_FAR waveformArray[],
                                                   ViInt32* actualPoints,
                                                   ViReal64* initialX,
                                                   ViReal64* xIncrement);
ViStatus _VI_FUNC rsrtm20xx_FetchEnvelopeWaveformHeader (ViSession instrumentHandle,
                                                         ViInt32 channel,
                                                         ViReal64* xStart,
                                                         ViReal64* xStop,
                                                         ViInt32* recordLength,
                                                         ViInt32* valuesPerSample);
ViStatus _VI_FUNC rsrtm20xx_FetchEnvelopeWaveformConversionData
             (ViSession instrumentHandle, ViInt32 channel, ViReal64* xStart,
              ViReal64* xIncrement, ViReal64* yStart, ViReal64* yIncrement,
              ViInt32* yResolution);
ViStatus _VI_FUNC rsrtm20xx_FetchMathWaveform (ViSession instrumentHandle,
                                               ViInt32 mathNumber,
                                               ViInt32 waveformSize,
                                               ViReal64 _VI_FAR waveformArray[],
                                               ViInt32* actualPoints);
ViStatus _VI_FUNC rsrtm20xx_FetchMathWaveformHeader (ViSession instrumentHandle,
                                                     ViInt32 mathNumber,
                                                     ViReal64* xStart,
                                                     ViReal64* xStop,
                                                     ViInt32* recordLength,
                                                     ViInt32* valuesPerSample);
ViStatus _VI_FUNC rsrtm20xx_FetchMathWaveformConversionData
             (ViSession instrumentHandle, ViInt32 channel, ViReal64* xStart,
              ViReal64* xIncrement, ViReal64* yStart, ViReal64* yIncrement,
              ViInt32* yResolution);
ViStatus _VI_FUNC rsrtm20xx_FetchReferenceWaveform (ViSession instrumentHandle,
                                                    ViInt32 referenceWfm,
                                                    ViInt32 waveformSize,
                                                    ViReal64 _VI_FAR waveformArray[],
                                                    ViInt32* actualPoints);
ViStatus _VI_FUNC rsrtm20xx_FetchReferenceWaveformHeader (ViSession instrumentHandle,
                                                          ViInt32 referenceWfm,
                                                          ViReal64* xStart,
                                                          ViReal64* xStop,
                                                          ViInt32* recordLength,
                                                          ViInt32* valuesPerSample);
ViStatus _VI_FUNC rsrtm20xx_FetchReferenceWaveformConversionData
             (ViSession instrumentHandle, ViInt32 channel, ViReal64* xStart,
              ViReal64* xIncrement, ViReal64* yStart, ViReal64* yIncrement,
              ViInt32* yResolution);
ViStatus _VI_FUNC rsrtm20xx_FetchMaskData (ViSession instrumentHandle,
                                           ViInt32 waveformSize,
                                           ViReal64 _VI_FAR waveformArray[],
                                           ViInt32* actualPoints,
                                           ViReal64* initialX,
                                           ViReal64* xIncrement);
ViStatus _VI_FUNC rsrtm20xx_FetchMaskDataHeader (ViSession instrumentHandle,
                                                 ViReal64* xStart, ViReal64* xStop,
                                                 ViInt32* recordLength,
                                                 ViInt32* valuesPerSample);
ViStatus _VI_FUNC rsrtm20xx_FetchMaskConversionData (ViSession instrumentHandle,
                                                     ViReal64* xStart,
                                                     ViReal64* xIncrement,
                                                     ViReal64* yStart,
                                                     ViReal64* yIncrement,
                                                     ViInt32* yResolution);
ViStatus _VI_FUNC rsrtm20xx_FetchMainWaveformMeasurement (ViSession instrumentHandle,
                                                          ViInt32 measurement,
                                                          ViInt32 measurementFunction,
                                                          ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_Abort (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_ConfigurePowerAnalysisState (ViSession instrumentHandle,
                                                         ViBoolean state);
ViStatus _VI_FUNC rsrtm20xx_ConfigurePowerAnalysisType (ViSession instrumentHandle,
                                                        ViInt32 type);
ViStatus _VI_FUNC rsrtm20xx_ResetPowerAnalysisStatistics
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_StartPowerAnalysisDeskew (ViSession instrumentHandle,
                                                      ViInt32 maximumTime);
ViStatus _VI_FUNC rsrtm20xx_SetPowerAnalysisZeroOffset (ViSession instrumentHandle,
                                                        ViInt32 maximumTime);
ViStatus _VI_FUNC rsrtm20xx_PowerAnalysisAddReport (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_StartPowerQualityMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_PowerQualityMeasurementAddReport
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerQualityMeasurementVoltageResult
             (ViSession instrumentHandle, ViInt32 type, ViInt32 statistics,
              ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerQualityMeasurementCurrentResult
             (ViSession instrumentHandle, ViInt32 type, ViInt32 statistics,
              ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerQualityMeasurementPowerResult
             (ViSession instrumentHandle, ViInt32 type, ViInt32 statistics,
              ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_StartPowerConsumptionMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_RestartPowerConsumptionMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_PowerConsumptionMeasurementAddReport
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerConsumptionMeasurementResults
             (ViSession instrumentHandle, ViInt32 type,
              ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_StartPowerCurrentHarmonicsMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_PowerCurrentHarmonicsMeasurementAddReport
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerCurrentHarmonicsMeasurementResult
             (ViSession instrumentHandle, ViInt32 resultNumber, ViInt32 type,
              ViReal64* measurementResult);
ViStatus 
    _VI_FUNC rsrtm20xx_QueryPowerCurrentHarmonicsMeasurementLimitCheckResult
        (ViSession instrumentHandle, ViInt32 resultNumber, ViInt32* limitCheck);
ViStatus _VI_FUNC rsrtm20xx_ResetPowerCurrentHarmonicsMeasurementResults
             (ViSession instrumentHandle, ViInt32 resultNumber);
ViStatus _VI_FUNC rsrtm20xx_StartPowerInrushCurrentMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_PowerInrushCurrentMeasurementAddReport
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerInrushCurrentMeasurementResult
             (ViSession instrumentHandle, ViInt32 gateNumber, ViInt32 type,
              ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_StartPowerRippleMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_PowerRippleMeasurementAddReport
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerRippleMeasurementResult
             (ViSession instrumentHandle, ViInt32 type, ViInt32 statistics,
              ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_StartPowerSpectrumMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSpectrumMeasurementResult
             (ViSession instrumentHandle, ViInt32 resultNumber, ViInt32 type,
              ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_PowerSpectrumMeasurementAddReport
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_ResetPowerSpectrumMeasurementResults
             (ViSession instrumentHandle, ViInt32 resultNumber);
ViStatus _VI_FUNC rsrtm20xx_StartPowerSlewRateMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_PowerSlewRateMeasurementAddReport
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSlewRateMeasurementResult
             (ViSession instrumentHandle, ViInt32 type, ViInt32 statistics,
              ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_StartPowerModulationMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_PowerModulationMeasurementAddReport
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerModulationMeasurementResult
             (ViSession instrumentHandle, ViInt32 type, ViInt32 statistics,
              ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_StartPowerDynamicOnResistanceMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_PowerDynamicOnResistanceMeasurementAddReport
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerDynamicOnResistanceMeasurementResult
             (ViSession instrumentHandle, ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_StartPowerEfficiencyMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_PowerEfficiencyMeasurementAddReport
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerEfficiencyMeasurementResult
             (ViSession instrumentHandle, ViInt32 type, ViInt32 statistics,
              ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_StartPowerSwitchinglossMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_SetPowerSwitschinglossMeasurementGateToWaveform
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_PowerSwitchinglossMeasurementAddReport
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSwitchinglossMeasurementEnergyResult
             (ViSession instrumentHandle, ViInt32 type,
              ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSwitchinglossMeasurementPowerResult
             (ViSession instrumentHandle, ViInt32 type,
              ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_StartPowerOnOffMeasurement (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_PowerOnOffMeasurementAddReport
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerOnOffMeasurementResult
             (ViSession instrumentHandle, ViInt32 resultNumber,
              ViReal64* measurementResult);
ViStatus _VI_FUNC rsrtm20xx_StartPowerSafeOperatingAreaMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_RestartPowerSafeOperatingAreaMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_PowerSafeOperatingAreaAddToReport
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_PowerSafeOperatingAreaScaling
             (ViSession instrumentHandle, ViInt32 scale, ViInt32 maskScale,
              ViInt32 displayScale);
ViStatus _VI_FUNC rsrtm20xx_PowerSafeOperatingAreaLinPointManagement
             (ViSession instrumentHandle, ViInt32 operation, ViInt32 pointNumber);
ViStatus _VI_FUNC rsrtm20xx_PowerSafeOperatingAreaLinPointValue
             (ViSession instrumentHandle, ViInt32 pointNumber, ViInt32 valueType,
              ViReal64 pointValue);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSafeOperatingAreaLinCount
             (ViSession instrumentHandle, ViInt32* count);
ViStatus _VI_FUNC rsrtm20xx_PowerSafeOperatingAreaLogPointManagement
             (ViSession instrumentHandle, ViInt32 operation, ViInt32 pointNumber);
ViStatus _VI_FUNC rsrtm20xx_PowerSafeOperatingAreaLogPointValue
             (ViSession instrumentHandle, ViInt32 pointNumber, ViInt32 valueType,
              ViReal64 pointValue);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSafeOperatingAreaLogCount
             (ViSession instrumentHandle, ViInt32* count);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSafeOperatingAcquisitionPointResults
             (ViSession instrumentHandle, ViInt32 pointNumber, ViInt32 valueType,
              ViReal64* result);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSafeOperatingTotalPointResults
             (ViSession instrumentHandle, ViInt32 pointNumber, ViInt32 valueType,
              ViReal64* result);
ViStatus _VI_FUNC rsrtm20xx_ReadPowerSafeOperatingAreaTotalData
             (ViSession instrumentHandle, ViInt32 pointNumber, ViInt32 valueType,
              ViInt32 waveformSize, ViInt32 maximumTime,
              ViReal64 _VI_FAR waveformArray[], ViInt32* actualPoints,
              ViReal64* initialX, ViReal64* xIncrement);
ViStatus _VI_FUNC rsrtm20xx_FetchPowerSafeOperatingAreaTotalData
             (ViSession instrumentHandle, ViInt32 pointNumber, ViInt32 valueType,
              ViInt32 waveformSize, ViReal64 _VI_FAR waveformArray[],
              ViInt32* actualPoints, ViReal64* initialX, ViReal64* xIncrement);
ViStatus _VI_FUNC rsrtm20xx_FetchPowerSafeOperatingAreaTotalDataHeader
             (ViSession instrumentHandle, ViInt32 pointNumber, ViInt32 valueType,
              ViReal64* xStart, ViReal64* xStop, ViInt32* recordLength,
              ViInt32* valuesPerSample);
ViStatus _VI_FUNC rsrtm20xx_FetchPowerSafeOperatingAreaTotalConversionData
             (ViSession instrumentHandle, ViInt32 pointNumber, ViInt32 valueType,
              ViReal64* xStart, ViReal64* xIncrement, ViReal64* yStart,
              ViReal64* yIncrement, ViInt32* yResolution);
ViStatus _VI_FUNC rsrtm20xx_StartPowerTransientResponseMeasurement
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_TransientResponseSettings (ViSession instrumentHandle,
                                                       ViReal64 signalHigh,
                                                       ViReal64 signalLow,
                                                       ViReal64 startTime,
                                                       ViReal64 stopTime);
ViStatus _VI_FUNC rsrtm20xx_TransientResponseAddToReport
             (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryPowerTransientResponseResult
             (ViSession instrumentHandle, ViInt32 type, ViReal64* result);
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisState
             (ViSession instrumentHandle, ViBoolean state);
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisSource
             (ViSession instrumentHandle, ViInt32 source);
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisFrequency
             (ViSession instrumentHandle, ViReal64 start, ViReal64 stop,
              ViReal64 center, ViReal64 span);
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisWaveform
             (ViSession instrumentHandle, ViBoolean spectrum, ViBoolean minHold,
              ViBoolean maxHold, ViBoolean average);
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisTimeDomain
             (ViSession instrumentHandle, ViReal64 position, ViReal64 range);
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisDisplay
             (ViSession instrumentHandle, ViInt32 colorScheme,
              ViBoolean magnitudeMode, ViReal64 minimum, ViReal64 maximum);
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisMarkerState
             (ViSession instrumentHandle, ViBoolean state);
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisMarkerSource
             (ViSession instrumentHandle, ViInt32 source);
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisMarkerReference
             (ViSession instrumentHandle, ViInt32 reference);
ViStatus _VI_FUNC rsrtm20xx_ConfigureDVMState (ViSession instrumentHandle,
                                               ViInt32 measurementPlace,
                                               ViBoolean state);
ViStatus _VI_FUNC rsrtm20xx_ConfigureDVM (ViSession instrumentHandle,
                                          ViInt32 measurementPlace, ViInt32 source,
                                          ViInt32 type, ViInt32 position);
ViStatus _VI_FUNC rsrtm20xx_QueryDVMResult (ViSession instrumentHandle,
                                            ViInt32 measurementPlace,
                                            ViReal64* result);
ViStatus _VI_FUNC rsrtm20xx_ConfigureCounterState (ViSession instrumentHandle,
                                                   ViInt32 counter,
                                                   ViBoolean state);
ViStatus _VI_FUNC rsrtm20xx_reset (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_ConfigureErrorChecking (ViSession instrumentHandle,
                                                    ViBoolean optionChecking,
                                                    ViBoolean rangeChecking,
                                                    ViBoolean statusChecking);
ViStatus _VI_FUNC rsrtm20xx_self_test (ViSession instrumentHandle,
                                       ViInt16* selfTestResult,
                                       ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC rsrtm20xx_revision_query (ViSession instrumentHandle,
                                            ViChar _VI_FAR instrumentDriverRevision[],
                                            ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rsrtm20xx_error_query (ViSession instrumentHandle,
                                         ViInt32* errorCode,
                                         ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsrtm20xx_ErrorList (ViSession instrumentHandle,
                                       ViInt32 bufferSize, ViChar _VI_FAR errors[]);
ViStatus _VI_FUNC rsrtm20xx_error_message (ViSession instrumentHandle,
                                           ViStatus errorCode,
                                           ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rsrtm20xx_ConfigureInstrumentName (ViSession instrumentHandle,
                                                     ViString name);
ViStatus _VI_FUNC rsrtm20xx_ResetStatusRegisters (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_SetStatusRegister (ViSession instrumentHandle,
                                               ViInt32 registerType,
                                               ViInt32 statusRegister,
                                               ViInt32 registerPart,
                                               ViInt32 registerValue);
ViStatus _VI_FUNC rsrtm20xx_GetStatusRegister (ViSession instrumentHandle,
                                               ViInt32 registerType,
                                               ViInt32 statusRegister,
                                               ViInt32 registerPart,
                                               ViInt32* registerValue);
ViStatus _VI_FUNC rsrtm20xx_SetVISATimeout (ViSession instrumentHandle,
                                            ViUInt32 VISATimeout);
ViStatus _VI_FUNC rsrtm20xx_GetVISATimeout (ViSession instrumentHandle,
                                            ViUInt32* VISATimeout);
ViStatus _VI_FUNC rsrtm20xx_SelfAlingment (ViSession instrumentHandle,
                                           ViInt32 maximumTime);
ViStatus _VI_FUNC rsrtm20xx_ClearStatus (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_IDQueryResponse (ViSession instrumentHandle,
                                             ViChar _VI_FAR IDQueryResponse[]);
ViStatus _VI_FUNC rsrtm20xx_ProcessAllPreviousCommands (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_QueryOPC (ViSession instrumentHandle, ViInt32* OPC);
ViStatus _VI_FUNC rsrtm20xx_GetError (ViSession instrumentHandle, ViStatus* code,
                                      ViInt32 bufferSize,
                                      ViChar _VI_FAR description[]);
ViStatus _VI_FUNC rsrtm20xx_ClearError (ViSession instrumentHandle);
ViStatus _VI_FUNC rsrtm20xx_WriteInstrData (ViSession instrumentHandle,
                                            ViString writeBuffer);
ViStatus _VI_FUNC rsrtm20xx_ReadInstrData (ViSession instrumentHandle,
                                           ViInt32 numberOfBytesToRead,
                                           ViChar _VI_FAR readBuffer[],
                                           ViUInt32* numBytesRead);
ViStatus _VI_FUNC rsrtm20xx_QueryViBoolean (ViSession instrumentHandle,
                                            ViString command, ViBoolean* value);
ViStatus _VI_FUNC rsrtm20xx_QueryViInt32 (ViSession instrumentHandle,
                                          ViString command, ViInt32* value);
ViStatus _VI_FUNC rsrtm20xx_QueryViReal64 (ViSession instrumentHandle,
                                           ViString command, ViReal64* value);
ViStatus _VI_FUNC rsrtm20xx_QueryViString (ViSession instrumentHandle,
                                           ViString command, ViInt32 bufferSize,
                                           ViChar _VI_FAR value[]);
ViStatus _VI_FUNC rsrtm20xx_WriteCommandWithOPCSync (ViSession instrumentHandle,
                                                     ViString writeBuffer,
                                                     ViUInt32 timeout);
ViStatus _VI_FUNC rsrtm20xx_QueryWithOPCSync (ViSession instrumentHandle,
                                              ViString writeBuffer,
                                              ViUInt32 timeout, ViInt32 bufferSize,
                                              ViChar _VI_FAR readBuffer[],
                                              ViUInt32* numBytesRead);
ViStatus _VI_FUNC rsrtm20xx_FileManagerOperations (ViSession instrumentHandle,
                                                   ViInt32 operation,
                                                   ViString source,
                                                   ViString destination);
ViStatus _VI_FUNC rsrtm20xx_FileCount (ViSession instrumentHandle,
                                       ViString directory,
                                       ViInt32* numberOfFilesAndFolders);
ViStatus _VI_FUNC rsrtm20xx_FileDirectoryContent (ViSession instrumentHandle,
                                                  ViString directory,
                                                  ViInt32 bufferSize,
                                                  ViChar _VI_FAR output[]);
ViStatus _VI_FUNC rsrtm20xx_FileSubdirectoriesCount (ViSession instrumentHandle,
                                                     ViString directory,
                                                     ViInt32* numberOfSubdirectories);
ViStatus _VI_FUNC rsrtm20xx_FileSubdirectories (ViSession instrumentHandle,
                                                ViString directory,
                                                ViInt32 bufferSize,
                                                ViChar _VI_FAR output[]);
ViStatus _VI_FUNC rsrtm20xx_FileDrives (ViSession instrumentHandle,
                                        ViInt32 bufferSize,
                                        ViChar _VI_FAR output[]);
ViStatus _VI_FUNC rsrtm20xx_ReadToFileFromInstrument (ViSession instrumentHandle,
                                                      ViString source,
                                                      ViString destination);
ViStatus _VI_FUNC rsrtm20xx_WriteFromFileToInstrument (ViSession instrumentHandle,
                                                       ViString source,
                                                       ViString destination);
ViStatus _VI_FUNC rsrtm20xx_DataSetFileOperations (ViSession instrumentHandle,
                                                   ViInt32 operation,
                                                   ViInt32 stateNumber,
                                                   ViString path);
ViStatus _VI_FUNC rsrtm20xx_close (ViSession instrumentHandle);


/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSRTM20XX_HEADER */



